#!/usr/bin/env node
import * as le from "node:os";
import ee, { resolve as fe } from "node:path";
import { aT as de, aU as _e, C as pe, aV as ne, aW as ge, aX as ve, aI as oe, aQ as me, aH as be, aC as Se, O as se } from "./console-RPzEsLY0.js";
import { loadProfileEnvironmentVariables as Ce } from "./load-profile-CgexvkzB.js";
const ye = [
  // Basic read/info operations
  "ls",
  "dir",
  "find",
  "cat",
  "head",
  "tail",
  "less",
  "more",
  "grep",
  "egrep",
  "fgrep",
  "tree",
  "file",
  "wc",
  "pwd",
  "stat",
  "du",
  "df",
  "ps",
  "top",
  "htop",
  "echo",
  "printenv",
  "id",
  "which",
  "whereis",
  "date",
  "cal",
  "uptime",
  "free",
  "ping",
  "dig",
  "nslookup",
  "host",
  "netstat",
  "ss",
  "lsof",
  "ifconfig",
  "ip",
  "man",
  "info",
  "mkdir",
  "touch",
  "uname",
  "whoami",
  // Go
  "go version",
  "go env",
  "go help",
  // Cargo
  "cargo version",
  "cargo --version",
  "cargo help",
  // Rust
  "rustc --version",
  "rustc --help",
  "rustc --explain",
  // Java
  "javac --version",
  "javac -version",
  "javac -help",
  "javac --help",
  // .NET
  "dotnet --info",
  "dotnet --version",
  "dotnet --help",
  "dotnet help",
  // C/C++
  "gcc --version",
  "gcc -v",
  "gcc --help",
  "gcc -dumpversion",
  "g++ --version",
  "g++ -v",
  "g++ --help",
  "g++ -dumpversion",
  "clang --version",
  "clang --help",
  "clang++ --version",
  "clang++ --help",
  // Python
  "python -V",
  "python --version",
  "python -h",
  "python --help",
  "python3 -V",
  "python3 --version",
  "python3 -h",
  "python3 --help",
  // Ruby
  "ruby -v",
  "ruby --version",
  "ruby -h",
  "ruby --help",
  // Node
  "node -v",
  "node --version",
  "node -h",
  "node --help",
  // npm
  "npm --help",
  "npm --version",
  "npm -v",
  "npm help",
  // yarn
  "yarn --help",
  "yarn --version",
  "yarn -v",
  "yarn help",
  // pnpm
  "pnpm --help",
  "pnpm --version",
  "pnpm -v",
  "pnpm help",
  // Testing
  "pytest -h",
  "pytest --help",
  "pytest --version",
  "jest --help",
  "jest --version",
  "mocha --help",
  "mocha --version",
  // Make
  "make --version",
  "make --help",
  // Docker
  "docker --version",
  "docker --help",
  "docker version",
  "docker help",
  // Git
  "git --version",
  "git --help",
  "git help",
  "git version"
], ae = [
  // Go commands
  "go test",
  "go run",
  "go build",
  "go vet",
  "go fmt",
  "go list",
  // Cargo commands
  "cargo test",
  "cargo run",
  "cargo build",
  "cargo check",
  "cargo fmt",
  "cargo tree",
  // Make (only dry runs)
  "make -n",
  "make --dry-run",
  // Maven
  "mvn test",
  "mvn verify",
  "mvn dependency:tree",
  // Gradle
  "gradle tasks",
  "gradle dependencies",
  "gradle properties",
  // .NET
  "dotnet test",
  "dotnet list",
  // Python
  "python -c",
  // Ruby
  "ruby -e",
  // Node
  "node -e",
  // npm
  "npm list",
  "npm ls",
  "npm outdated",
  "npm test",
  "npm run",
  "npm view",
  "npm info",
  // yarn
  "yarn list",
  "yarn ls",
  "yarn info",
  "yarn test",
  "yarn run",
  "yarn why",
  // pnpm
  "pnpm list",
  "pnpm ls",
  "pnpm outdated",
  "pnpm test",
  "pnpm run",
  // Testing
  "pytest --collect-only",
  "jest --listTests",
  "jest --showConfig",
  "mocha --list",
  // Git
  "git status",
  "git show",
  "git diff",
  "git grep",
  "git branch",
  "git tag",
  "git remote -v",
  "git rev-parse --is-inside-work-tree",
  "git rev-parse --show-toplevel",
  "git config --list",
  "git log"
], we = ["/dev/null", "/dev/zero", "/dev/stdout", "/dev/stderr", "/dev/stdin"];
function ue(L, O) {
  if (!L || we.includes(L) || (L.endsWith(ee.sep) && (L = L.slice(0, -1)), O.includes(L)))
    return !1;
  if (ee.isAbsolute(L))
    return !O.some((R) => Be(R, L));
  const I = L.split("/");
  let k = 0;
  for (const R of I)
    if (R === "..") {
      if (k--, k < 0)
        return !0;
    } else R !== "" && R !== "." && k++;
  return !1;
}
function Be(L, O) {
  const I = ee.relative(ee.normalize(L), ee.normalize(O));
  return !!I && !I.startsWith("..") && !I.startsWith("/");
}
function re(L, O, I) {
  if (typeof L == "string")
    return !1;
  for (const k of L.arguments)
    if (typeof k == "string" && !(k.startsWith("-") || k.includes("*") || k.startsWith('"') || k.startsWith("'")) && !(I && !I.absolutePaths && k.startsWith("//")) && k.includes("/") && ue(k, O))
      return !0;
  return !1;
}
function Ee(L, O = !1) {
  const I = {
    absolutePaths: !0,
    environmentVariables: !0
  };
  if (O || typeof L == "string")
    return I;
  const k = typeof L.program == "string" ? L.program.toLowerCase() : "", R = L.arguments.filter((N) => typeof N == "string").join(" "), q = `${k} ${R}`.toLowerCase();
  return k.includes("bazel") || q.includes("bazel") || k === "ibazel" ? {
    ...I,
    absolutePaths: !1
    // Allow bazel-style absolute paths like //path/to/target
  } : I;
}
function Ae(L) {
  for (const O of ye)
    if (L === O || L.startsWith(O + " "))
      return !0;
  return !1;
}
function Te(L) {
  const O = /* @__PURE__ */ new Set(), I = L.map(
    (k) => k.endsWith(" *") ? k.slice(0, -2) : k
  );
  for (const k of I)
    for (const R of ae)
      if (!O.has(R) && k !== R && k.startsWith(R + " ")) {
        O.add(R);
        for (const q of ae)
          q.startsWith(R + " ") && O.add(q);
      }
  return O;
}
function ke(L, O, I) {
  let k = [];
  try {
    const t = de(L, I?.platform);
    k = _e(t);
  } catch (t) {
    pe.warn("Failed to parse command", { cmd: L }, t), k = [L];
  }
  const R = I?.config?.["commands.allowlist"] || [], q = I?.config?.["commands.strict"] ?? !1;
  if (R.includes("*"))
    return {
      requiresConfirmation: !1,
      parts: k.map((t) => ({ command: t, requiresConfirmation: !1 }))
    };
  const N = k.map((t) => {
    if (typeof t == "string")
      return { command: t, requiresConfirmation: !0 };
    const l = Ee(t, q), d = t.arguments.some((a) => typeof a != "string");
    function _(a) {
      return a === ne ? "<dynamic>" : a;
    }
    const n = `${_(t.program)} ${t.arguments.map(_).join(" ")}`;
    if (!d) {
      for (const a of R)
        if (n === a)
          return { command: t, requiresConfirmation: !1 };
    }
    if (Ae(n))
      return re(t, O, l) ? { command: t, requiresConfirmation: !0 } : { command: t, requiresConfirmation: !1 };
    let c;
    if (q)
      c = R;
    else {
      const a = Te(R);
      c = [...ae.filter(
        (u) => !a.has(u)
      ), ...R];
    }
    for (let a of c) {
      a.endsWith(" *") && (a = a.slice(0, -2));
      const h = " * ";
      if (a.includes(h) && !a.startsWith(h)) {
        const u = a.split(h).map((r) => r.replace(/[.*+?^${}()|[\]\\]/g, "\\$&"));
        if (new RegExp(`^${u.join(" .* ")}(\\s.*)?$`).test(n))
          return re(t, O, l) ? { command: t, requiresConfirmation: !0 } : { command: t, requiresConfirmation: !1 };
      } else if (n === a || n.startsWith(a + " "))
        return re(t, O, l) ? { command: t, requiresConfirmation: !0 } : { command: t, requiresConfirmation: !1 };
    }
    return { command: t, requiresConfirmation: !0 };
  });
  return { requiresConfirmation: N.some((t) => t.requiresConfirmation), parts: N };
}
var ie = {}, he;
function Le() {
  return he || (he = 1, function(L) {
    (() => {
      var O = { 349: (w, t, l) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.CircularList = void 0;
        const d = l(460), _ = l(844);
        class n extends _.Disposable {
          constructor(a) {
            super(), this._maxLength = a, this.onDeleteEmitter = this.register(new d.EventEmitter()), this.onDelete = this.onDeleteEmitter.event, this.onInsertEmitter = this.register(new d.EventEmitter()), this.onInsert = this.onInsertEmitter.event, this.onTrimEmitter = this.register(new d.EventEmitter()), this.onTrim = this.onTrimEmitter.event, this._array = new Array(this._maxLength), this._startIndex = 0, this._length = 0;
          }
          get maxLength() {
            return this._maxLength;
          }
          set maxLength(a) {
            if (this._maxLength === a) return;
            const h = new Array(a);
            for (let u = 0; u < Math.min(a, this.length); u++) h[u] = this._array[this._getCyclicIndex(u)];
            this._array = h, this._maxLength = a, this._startIndex = 0;
          }
          get length() {
            return this._length;
          }
          set length(a) {
            if (a > this._length) for (let h = this._length; h < a; h++) this._array[h] = void 0;
            this._length = a;
          }
          get(a) {
            return this._array[this._getCyclicIndex(a)];
          }
          set(a, h) {
            this._array[this._getCyclicIndex(a)] = h;
          }
          push(a) {
            this._array[this._getCyclicIndex(this._length)] = a, this._length === this._maxLength ? (this._startIndex = ++this._startIndex % this._maxLength, this.onTrimEmitter.fire(1)) : this._length++;
          }
          recycle() {
            if (this._length !== this._maxLength) throw new Error("Can only recycle when the buffer is full");
            return this._startIndex = ++this._startIndex % this._maxLength, this.onTrimEmitter.fire(1), this._array[this._getCyclicIndex(this._length - 1)];
          }
          get isFull() {
            return this._length === this._maxLength;
          }
          pop() {
            return this._array[this._getCyclicIndex(this._length-- - 1)];
          }
          splice(a, h, ...u) {
            if (h) {
              for (let e = a; e < this._length - h; e++) this._array[this._getCyclicIndex(e)] = this._array[this._getCyclicIndex(e + h)];
              this._length -= h, this.onDeleteEmitter.fire({ index: a, amount: h });
            }
            for (let e = this._length - 1; e >= a; e--) this._array[this._getCyclicIndex(e + u.length)] = this._array[this._getCyclicIndex(e)];
            for (let e = 0; e < u.length; e++) this._array[this._getCyclicIndex(a + e)] = u[e];
            if (u.length && this.onInsertEmitter.fire({ index: a, amount: u.length }), this._length + u.length > this._maxLength) {
              const e = this._length + u.length - this._maxLength;
              this._startIndex += e, this._length = this._maxLength, this.onTrimEmitter.fire(e);
            } else this._length += u.length;
          }
          trimStart(a) {
            a > this._length && (a = this._length), this._startIndex += a, this._length -= a, this.onTrimEmitter.fire(a);
          }
          shiftElements(a, h, u) {
            if (!(h <= 0)) {
              if (a < 0 || a >= this._length) throw new Error("start argument out of range");
              if (a + u < 0) throw new Error("Cannot shift elements in list beyond index 0");
              if (u > 0) {
                for (let r = h - 1; r >= 0; r--) this.set(a + r + u, this.get(a + r));
                const e = a + h + u - this._length;
                if (e > 0) for (this._length += e; this._length > this._maxLength; ) this._length--, this._startIndex++, this.onTrimEmitter.fire(1);
              } else for (let e = 0; e < h; e++) this.set(a + e + u, this.get(a + e));
            }
          }
          _getCyclicIndex(a) {
            return (this._startIndex + a) % this._maxLength;
          }
        }
        t.CircularList = n;
      }, 439: (w, t) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.clone = void 0, t.clone = function l(d, _ = 5) {
          if (typeof d != "object") return d;
          const n = Array.isArray(d) ? [] : {};
          for (const c in d) n[c] = _ <= 1 ? d[c] : d[c] && l(d[c], _ - 1);
          return n;
        };
      }, 969: (w, t, l) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.CoreTerminal = void 0;
        const d = l(844), _ = l(585), n = l(348), c = l(866), a = l(744), h = l(302), u = l(83), e = l(460), r = l(753), i = l(480), s = l(994), f = l(282), p = l(435), S = l(981), y = l(660);
        let C = !1;
        class T extends d.Disposable {
          get onScroll() {
            return this._onScrollApi || (this._onScrollApi = this.register(new e.EventEmitter()), this._onScroll.event((E) => {
              this._onScrollApi?.fire(E.position);
            })), this._onScrollApi.event;
          }
          get cols() {
            return this._bufferService.cols;
          }
          get rows() {
            return this._bufferService.rows;
          }
          get buffers() {
            return this._bufferService.buffers;
          }
          get options() {
            return this.optionsService.options;
          }
          set options(E) {
            for (const B in E) this.optionsService.options[B] = E[B];
          }
          constructor(E) {
            super(), this._windowsWrappingHeuristics = this.register(new d.MutableDisposable()), this._onBinary = this.register(new e.EventEmitter()), this.onBinary = this._onBinary.event, this._onData = this.register(new e.EventEmitter()), this.onData = this._onData.event, this._onLineFeed = this.register(new e.EventEmitter()), this.onLineFeed = this._onLineFeed.event, this._onResize = this.register(new e.EventEmitter()), this.onResize = this._onResize.event, this._onWriteParsed = this.register(new e.EventEmitter()), this.onWriteParsed = this._onWriteParsed.event, this._onScroll = this.register(new e.EventEmitter()), this._instantiationService = new n.InstantiationService(), this.optionsService = this.register(new h.OptionsService(E)), this._instantiationService.setService(_.IOptionsService, this.optionsService), this._bufferService = this.register(this._instantiationService.createInstance(a.BufferService)), this._instantiationService.setService(_.IBufferService, this._bufferService), this._logService = this.register(this._instantiationService.createInstance(c.LogService)), this._instantiationService.setService(_.ILogService, this._logService), this.coreService = this.register(this._instantiationService.createInstance(u.CoreService)), this._instantiationService.setService(_.ICoreService, this.coreService), this.coreMouseService = this.register(this._instantiationService.createInstance(r.CoreMouseService)), this._instantiationService.setService(_.ICoreMouseService, this.coreMouseService), this.unicodeService = this.register(this._instantiationService.createInstance(i.UnicodeService)), this._instantiationService.setService(_.IUnicodeService, this.unicodeService), this._charsetService = this._instantiationService.createInstance(s.CharsetService), this._instantiationService.setService(_.ICharsetService, this._charsetService), this._oscLinkService = this._instantiationService.createInstance(y.OscLinkService), this._instantiationService.setService(_.IOscLinkService, this._oscLinkService), this._inputHandler = this.register(new p.InputHandler(this._bufferService, this._charsetService, this.coreService, this._logService, this.optionsService, this._oscLinkService, this.coreMouseService, this.unicodeService)), this.register((0, e.forwardEvent)(this._inputHandler.onLineFeed, this._onLineFeed)), this.register(this._inputHandler), this.register((0, e.forwardEvent)(this._bufferService.onResize, this._onResize)), this.register((0, e.forwardEvent)(this.coreService.onData, this._onData)), this.register((0, e.forwardEvent)(this.coreService.onBinary, this._onBinary)), this.register(this.coreService.onRequestScrollToBottom(() => this.scrollToBottom())), this.register(this.coreService.onUserInput(() => this._writeBuffer.handleUserInput())), this.register(this.optionsService.onMultipleOptionChange(["windowsMode", "windowsPty"], () => this._handleWindowsPtyOptionChange())), this.register(this._bufferService.onScroll((B) => {
              this._onScroll.fire({ position: this._bufferService.buffer.ydisp, source: 0 }), this._inputHandler.markRangeDirty(this._bufferService.buffer.scrollTop, this._bufferService.buffer.scrollBottom);
            })), this.register(this._inputHandler.onScroll((B) => {
              this._onScroll.fire({ position: this._bufferService.buffer.ydisp, source: 0 }), this._inputHandler.markRangeDirty(this._bufferService.buffer.scrollTop, this._bufferService.buffer.scrollBottom);
            })), this._writeBuffer = this.register(new S.WriteBuffer((B, D) => this._inputHandler.parse(B, D))), this.register((0, e.forwardEvent)(this._writeBuffer.onWriteParsed, this._onWriteParsed));
          }
          write(E, B) {
            this._writeBuffer.write(E, B);
          }
          writeSync(E, B) {
            this._logService.logLevel <= _.LogLevelEnum.WARN && !C && (this._logService.warn("writeSync is unreliable and will be removed soon."), C = !0), this._writeBuffer.writeSync(E, B);
          }
          input(E, B = !0) {
            this.coreService.triggerDataEvent(E, B);
          }
          resize(E, B) {
            isNaN(E) || isNaN(B) || (E = Math.max(E, a.MINIMUM_COLS), B = Math.max(B, a.MINIMUM_ROWS), this._bufferService.resize(E, B));
          }
          scroll(E, B = !1) {
            this._bufferService.scroll(E, B);
          }
          scrollLines(E, B, D) {
            this._bufferService.scrollLines(E, B, D);
          }
          scrollPages(E) {
            this.scrollLines(E * (this.rows - 1));
          }
          scrollToTop() {
            this.scrollLines(-this._bufferService.buffer.ydisp);
          }
          scrollToBottom() {
            this.scrollLines(this._bufferService.buffer.ybase - this._bufferService.buffer.ydisp);
          }
          scrollToLine(E) {
            const B = E - this._bufferService.buffer.ydisp;
            B !== 0 && this.scrollLines(B);
          }
          registerEscHandler(E, B) {
            return this._inputHandler.registerEscHandler(E, B);
          }
          registerDcsHandler(E, B) {
            return this._inputHandler.registerDcsHandler(E, B);
          }
          registerCsiHandler(E, B) {
            return this._inputHandler.registerCsiHandler(E, B);
          }
          registerOscHandler(E, B) {
            return this._inputHandler.registerOscHandler(E, B);
          }
          _setup() {
            this._handleWindowsPtyOptionChange();
          }
          reset() {
            this._inputHandler.reset(), this._bufferService.reset(), this._charsetService.reset(), this.coreService.reset(), this.coreMouseService.reset();
          }
          _handleWindowsPtyOptionChange() {
            let E = !1;
            const B = this.optionsService.rawOptions.windowsPty;
            B && B.buildNumber !== void 0 && B.buildNumber !== void 0 ? E = B.backend === "conpty" && B.buildNumber < 21376 : this.optionsService.rawOptions.windowsMode && (E = !0), E ? this._enableWindowsWrappingHeuristics() : this._windowsWrappingHeuristics.clear();
          }
          _enableWindowsWrappingHeuristics() {
            if (!this._windowsWrappingHeuristics.value) {
              const E = [];
              E.push(this.onLineFeed(f.updateWindowsModeWrappedState.bind(null, this._bufferService))), E.push(this.registerCsiHandler({ final: "H" }, () => ((0, f.updateWindowsModeWrappedState)(this._bufferService), !1))), this._windowsWrappingHeuristics.value = (0, d.toDisposable)(() => {
                for (const B of E) B.dispose();
              });
            }
          }
        }
        t.CoreTerminal = T;
      }, 460: (w, t) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.runAndSubscribe = t.forwardEvent = t.EventEmitter = void 0, t.EventEmitter = class {
          constructor() {
            this._listeners = [], this._disposed = !1;
          }
          get event() {
            return this._event || (this._event = (l) => (this._listeners.push(l), { dispose: () => {
              if (!this._disposed) {
                for (let _ = 0; _ < this._listeners.length; _++) if (this._listeners[_] === l) return void this._listeners.splice(_, 1);
              }
            } })), this._event;
          }
          fire(l, d) {
            const _ = [];
            for (let n = 0; n < this._listeners.length; n++) _.push(this._listeners[n]);
            for (let n = 0; n < _.length; n++) _[n].call(void 0, l, d);
          }
          dispose() {
            this.clearListeners(), this._disposed = !0;
          }
          clearListeners() {
            this._listeners && (this._listeners.length = 0);
          }
        }, t.forwardEvent = function(l, d) {
          return l((_) => d.fire(_));
        }, t.runAndSubscribe = function(l, d) {
          return d(void 0), l((_) => d(_));
        };
      }, 435: function(w, t, l) {
        var d = this && this.__decorate || function(H, o, m, g) {
          var b, A = arguments.length, P = A < 3 ? o : g === null ? g = Object.getOwnPropertyDescriptor(o, m) : g;
          if (typeof Reflect == "object" && typeof Reflect.decorate == "function") P = Reflect.decorate(H, o, m, g);
          else for (var W = H.length - 1; W >= 0; W--) (b = H[W]) && (P = (A < 3 ? b(P) : A > 3 ? b(o, m, P) : b(o, m)) || P);
          return A > 3 && P && Object.defineProperty(o, m, P), P;
        }, _ = this && this.__param || function(H, o) {
          return function(m, g) {
            o(m, g, H);
          };
        };
        Object.defineProperty(t, "__esModule", { value: !0 }), t.InputHandler = t.WindowsOptionsReportType = void 0;
        const n = l(584), c = l(116), a = l(15), h = l(844), u = l(482), e = l(437), r = l(460), i = l(643), s = l(511), f = l(734), p = l(585), S = l(480), y = l(242), C = l(351), T = l(941), $ = { "(": 0, ")": 1, "*": 2, "+": 3, "-": 1, ".": 2 }, E = 131072;
        function B(H, o) {
          if (H > 24) return o.setWinLines || !1;
          switch (H) {
            case 1:
              return !!o.restoreWin;
            case 2:
              return !!o.minimizeWin;
            case 3:
              return !!o.setWinPosition;
            case 4:
              return !!o.setWinSizePixels;
            case 5:
              return !!o.raiseWin;
            case 6:
              return !!o.lowerWin;
            case 7:
              return !!o.refreshWin;
            case 8:
              return !!o.setWinSizeChars;
            case 9:
              return !!o.maximizeWin;
            case 10:
              return !!o.fullscreenWin;
            case 11:
              return !!o.getWinState;
            case 13:
              return !!o.getWinPosition;
            case 14:
              return !!o.getWinSizePixels;
            case 15:
              return !!o.getScreenSizePixels;
            case 16:
              return !!o.getCellSizePixels;
            case 18:
              return !!o.getWinSizeChars;
            case 19:
              return !!o.getScreenSizeChars;
            case 20:
              return !!o.getIconTitle;
            case 21:
              return !!o.getWinTitle;
            case 22:
              return !!o.pushTitle;
            case 23:
              return !!o.popTitle;
            case 24:
              return !!o.setWinLines;
          }
          return !1;
        }
        var D;
        (function(H) {
          H[H.GET_WIN_SIZE_PIXELS = 0] = "GET_WIN_SIZE_PIXELS", H[H.GET_CELL_SIZE_PIXELS = 1] = "GET_CELL_SIZE_PIXELS";
        })(D || (t.WindowsOptionsReportType = D = {}));
        let G = 0;
        class V extends h.Disposable {
          getAttrData() {
            return this._curAttrData;
          }
          constructor(o, m, g, b, A, P, W, j, z = new a.EscapeSequenceParser()) {
            super(), this._bufferService = o, this._charsetService = m, this._coreService = g, this._logService = b, this._optionsService = A, this._oscLinkService = P, this._coreMouseService = W, this._unicodeService = j, this._parser = z, this._parseBuffer = new Uint32Array(4096), this._stringDecoder = new u.StringToUtf32(), this._utf8Decoder = new u.Utf8ToUtf32(), this._workCell = new s.CellData(), this._windowTitle = "", this._iconName = "", this._windowTitleStack = [], this._iconNameStack = [], this._curAttrData = e.DEFAULT_ATTR_DATA.clone(), this._eraseAttrDataInternal = e.DEFAULT_ATTR_DATA.clone(), this._onRequestBell = this.register(new r.EventEmitter()), this.onRequestBell = this._onRequestBell.event, this._onRequestRefreshRows = this.register(new r.EventEmitter()), this.onRequestRefreshRows = this._onRequestRefreshRows.event, this._onRequestReset = this.register(new r.EventEmitter()), this.onRequestReset = this._onRequestReset.event, this._onRequestSendFocus = this.register(new r.EventEmitter()), this.onRequestSendFocus = this._onRequestSendFocus.event, this._onRequestSyncScrollBar = this.register(new r.EventEmitter()), this.onRequestSyncScrollBar = this._onRequestSyncScrollBar.event, this._onRequestWindowsOptionsReport = this.register(new r.EventEmitter()), this.onRequestWindowsOptionsReport = this._onRequestWindowsOptionsReport.event, this._onA11yChar = this.register(new r.EventEmitter()), this.onA11yChar = this._onA11yChar.event, this._onA11yTab = this.register(new r.EventEmitter()), this.onA11yTab = this._onA11yTab.event, this._onCursorMove = this.register(new r.EventEmitter()), this.onCursorMove = this._onCursorMove.event, this._onLineFeed = this.register(new r.EventEmitter()), this.onLineFeed = this._onLineFeed.event, this._onScroll = this.register(new r.EventEmitter()), this.onScroll = this._onScroll.event, this._onTitleChange = this.register(new r.EventEmitter()), this.onTitleChange = this._onTitleChange.event, this._onColor = this.register(new r.EventEmitter()), this.onColor = this._onColor.event, this._parseStack = { paused: !1, cursorStartX: 0, cursorStartY: 0, decodedLength: 0, position: 0 }, this._specialColors = [256, 257, 258], this.register(this._parser), this._dirtyRowTracker = new U(this._bufferService), this._activeBuffer = this._bufferService.buffer, this.register(this._bufferService.buffers.onBufferActivate((v) => this._activeBuffer = v.activeBuffer)), this._parser.setCsiHandlerFallback((v, F) => {
              this._logService.debug("Unknown CSI code: ", { identifier: this._parser.identToString(v), params: F.toArray() });
            }), this._parser.setEscHandlerFallback((v) => {
              this._logService.debug("Unknown ESC code: ", { identifier: this._parser.identToString(v) });
            }), this._parser.setExecuteHandlerFallback((v) => {
              this._logService.debug("Unknown EXECUTE code: ", { code: v });
            }), this._parser.setOscHandlerFallback((v, F, x) => {
              this._logService.debug("Unknown OSC code: ", { identifier: v, action: F, data: x });
            }), this._parser.setDcsHandlerFallback((v, F, x) => {
              F === "HOOK" && (x = x.toArray()), this._logService.debug("Unknown DCS code: ", { identifier: this._parser.identToString(v), action: F, payload: x });
            }), this._parser.setPrintHandler((v, F, x) => this.print(v, F, x)), this._parser.registerCsiHandler({ final: "@" }, (v) => this.insertChars(v)), this._parser.registerCsiHandler({ intermediates: " ", final: "@" }, (v) => this.scrollLeft(v)), this._parser.registerCsiHandler({ final: "A" }, (v) => this.cursorUp(v)), this._parser.registerCsiHandler({ intermediates: " ", final: "A" }, (v) => this.scrollRight(v)), this._parser.registerCsiHandler({ final: "B" }, (v) => this.cursorDown(v)), this._parser.registerCsiHandler({ final: "C" }, (v) => this.cursorForward(v)), this._parser.registerCsiHandler({ final: "D" }, (v) => this.cursorBackward(v)), this._parser.registerCsiHandler({ final: "E" }, (v) => this.cursorNextLine(v)), this._parser.registerCsiHandler({ final: "F" }, (v) => this.cursorPrecedingLine(v)), this._parser.registerCsiHandler({ final: "G" }, (v) => this.cursorCharAbsolute(v)), this._parser.registerCsiHandler({ final: "H" }, (v) => this.cursorPosition(v)), this._parser.registerCsiHandler({ final: "I" }, (v) => this.cursorForwardTab(v)), this._parser.registerCsiHandler({ final: "J" }, (v) => this.eraseInDisplay(v, !1)), this._parser.registerCsiHandler({ prefix: "?", final: "J" }, (v) => this.eraseInDisplay(v, !0)), this._parser.registerCsiHandler({ final: "K" }, (v) => this.eraseInLine(v, !1)), this._parser.registerCsiHandler({ prefix: "?", final: "K" }, (v) => this.eraseInLine(v, !0)), this._parser.registerCsiHandler({ final: "L" }, (v) => this.insertLines(v)), this._parser.registerCsiHandler({ final: "M" }, (v) => this.deleteLines(v)), this._parser.registerCsiHandler({ final: "P" }, (v) => this.deleteChars(v)), this._parser.registerCsiHandler({ final: "S" }, (v) => this.scrollUp(v)), this._parser.registerCsiHandler({ final: "T" }, (v) => this.scrollDown(v)), this._parser.registerCsiHandler({ final: "X" }, (v) => this.eraseChars(v)), this._parser.registerCsiHandler({ final: "Z" }, (v) => this.cursorBackwardTab(v)), this._parser.registerCsiHandler({ final: "`" }, (v) => this.charPosAbsolute(v)), this._parser.registerCsiHandler({ final: "a" }, (v) => this.hPositionRelative(v)), this._parser.registerCsiHandler({ final: "b" }, (v) => this.repeatPrecedingCharacter(v)), this._parser.registerCsiHandler({ final: "c" }, (v) => this.sendDeviceAttributesPrimary(v)), this._parser.registerCsiHandler({ prefix: ">", final: "c" }, (v) => this.sendDeviceAttributesSecondary(v)), this._parser.registerCsiHandler({ final: "d" }, (v) => this.linePosAbsolute(v)), this._parser.registerCsiHandler({ final: "e" }, (v) => this.vPositionRelative(v)), this._parser.registerCsiHandler({ final: "f" }, (v) => this.hVPosition(v)), this._parser.registerCsiHandler({ final: "g" }, (v) => this.tabClear(v)), this._parser.registerCsiHandler({ final: "h" }, (v) => this.setMode(v)), this._parser.registerCsiHandler({ prefix: "?", final: "h" }, (v) => this.setModePrivate(v)), this._parser.registerCsiHandler({ final: "l" }, (v) => this.resetMode(v)), this._parser.registerCsiHandler({ prefix: "?", final: "l" }, (v) => this.resetModePrivate(v)), this._parser.registerCsiHandler({ final: "m" }, (v) => this.charAttributes(v)), this._parser.registerCsiHandler({ final: "n" }, (v) => this.deviceStatus(v)), this._parser.registerCsiHandler({ prefix: "?", final: "n" }, (v) => this.deviceStatusPrivate(v)), this._parser.registerCsiHandler({ intermediates: "!", final: "p" }, (v) => this.softReset(v)), this._parser.registerCsiHandler({ intermediates: " ", final: "q" }, (v) => this.setCursorStyle(v)), this._parser.registerCsiHandler({ final: "r" }, (v) => this.setScrollRegion(v)), this._parser.registerCsiHandler({ final: "s" }, (v) => this.saveCursor(v)), this._parser.registerCsiHandler({ final: "t" }, (v) => this.windowOptions(v)), this._parser.registerCsiHandler({ final: "u" }, (v) => this.restoreCursor(v)), this._parser.registerCsiHandler({ intermediates: "'", final: "}" }, (v) => this.insertColumns(v)), this._parser.registerCsiHandler({ intermediates: "'", final: "~" }, (v) => this.deleteColumns(v)), this._parser.registerCsiHandler({ intermediates: '"', final: "q" }, (v) => this.selectProtected(v)), this._parser.registerCsiHandler({ intermediates: "$", final: "p" }, (v) => this.requestMode(v, !0)), this._parser.registerCsiHandler({ prefix: "?", intermediates: "$", final: "p" }, (v) => this.requestMode(v, !1)), this._parser.setExecuteHandler(n.C0.BEL, () => this.bell()), this._parser.setExecuteHandler(n.C0.LF, () => this.lineFeed()), this._parser.setExecuteHandler(n.C0.VT, () => this.lineFeed()), this._parser.setExecuteHandler(n.C0.FF, () => this.lineFeed()), this._parser.setExecuteHandler(n.C0.CR, () => this.carriageReturn()), this._parser.setExecuteHandler(n.C0.BS, () => this.backspace()), this._parser.setExecuteHandler(n.C0.HT, () => this.tab()), this._parser.setExecuteHandler(n.C0.SO, () => this.shiftOut()), this._parser.setExecuteHandler(n.C0.SI, () => this.shiftIn()), this._parser.setExecuteHandler(n.C1.IND, () => this.index()), this._parser.setExecuteHandler(n.C1.NEL, () => this.nextLine()), this._parser.setExecuteHandler(n.C1.HTS, () => this.tabSet()), this._parser.registerOscHandler(0, new y.OscHandler((v) => (this.setTitle(v), this.setIconName(v), !0))), this._parser.registerOscHandler(1, new y.OscHandler((v) => this.setIconName(v))), this._parser.registerOscHandler(2, new y.OscHandler((v) => this.setTitle(v))), this._parser.registerOscHandler(4, new y.OscHandler((v) => this.setOrReportIndexedColor(v))), this._parser.registerOscHandler(8, new y.OscHandler((v) => this.setHyperlink(v))), this._parser.registerOscHandler(10, new y.OscHandler((v) => this.setOrReportFgColor(v))), this._parser.registerOscHandler(11, new y.OscHandler((v) => this.setOrReportBgColor(v))), this._parser.registerOscHandler(12, new y.OscHandler((v) => this.setOrReportCursorColor(v))), this._parser.registerOscHandler(104, new y.OscHandler((v) => this.restoreIndexedColor(v))), this._parser.registerOscHandler(110, new y.OscHandler((v) => this.restoreFgColor(v))), this._parser.registerOscHandler(111, new y.OscHandler((v) => this.restoreBgColor(v))), this._parser.registerOscHandler(112, new y.OscHandler((v) => this.restoreCursorColor(v))), this._parser.registerEscHandler({ final: "7" }, () => this.saveCursor()), this._parser.registerEscHandler({ final: "8" }, () => this.restoreCursor()), this._parser.registerEscHandler({ final: "D" }, () => this.index()), this._parser.registerEscHandler({ final: "E" }, () => this.nextLine()), this._parser.registerEscHandler({ final: "H" }, () => this.tabSet()), this._parser.registerEscHandler({ final: "M" }, () => this.reverseIndex()), this._parser.registerEscHandler({ final: "=" }, () => this.keypadApplicationMode()), this._parser.registerEscHandler({ final: ">" }, () => this.keypadNumericMode()), this._parser.registerEscHandler({ final: "c" }, () => this.fullReset()), this._parser.registerEscHandler({ final: "n" }, () => this.setgLevel(2)), this._parser.registerEscHandler({ final: "o" }, () => this.setgLevel(3)), this._parser.registerEscHandler({ final: "|" }, () => this.setgLevel(3)), this._parser.registerEscHandler({ final: "}" }, () => this.setgLevel(2)), this._parser.registerEscHandler({ final: "~" }, () => this.setgLevel(1)), this._parser.registerEscHandler({ intermediates: "%", final: "@" }, () => this.selectDefaultCharset()), this._parser.registerEscHandler({ intermediates: "%", final: "G" }, () => this.selectDefaultCharset());
            for (const v in c.CHARSETS) this._parser.registerEscHandler({ intermediates: "(", final: v }, () => this.selectCharset("(" + v)), this._parser.registerEscHandler({ intermediates: ")", final: v }, () => this.selectCharset(")" + v)), this._parser.registerEscHandler({ intermediates: "*", final: v }, () => this.selectCharset("*" + v)), this._parser.registerEscHandler({ intermediates: "+", final: v }, () => this.selectCharset("+" + v)), this._parser.registerEscHandler({ intermediates: "-", final: v }, () => this.selectCharset("-" + v)), this._parser.registerEscHandler({ intermediates: ".", final: v }, () => this.selectCharset("." + v)), this._parser.registerEscHandler({ intermediates: "/", final: v }, () => this.selectCharset("/" + v));
            this._parser.registerEscHandler({ intermediates: "#", final: "8" }, () => this.screenAlignmentPattern()), this._parser.setErrorHandler((v) => (this._logService.error("Parsing error: ", v), v)), this._parser.registerDcsHandler({ intermediates: "$", final: "q" }, new C.DcsHandler((v, F) => this.requestStatusString(v, F)));
          }
          _preserveStack(o, m, g, b) {
            this._parseStack.paused = !0, this._parseStack.cursorStartX = o, this._parseStack.cursorStartY = m, this._parseStack.decodedLength = g, this._parseStack.position = b;
          }
          _logSlowResolvingAsync(o) {
            this._logService.logLevel <= p.LogLevelEnum.WARN && Promise.race([o, new Promise((m, g) => setTimeout(() => g("#SLOW_TIMEOUT"), 5e3))]).catch((m) => {
              if (m !== "#SLOW_TIMEOUT") throw m;
              console.warn("async parser handler taking longer than 5000 ms");
            });
          }
          _getCurrentLinkId() {
            return this._curAttrData.extended.urlId;
          }
          parse(o, m) {
            let g, b = this._activeBuffer.x, A = this._activeBuffer.y, P = 0;
            const W = this._parseStack.paused;
            if (W) {
              if (g = this._parser.parse(this._parseBuffer, this._parseStack.decodedLength, m)) return this._logSlowResolvingAsync(g), g;
              b = this._parseStack.cursorStartX, A = this._parseStack.cursorStartY, this._parseStack.paused = !1, o.length > E && (P = this._parseStack.position + E);
            }
            if (this._logService.logLevel <= p.LogLevelEnum.DEBUG && this._logService.debug("parsing data" + (typeof o == "string" ? ` "${o}"` : ` "${Array.prototype.map.call(o, (v) => String.fromCharCode(v)).join("")}"`), typeof o == "string" ? o.split("").map((v) => v.charCodeAt(0)) : o), this._parseBuffer.length < o.length && this._parseBuffer.length < E && (this._parseBuffer = new Uint32Array(Math.min(o.length, E))), W || this._dirtyRowTracker.clearRange(), o.length > E) for (let v = P; v < o.length; v += E) {
              const F = v + E < o.length ? v + E : o.length, x = typeof o == "string" ? this._stringDecoder.decode(o.substring(v, F), this._parseBuffer) : this._utf8Decoder.decode(o.subarray(v, F), this._parseBuffer);
              if (g = this._parser.parse(this._parseBuffer, x)) return this._preserveStack(b, A, x, v), this._logSlowResolvingAsync(g), g;
            }
            else if (!W) {
              const v = typeof o == "string" ? this._stringDecoder.decode(o, this._parseBuffer) : this._utf8Decoder.decode(o, this._parseBuffer);
              if (g = this._parser.parse(this._parseBuffer, v)) return this._preserveStack(b, A, v, 0), this._logSlowResolvingAsync(g), g;
            }
            this._activeBuffer.x === b && this._activeBuffer.y === A || this._onCursorMove.fire();
            const j = this._dirtyRowTracker.end + (this._bufferService.buffer.ybase - this._bufferService.buffer.ydisp), z = this._dirtyRowTracker.start + (this._bufferService.buffer.ybase - this._bufferService.buffer.ydisp);
            z < this._bufferService.rows && this._onRequestRefreshRows.fire(Math.min(z, this._bufferService.rows - 1), Math.min(j, this._bufferService.rows - 1));
          }
          print(o, m, g) {
            let b, A;
            const P = this._charsetService.charset, W = this._optionsService.rawOptions.screenReaderMode, j = this._bufferService.cols, z = this._coreService.decPrivateModes.wraparound, v = this._coreService.modes.insertMode, F = this._curAttrData;
            let x = this._activeBuffer.lines.get(this._activeBuffer.ybase + this._activeBuffer.y);
            this._dirtyRowTracker.markDirty(this._activeBuffer.y), this._activeBuffer.x && g - m > 0 && x.getWidth(this._activeBuffer.x - 1) === 2 && x.setCellFromCodepoint(this._activeBuffer.x - 1, 0, 1, F);
            let M = this._parser.precedingJoinState;
            for (let J = m; J < g; ++J) {
              if (b = o[J], b < 127 && P) {
                const Y = P[String.fromCharCode(b)];
                Y && (b = Y.charCodeAt(0));
              }
              const Q = this._unicodeService.charProperties(b, M);
              A = S.UnicodeService.extractWidth(Q);
              const te = S.UnicodeService.extractShouldJoin(Q), K = te ? S.UnicodeService.extractWidth(M) : 0;
              if (M = Q, W && this._onA11yChar.fire((0, u.stringFromCodePoint)(b)), this._getCurrentLinkId() && this._oscLinkService.addLineToLink(this._getCurrentLinkId(), this._activeBuffer.ybase + this._activeBuffer.y), this._activeBuffer.x + A - K > j) {
                if (z) {
                  const Y = x;
                  let Z = this._activeBuffer.x - K;
                  for (this._activeBuffer.x = K, this._activeBuffer.y++, this._activeBuffer.y === this._activeBuffer.scrollBottom + 1 ? (this._activeBuffer.y--, this._bufferService.scroll(this._eraseAttrData(), !0)) : (this._activeBuffer.y >= this._bufferService.rows && (this._activeBuffer.y = this._bufferService.rows - 1), this._activeBuffer.lines.get(this._activeBuffer.ybase + this._activeBuffer.y).isWrapped = !0), x = this._activeBuffer.lines.get(this._activeBuffer.ybase + this._activeBuffer.y), K > 0 && x instanceof e.BufferLine && x.copyCellsFrom(Y, Z, 0, K, !1); Z < j; ) Y.setCellFromCodepoint(Z++, 0, 1, F);
                } else if (this._activeBuffer.x = j - 1, A === 2) continue;
              }
              if (te && this._activeBuffer.x) {
                const Y = x.getWidth(this._activeBuffer.x - 1) ? 1 : 2;
                x.addCodepointToCell(this._activeBuffer.x - Y, b, A);
                for (let Z = A - K; --Z >= 0; ) x.setCellFromCodepoint(this._activeBuffer.x++, 0, 0, F);
              } else if (v && (x.insertCells(this._activeBuffer.x, A - K, this._activeBuffer.getNullCell(F)), x.getWidth(j - 1) === 2 && x.setCellFromCodepoint(j - 1, i.NULL_CELL_CODE, i.NULL_CELL_WIDTH, F)), x.setCellFromCodepoint(this._activeBuffer.x++, b, A, F), A > 0) for (; --A; ) x.setCellFromCodepoint(this._activeBuffer.x++, 0, 0, F);
            }
            this._parser.precedingJoinState = M, this._activeBuffer.x < j && g - m > 0 && x.getWidth(this._activeBuffer.x) === 0 && !x.hasContent(this._activeBuffer.x) && x.setCellFromCodepoint(this._activeBuffer.x, 0, 1, F), this._dirtyRowTracker.markDirty(this._activeBuffer.y);
          }
          registerCsiHandler(o, m) {
            return o.final !== "t" || o.prefix || o.intermediates ? this._parser.registerCsiHandler(o, m) : this._parser.registerCsiHandler(o, (g) => !B(g.params[0], this._optionsService.rawOptions.windowOptions) || m(g));
          }
          registerDcsHandler(o, m) {
            return this._parser.registerDcsHandler(o, new C.DcsHandler(m));
          }
          registerEscHandler(o, m) {
            return this._parser.registerEscHandler(o, m);
          }
          registerOscHandler(o, m) {
            return this._parser.registerOscHandler(o, new y.OscHandler(m));
          }
          bell() {
            return this._onRequestBell.fire(), !0;
          }
          lineFeed() {
            return this._dirtyRowTracker.markDirty(this._activeBuffer.y), this._optionsService.rawOptions.convertEol && (this._activeBuffer.x = 0), this._activeBuffer.y++, this._activeBuffer.y === this._activeBuffer.scrollBottom + 1 ? (this._activeBuffer.y--, this._bufferService.scroll(this._eraseAttrData())) : this._activeBuffer.y >= this._bufferService.rows ? this._activeBuffer.y = this._bufferService.rows - 1 : this._activeBuffer.lines.get(this._activeBuffer.ybase + this._activeBuffer.y).isWrapped = !1, this._activeBuffer.x >= this._bufferService.cols && this._activeBuffer.x--, this._dirtyRowTracker.markDirty(this._activeBuffer.y), this._onLineFeed.fire(), !0;
          }
          carriageReturn() {
            return this._activeBuffer.x = 0, !0;
          }
          backspace() {
            if (!this._coreService.decPrivateModes.reverseWraparound) return this._restrictCursor(), this._activeBuffer.x > 0 && this._activeBuffer.x--, !0;
            if (this._restrictCursor(this._bufferService.cols), this._activeBuffer.x > 0) this._activeBuffer.x--;
            else if (this._activeBuffer.x === 0 && this._activeBuffer.y > this._activeBuffer.scrollTop && this._activeBuffer.y <= this._activeBuffer.scrollBottom && this._activeBuffer.lines.get(this._activeBuffer.ybase + this._activeBuffer.y)?.isWrapped) {
              this._activeBuffer.lines.get(this._activeBuffer.ybase + this._activeBuffer.y).isWrapped = !1, this._activeBuffer.y--, this._activeBuffer.x = this._bufferService.cols - 1;
              const o = this._activeBuffer.lines.get(this._activeBuffer.ybase + this._activeBuffer.y);
              o.hasWidth(this._activeBuffer.x) && !o.hasContent(this._activeBuffer.x) && this._activeBuffer.x--;
            }
            return this._restrictCursor(), !0;
          }
          tab() {
            if (this._activeBuffer.x >= this._bufferService.cols) return !0;
            const o = this._activeBuffer.x;
            return this._activeBuffer.x = this._activeBuffer.nextStop(), this._optionsService.rawOptions.screenReaderMode && this._onA11yTab.fire(this._activeBuffer.x - o), !0;
          }
          shiftOut() {
            return this._charsetService.setgLevel(1), !0;
          }
          shiftIn() {
            return this._charsetService.setgLevel(0), !0;
          }
          _restrictCursor(o = this._bufferService.cols - 1) {
            this._activeBuffer.x = Math.min(o, Math.max(0, this._activeBuffer.x)), this._activeBuffer.y = this._coreService.decPrivateModes.origin ? Math.min(this._activeBuffer.scrollBottom, Math.max(this._activeBuffer.scrollTop, this._activeBuffer.y)) : Math.min(this._bufferService.rows - 1, Math.max(0, this._activeBuffer.y)), this._dirtyRowTracker.markDirty(this._activeBuffer.y);
          }
          _setCursor(o, m) {
            this._dirtyRowTracker.markDirty(this._activeBuffer.y), this._coreService.decPrivateModes.origin ? (this._activeBuffer.x = o, this._activeBuffer.y = this._activeBuffer.scrollTop + m) : (this._activeBuffer.x = o, this._activeBuffer.y = m), this._restrictCursor(), this._dirtyRowTracker.markDirty(this._activeBuffer.y);
          }
          _moveCursor(o, m) {
            this._restrictCursor(), this._setCursor(this._activeBuffer.x + o, this._activeBuffer.y + m);
          }
          cursorUp(o) {
            const m = this._activeBuffer.y - this._activeBuffer.scrollTop;
            return m >= 0 ? this._moveCursor(0, -Math.min(m, o.params[0] || 1)) : this._moveCursor(0, -(o.params[0] || 1)), !0;
          }
          cursorDown(o) {
            const m = this._activeBuffer.scrollBottom - this._activeBuffer.y;
            return m >= 0 ? this._moveCursor(0, Math.min(m, o.params[0] || 1)) : this._moveCursor(0, o.params[0] || 1), !0;
          }
          cursorForward(o) {
            return this._moveCursor(o.params[0] || 1, 0), !0;
          }
          cursorBackward(o) {
            return this._moveCursor(-(o.params[0] || 1), 0), !0;
          }
          cursorNextLine(o) {
            return this.cursorDown(o), this._activeBuffer.x = 0, !0;
          }
          cursorPrecedingLine(o) {
            return this.cursorUp(o), this._activeBuffer.x = 0, !0;
          }
          cursorCharAbsolute(o) {
            return this._setCursor((o.params[0] || 1) - 1, this._activeBuffer.y), !0;
          }
          cursorPosition(o) {
            return this._setCursor(o.length >= 2 ? (o.params[1] || 1) - 1 : 0, (o.params[0] || 1) - 1), !0;
          }
          charPosAbsolute(o) {
            return this._setCursor((o.params[0] || 1) - 1, this._activeBuffer.y), !0;
          }
          hPositionRelative(o) {
            return this._moveCursor(o.params[0] || 1, 0), !0;
          }
          linePosAbsolute(o) {
            return this._setCursor(this._activeBuffer.x, (o.params[0] || 1) - 1), !0;
          }
          vPositionRelative(o) {
            return this._moveCursor(0, o.params[0] || 1), !0;
          }
          hVPosition(o) {
            return this.cursorPosition(o), !0;
          }
          tabClear(o) {
            const m = o.params[0];
            return m === 0 ? delete this._activeBuffer.tabs[this._activeBuffer.x] : m === 3 && (this._activeBuffer.tabs = {}), !0;
          }
          cursorForwardTab(o) {
            if (this._activeBuffer.x >= this._bufferService.cols) return !0;
            let m = o.params[0] || 1;
            for (; m--; ) this._activeBuffer.x = this._activeBuffer.nextStop();
            return !0;
          }
          cursorBackwardTab(o) {
            if (this._activeBuffer.x >= this._bufferService.cols) return !0;
            let m = o.params[0] || 1;
            for (; m--; ) this._activeBuffer.x = this._activeBuffer.prevStop();
            return !0;
          }
          selectProtected(o) {
            const m = o.params[0];
            return m === 1 && (this._curAttrData.bg |= 536870912), m !== 2 && m !== 0 || (this._curAttrData.bg &= -536870913), !0;
          }
          _eraseInBufferLine(o, m, g, b = !1, A = !1) {
            const P = this._activeBuffer.lines.get(this._activeBuffer.ybase + o);
            P.replaceCells(m, g, this._activeBuffer.getNullCell(this._eraseAttrData()), A), b && (P.isWrapped = !1);
          }
          _resetBufferLine(o, m = !1) {
            const g = this._activeBuffer.lines.get(this._activeBuffer.ybase + o);
            g && (g.fill(this._activeBuffer.getNullCell(this._eraseAttrData()), m), this._bufferService.buffer.clearMarkers(this._activeBuffer.ybase + o), g.isWrapped = !1);
          }
          eraseInDisplay(o, m = !1) {
            let g;
            switch (this._restrictCursor(this._bufferService.cols), o.params[0]) {
              case 0:
                for (g = this._activeBuffer.y, this._dirtyRowTracker.markDirty(g), this._eraseInBufferLine(g++, this._activeBuffer.x, this._bufferService.cols, this._activeBuffer.x === 0, m); g < this._bufferService.rows; g++) this._resetBufferLine(g, m);
                this._dirtyRowTracker.markDirty(g);
                break;
              case 1:
                for (g = this._activeBuffer.y, this._dirtyRowTracker.markDirty(g), this._eraseInBufferLine(g, 0, this._activeBuffer.x + 1, !0, m), this._activeBuffer.x + 1 >= this._bufferService.cols && (this._activeBuffer.lines.get(g + 1).isWrapped = !1); g--; ) this._resetBufferLine(g, m);
                this._dirtyRowTracker.markDirty(0);
                break;
              case 2:
                for (g = this._bufferService.rows, this._dirtyRowTracker.markDirty(g - 1); g--; ) this._resetBufferLine(g, m);
                this._dirtyRowTracker.markDirty(0);
                break;
              case 3:
                const b = this._activeBuffer.lines.length - this._bufferService.rows;
                b > 0 && (this._activeBuffer.lines.trimStart(b), this._activeBuffer.ybase = Math.max(this._activeBuffer.ybase - b, 0), this._activeBuffer.ydisp = Math.max(this._activeBuffer.ydisp - b, 0), this._onScroll.fire(0));
            }
            return !0;
          }
          eraseInLine(o, m = !1) {
            switch (this._restrictCursor(this._bufferService.cols), o.params[0]) {
              case 0:
                this._eraseInBufferLine(this._activeBuffer.y, this._activeBuffer.x, this._bufferService.cols, this._activeBuffer.x === 0, m);
                break;
              case 1:
                this._eraseInBufferLine(this._activeBuffer.y, 0, this._activeBuffer.x + 1, !1, m);
                break;
              case 2:
                this._eraseInBufferLine(this._activeBuffer.y, 0, this._bufferService.cols, !0, m);
            }
            return this._dirtyRowTracker.markDirty(this._activeBuffer.y), !0;
          }
          insertLines(o) {
            this._restrictCursor();
            let m = o.params[0] || 1;
            if (this._activeBuffer.y > this._activeBuffer.scrollBottom || this._activeBuffer.y < this._activeBuffer.scrollTop) return !0;
            const g = this._activeBuffer.ybase + this._activeBuffer.y, b = this._bufferService.rows - 1 - this._activeBuffer.scrollBottom, A = this._bufferService.rows - 1 + this._activeBuffer.ybase - b + 1;
            for (; m--; ) this._activeBuffer.lines.splice(A - 1, 1), this._activeBuffer.lines.splice(g, 0, this._activeBuffer.getBlankLine(this._eraseAttrData()));
            return this._dirtyRowTracker.markRangeDirty(this._activeBuffer.y, this._activeBuffer.scrollBottom), this._activeBuffer.x = 0, !0;
          }
          deleteLines(o) {
            this._restrictCursor();
            let m = o.params[0] || 1;
            if (this._activeBuffer.y > this._activeBuffer.scrollBottom || this._activeBuffer.y < this._activeBuffer.scrollTop) return !0;
            const g = this._activeBuffer.ybase + this._activeBuffer.y;
            let b;
            for (b = this._bufferService.rows - 1 - this._activeBuffer.scrollBottom, b = this._bufferService.rows - 1 + this._activeBuffer.ybase - b; m--; ) this._activeBuffer.lines.splice(g, 1), this._activeBuffer.lines.splice(b, 0, this._activeBuffer.getBlankLine(this._eraseAttrData()));
            return this._dirtyRowTracker.markRangeDirty(this._activeBuffer.y, this._activeBuffer.scrollBottom), this._activeBuffer.x = 0, !0;
          }
          insertChars(o) {
            this._restrictCursor();
            const m = this._activeBuffer.lines.get(this._activeBuffer.ybase + this._activeBuffer.y);
            return m && (m.insertCells(this._activeBuffer.x, o.params[0] || 1, this._activeBuffer.getNullCell(this._eraseAttrData())), this._dirtyRowTracker.markDirty(this._activeBuffer.y)), !0;
          }
          deleteChars(o) {
            this._restrictCursor();
            const m = this._activeBuffer.lines.get(this._activeBuffer.ybase + this._activeBuffer.y);
            return m && (m.deleteCells(this._activeBuffer.x, o.params[0] || 1, this._activeBuffer.getNullCell(this._eraseAttrData())), this._dirtyRowTracker.markDirty(this._activeBuffer.y)), !0;
          }
          scrollUp(o) {
            let m = o.params[0] || 1;
            for (; m--; ) this._activeBuffer.lines.splice(this._activeBuffer.ybase + this._activeBuffer.scrollTop, 1), this._activeBuffer.lines.splice(this._activeBuffer.ybase + this._activeBuffer.scrollBottom, 0, this._activeBuffer.getBlankLine(this._eraseAttrData()));
            return this._dirtyRowTracker.markRangeDirty(this._activeBuffer.scrollTop, this._activeBuffer.scrollBottom), !0;
          }
          scrollDown(o) {
            let m = o.params[0] || 1;
            for (; m--; ) this._activeBuffer.lines.splice(this._activeBuffer.ybase + this._activeBuffer.scrollBottom, 1), this._activeBuffer.lines.splice(this._activeBuffer.ybase + this._activeBuffer.scrollTop, 0, this._activeBuffer.getBlankLine(e.DEFAULT_ATTR_DATA));
            return this._dirtyRowTracker.markRangeDirty(this._activeBuffer.scrollTop, this._activeBuffer.scrollBottom), !0;
          }
          scrollLeft(o) {
            if (this._activeBuffer.y > this._activeBuffer.scrollBottom || this._activeBuffer.y < this._activeBuffer.scrollTop) return !0;
            const m = o.params[0] || 1;
            for (let g = this._activeBuffer.scrollTop; g <= this._activeBuffer.scrollBottom; ++g) {
              const b = this._activeBuffer.lines.get(this._activeBuffer.ybase + g);
              b.deleteCells(0, m, this._activeBuffer.getNullCell(this._eraseAttrData())), b.isWrapped = !1;
            }
            return this._dirtyRowTracker.markRangeDirty(this._activeBuffer.scrollTop, this._activeBuffer.scrollBottom), !0;
          }
          scrollRight(o) {
            if (this._activeBuffer.y > this._activeBuffer.scrollBottom || this._activeBuffer.y < this._activeBuffer.scrollTop) return !0;
            const m = o.params[0] || 1;
            for (let g = this._activeBuffer.scrollTop; g <= this._activeBuffer.scrollBottom; ++g) {
              const b = this._activeBuffer.lines.get(this._activeBuffer.ybase + g);
              b.insertCells(0, m, this._activeBuffer.getNullCell(this._eraseAttrData())), b.isWrapped = !1;
            }
            return this._dirtyRowTracker.markRangeDirty(this._activeBuffer.scrollTop, this._activeBuffer.scrollBottom), !0;
          }
          insertColumns(o) {
            if (this._activeBuffer.y > this._activeBuffer.scrollBottom || this._activeBuffer.y < this._activeBuffer.scrollTop) return !0;
            const m = o.params[0] || 1;
            for (let g = this._activeBuffer.scrollTop; g <= this._activeBuffer.scrollBottom; ++g) {
              const b = this._activeBuffer.lines.get(this._activeBuffer.ybase + g);
              b.insertCells(this._activeBuffer.x, m, this._activeBuffer.getNullCell(this._eraseAttrData())), b.isWrapped = !1;
            }
            return this._dirtyRowTracker.markRangeDirty(this._activeBuffer.scrollTop, this._activeBuffer.scrollBottom), !0;
          }
          deleteColumns(o) {
            if (this._activeBuffer.y > this._activeBuffer.scrollBottom || this._activeBuffer.y < this._activeBuffer.scrollTop) return !0;
            const m = o.params[0] || 1;
            for (let g = this._activeBuffer.scrollTop; g <= this._activeBuffer.scrollBottom; ++g) {
              const b = this._activeBuffer.lines.get(this._activeBuffer.ybase + g);
              b.deleteCells(this._activeBuffer.x, m, this._activeBuffer.getNullCell(this._eraseAttrData())), b.isWrapped = !1;
            }
            return this._dirtyRowTracker.markRangeDirty(this._activeBuffer.scrollTop, this._activeBuffer.scrollBottom), !0;
          }
          eraseChars(o) {
            this._restrictCursor();
            const m = this._activeBuffer.lines.get(this._activeBuffer.ybase + this._activeBuffer.y);
            return m && (m.replaceCells(this._activeBuffer.x, this._activeBuffer.x + (o.params[0] || 1), this._activeBuffer.getNullCell(this._eraseAttrData())), this._dirtyRowTracker.markDirty(this._activeBuffer.y)), !0;
          }
          repeatPrecedingCharacter(o) {
            const m = this._parser.precedingJoinState;
            if (!m) return !0;
            const g = o.params[0] || 1, b = S.UnicodeService.extractWidth(m), A = this._activeBuffer.x - b, P = this._activeBuffer.lines.get(this._activeBuffer.ybase + this._activeBuffer.y).getString(A), W = new Uint32Array(P.length * g);
            let j = 0;
            for (let v = 0; v < P.length; ) {
              const F = P.codePointAt(v) || 0;
              W[j++] = F, v += F > 65535 ? 2 : 1;
            }
            let z = j;
            for (let v = 1; v < g; ++v) W.copyWithin(z, 0, j), z += j;
            return this.print(W, 0, z), !0;
          }
          sendDeviceAttributesPrimary(o) {
            return o.params[0] > 0 || (this._is("xterm") || this._is("rxvt-unicode") || this._is("screen") ? this._coreService.triggerDataEvent(n.C0.ESC + "[?1;2c") : this._is("linux") && this._coreService.triggerDataEvent(n.C0.ESC + "[?6c")), !0;
          }
          sendDeviceAttributesSecondary(o) {
            return o.params[0] > 0 || (this._is("xterm") ? this._coreService.triggerDataEvent(n.C0.ESC + "[>0;276;0c") : this._is("rxvt-unicode") ? this._coreService.triggerDataEvent(n.C0.ESC + "[>85;95;0c") : this._is("linux") ? this._coreService.triggerDataEvent(o.params[0] + "c") : this._is("screen") && this._coreService.triggerDataEvent(n.C0.ESC + "[>83;40003;0c")), !0;
          }
          _is(o) {
            return (this._optionsService.rawOptions.termName + "").indexOf(o) === 0;
          }
          setMode(o) {
            for (let m = 0; m < o.length; m++) switch (o.params[m]) {
              case 4:
                this._coreService.modes.insertMode = !0;
                break;
              case 20:
                this._optionsService.options.convertEol = !0;
            }
            return !0;
          }
          setModePrivate(o) {
            for (let m = 0; m < o.length; m++) switch (o.params[m]) {
              case 1:
                this._coreService.decPrivateModes.applicationCursorKeys = !0;
                break;
              case 2:
                this._charsetService.setgCharset(0, c.DEFAULT_CHARSET), this._charsetService.setgCharset(1, c.DEFAULT_CHARSET), this._charsetService.setgCharset(2, c.DEFAULT_CHARSET), this._charsetService.setgCharset(3, c.DEFAULT_CHARSET);
                break;
              case 3:
                this._optionsService.rawOptions.windowOptions.setWinLines && (this._bufferService.resize(132, this._bufferService.rows), this._onRequestReset.fire());
                break;
              case 6:
                this._coreService.decPrivateModes.origin = !0, this._setCursor(0, 0);
                break;
              case 7:
                this._coreService.decPrivateModes.wraparound = !0;
                break;
              case 12:
                this._optionsService.options.cursorBlink = !0;
                break;
              case 45:
                this._coreService.decPrivateModes.reverseWraparound = !0;
                break;
              case 66:
                this._logService.debug("Serial port requested application keypad."), this._coreService.decPrivateModes.applicationKeypad = !0, this._onRequestSyncScrollBar.fire();
                break;
              case 9:
                this._coreMouseService.activeProtocol = "X10";
                break;
              case 1e3:
                this._coreMouseService.activeProtocol = "VT200";
                break;
              case 1002:
                this._coreMouseService.activeProtocol = "DRAG";
                break;
              case 1003:
                this._coreMouseService.activeProtocol = "ANY";
                break;
              case 1004:
                this._coreService.decPrivateModes.sendFocus = !0, this._onRequestSendFocus.fire();
                break;
              case 1005:
                this._logService.debug("DECSET 1005 not supported (see #2507)");
                break;
              case 1006:
                this._coreMouseService.activeEncoding = "SGR";
                break;
              case 1015:
                this._logService.debug("DECSET 1015 not supported (see #2507)");
                break;
              case 1016:
                this._coreMouseService.activeEncoding = "SGR_PIXELS";
                break;
              case 25:
                this._coreService.isCursorHidden = !1;
                break;
              case 1048:
                this.saveCursor();
                break;
              case 1049:
                this.saveCursor();
              case 47:
              case 1047:
                this._bufferService.buffers.activateAltBuffer(this._eraseAttrData()), this._coreService.isCursorInitialized = !0, this._onRequestRefreshRows.fire(0, this._bufferService.rows - 1), this._onRequestSyncScrollBar.fire();
                break;
              case 2004:
                this._coreService.decPrivateModes.bracketedPasteMode = !0;
            }
            return !0;
          }
          resetMode(o) {
            for (let m = 0; m < o.length; m++) switch (o.params[m]) {
              case 4:
                this._coreService.modes.insertMode = !1;
                break;
              case 20:
                this._optionsService.options.convertEol = !1;
            }
            return !0;
          }
          resetModePrivate(o) {
            for (let m = 0; m < o.length; m++) switch (o.params[m]) {
              case 1:
                this._coreService.decPrivateModes.applicationCursorKeys = !1;
                break;
              case 3:
                this._optionsService.rawOptions.windowOptions.setWinLines && (this._bufferService.resize(80, this._bufferService.rows), this._onRequestReset.fire());
                break;
              case 6:
                this._coreService.decPrivateModes.origin = !1, this._setCursor(0, 0);
                break;
              case 7:
                this._coreService.decPrivateModes.wraparound = !1;
                break;
              case 12:
                this._optionsService.options.cursorBlink = !1;
                break;
              case 45:
                this._coreService.decPrivateModes.reverseWraparound = !1;
                break;
              case 66:
                this._logService.debug("Switching back to normal keypad."), this._coreService.decPrivateModes.applicationKeypad = !1, this._onRequestSyncScrollBar.fire();
                break;
              case 9:
              case 1e3:
              case 1002:
              case 1003:
                this._coreMouseService.activeProtocol = "NONE";
                break;
              case 1004:
                this._coreService.decPrivateModes.sendFocus = !1;
                break;
              case 1005:
                this._logService.debug("DECRST 1005 not supported (see #2507)");
                break;
              case 1006:
              case 1016:
                this._coreMouseService.activeEncoding = "DEFAULT";
                break;
              case 1015:
                this._logService.debug("DECRST 1015 not supported (see #2507)");
                break;
              case 25:
                this._coreService.isCursorHidden = !0;
                break;
              case 1048:
                this.restoreCursor();
                break;
              case 1049:
              case 47:
              case 1047:
                this._bufferService.buffers.activateNormalBuffer(), o.params[m] === 1049 && this.restoreCursor(), this._coreService.isCursorInitialized = !0, this._onRequestRefreshRows.fire(0, this._bufferService.rows - 1), this._onRequestSyncScrollBar.fire();
                break;
              case 2004:
                this._coreService.decPrivateModes.bracketedPasteMode = !1;
            }
            return !0;
          }
          requestMode(o, m) {
            const g = this._coreService.decPrivateModes, { activeProtocol: b, activeEncoding: A } = this._coreMouseService, P = this._coreService, { buffers: W, cols: j } = this._bufferService, { active: z, alt: v } = W, F = this._optionsService.rawOptions, x = (te) => te ? 1 : 2, M = o.params[0];
            return J = M, Q = m ? M === 2 ? 4 : M === 4 ? x(P.modes.insertMode) : M === 12 ? 3 : M === 20 ? x(F.convertEol) : 0 : M === 1 ? x(g.applicationCursorKeys) : M === 3 ? F.windowOptions.setWinLines ? j === 80 ? 2 : j === 132 ? 1 : 0 : 0 : M === 6 ? x(g.origin) : M === 7 ? x(g.wraparound) : M === 8 ? 3 : M === 9 ? x(b === "X10") : M === 12 ? x(F.cursorBlink) : M === 25 ? x(!P.isCursorHidden) : M === 45 ? x(g.reverseWraparound) : M === 66 ? x(g.applicationKeypad) : M === 67 ? 4 : M === 1e3 ? x(b === "VT200") : M === 1002 ? x(b === "DRAG") : M === 1003 ? x(b === "ANY") : M === 1004 ? x(g.sendFocus) : M === 1005 ? 4 : M === 1006 ? x(A === "SGR") : M === 1015 ? 4 : M === 1016 ? x(A === "SGR_PIXELS") : M === 1048 ? 1 : M === 47 || M === 1047 || M === 1049 ? x(z === v) : M === 2004 ? x(g.bracketedPasteMode) : 0, P.triggerDataEvent(`${n.C0.ESC}[${m ? "" : "?"}${J};${Q}$y`), !0;
            var J, Q;
          }
          _updateAttrColor(o, m, g, b, A) {
            return m === 2 ? (o |= 50331648, o &= -16777216, o |= f.AttributeData.fromColorRGB([g, b, A])) : m === 5 && (o &= -50331904, o |= 33554432 | 255 & g), o;
          }
          _extractColor(o, m, g) {
            const b = [0, 0, -1, 0, 0, 0];
            let A = 0, P = 0;
            do {
              if (b[P + A] = o.params[m + P], o.hasSubParams(m + P)) {
                const W = o.getSubParams(m + P);
                let j = 0;
                do
                  b[1] === 5 && (A = 1), b[P + j + 1 + A] = W[j];
                while (++j < W.length && j + P + 1 + A < b.length);
                break;
              }
              if (b[1] === 5 && P + A >= 2 || b[1] === 2 && P + A >= 5) break;
              b[1] && (A = 1);
            } while (++P + m < o.length && P + A < b.length);
            for (let W = 2; W < b.length; ++W) b[W] === -1 && (b[W] = 0);
            switch (b[0]) {
              case 38:
                g.fg = this._updateAttrColor(g.fg, b[1], b[3], b[4], b[5]);
                break;
              case 48:
                g.bg = this._updateAttrColor(g.bg, b[1], b[3], b[4], b[5]);
                break;
              case 58:
                g.extended = g.extended.clone(), g.extended.underlineColor = this._updateAttrColor(g.extended.underlineColor, b[1], b[3], b[4], b[5]);
            }
            return P;
          }
          _processUnderline(o, m) {
            m.extended = m.extended.clone(), (!~o || o > 5) && (o = 1), m.extended.underlineStyle = o, m.fg |= 268435456, o === 0 && (m.fg &= -268435457), m.updateExtended();
          }
          _processSGR0(o) {
            o.fg = e.DEFAULT_ATTR_DATA.fg, o.bg = e.DEFAULT_ATTR_DATA.bg, o.extended = o.extended.clone(), o.extended.underlineStyle = 0, o.extended.underlineColor &= -67108864, o.updateExtended();
          }
          charAttributes(o) {
            if (o.length === 1 && o.params[0] === 0) return this._processSGR0(this._curAttrData), !0;
            const m = o.length;
            let g;
            const b = this._curAttrData;
            for (let A = 0; A < m; A++) g = o.params[A], g >= 30 && g <= 37 ? (b.fg &= -50331904, b.fg |= 16777216 | g - 30) : g >= 40 && g <= 47 ? (b.bg &= -50331904, b.bg |= 16777216 | g - 40) : g >= 90 && g <= 97 ? (b.fg &= -50331904, b.fg |= 16777224 | g - 90) : g >= 100 && g <= 107 ? (b.bg &= -50331904, b.bg |= 16777224 | g - 100) : g === 0 ? this._processSGR0(b) : g === 1 ? b.fg |= 134217728 : g === 3 ? b.bg |= 67108864 : g === 4 ? (b.fg |= 268435456, this._processUnderline(o.hasSubParams(A) ? o.getSubParams(A)[0] : 1, b)) : g === 5 ? b.fg |= 536870912 : g === 7 ? b.fg |= 67108864 : g === 8 ? b.fg |= 1073741824 : g === 9 ? b.fg |= 2147483648 : g === 2 ? b.bg |= 134217728 : g === 21 ? this._processUnderline(2, b) : g === 22 ? (b.fg &= -134217729, b.bg &= -134217729) : g === 23 ? b.bg &= -67108865 : g === 24 ? (b.fg &= -268435457, this._processUnderline(0, b)) : g === 25 ? b.fg &= -536870913 : g === 27 ? b.fg &= -67108865 : g === 28 ? b.fg &= -1073741825 : g === 29 ? b.fg &= 2147483647 : g === 39 ? (b.fg &= -67108864, b.fg |= 16777215 & e.DEFAULT_ATTR_DATA.fg) : g === 49 ? (b.bg &= -67108864, b.bg |= 16777215 & e.DEFAULT_ATTR_DATA.bg) : g === 38 || g === 48 || g === 58 ? A += this._extractColor(o, A, b) : g === 53 ? b.bg |= 1073741824 : g === 55 ? b.bg &= -1073741825 : g === 59 ? (b.extended = b.extended.clone(), b.extended.underlineColor = -1, b.updateExtended()) : g === 100 ? (b.fg &= -67108864, b.fg |= 16777215 & e.DEFAULT_ATTR_DATA.fg, b.bg &= -67108864, b.bg |= 16777215 & e.DEFAULT_ATTR_DATA.bg) : this._logService.debug("Unknown SGR attribute: %d.", g);
            return !0;
          }
          deviceStatus(o) {
            switch (o.params[0]) {
              case 5:
                this._coreService.triggerDataEvent(`${n.C0.ESC}[0n`);
                break;
              case 6:
                const m = this._activeBuffer.y + 1, g = this._activeBuffer.x + 1;
                this._coreService.triggerDataEvent(`${n.C0.ESC}[${m};${g}R`);
            }
            return !0;
          }
          deviceStatusPrivate(o) {
            if (o.params[0] === 6) {
              const m = this._activeBuffer.y + 1, g = this._activeBuffer.x + 1;
              this._coreService.triggerDataEvent(`${n.C0.ESC}[?${m};${g}R`);
            }
            return !0;
          }
          softReset(o) {
            return this._coreService.isCursorHidden = !1, this._onRequestSyncScrollBar.fire(), this._activeBuffer.scrollTop = 0, this._activeBuffer.scrollBottom = this._bufferService.rows - 1, this._curAttrData = e.DEFAULT_ATTR_DATA.clone(), this._coreService.reset(), this._charsetService.reset(), this._activeBuffer.savedX = 0, this._activeBuffer.savedY = this._activeBuffer.ybase, this._activeBuffer.savedCurAttrData.fg = this._curAttrData.fg, this._activeBuffer.savedCurAttrData.bg = this._curAttrData.bg, this._activeBuffer.savedCharset = this._charsetService.charset, this._coreService.decPrivateModes.origin = !1, !0;
          }
          setCursorStyle(o) {
            const m = o.params[0] || 1;
            switch (m) {
              case 1:
              case 2:
                this._optionsService.options.cursorStyle = "block";
                break;
              case 3:
              case 4:
                this._optionsService.options.cursorStyle = "underline";
                break;
              case 5:
              case 6:
                this._optionsService.options.cursorStyle = "bar";
            }
            const g = m % 2 == 1;
            return this._optionsService.options.cursorBlink = g, !0;
          }
          setScrollRegion(o) {
            const m = o.params[0] || 1;
            let g;
            return (o.length < 2 || (g = o.params[1]) > this._bufferService.rows || g === 0) && (g = this._bufferService.rows), g > m && (this._activeBuffer.scrollTop = m - 1, this._activeBuffer.scrollBottom = g - 1, this._setCursor(0, 0)), !0;
          }
          windowOptions(o) {
            if (!B(o.params[0], this._optionsService.rawOptions.windowOptions)) return !0;
            const m = o.length > 1 ? o.params[1] : 0;
            switch (o.params[0]) {
              case 14:
                m !== 2 && this._onRequestWindowsOptionsReport.fire(D.GET_WIN_SIZE_PIXELS);
                break;
              case 16:
                this._onRequestWindowsOptionsReport.fire(D.GET_CELL_SIZE_PIXELS);
                break;
              case 18:
                this._bufferService && this._coreService.triggerDataEvent(`${n.C0.ESC}[8;${this._bufferService.rows};${this._bufferService.cols}t`);
                break;
              case 22:
                m !== 0 && m !== 2 || (this._windowTitleStack.push(this._windowTitle), this._windowTitleStack.length > 10 && this._windowTitleStack.shift()), m !== 0 && m !== 1 || (this._iconNameStack.push(this._iconName), this._iconNameStack.length > 10 && this._iconNameStack.shift());
                break;
              case 23:
                m !== 0 && m !== 2 || this._windowTitleStack.length && this.setTitle(this._windowTitleStack.pop()), m !== 0 && m !== 1 || this._iconNameStack.length && this.setIconName(this._iconNameStack.pop());
            }
            return !0;
          }
          saveCursor(o) {
            return this._activeBuffer.savedX = this._activeBuffer.x, this._activeBuffer.savedY = this._activeBuffer.ybase + this._activeBuffer.y, this._activeBuffer.savedCurAttrData.fg = this._curAttrData.fg, this._activeBuffer.savedCurAttrData.bg = this._curAttrData.bg, this._activeBuffer.savedCharset = this._charsetService.charset, !0;
          }
          restoreCursor(o) {
            return this._activeBuffer.x = this._activeBuffer.savedX || 0, this._activeBuffer.y = Math.max(this._activeBuffer.savedY - this._activeBuffer.ybase, 0), this._curAttrData.fg = this._activeBuffer.savedCurAttrData.fg, this._curAttrData.bg = this._activeBuffer.savedCurAttrData.bg, this._charsetService.charset = this._savedCharset, this._activeBuffer.savedCharset && (this._charsetService.charset = this._activeBuffer.savedCharset), this._restrictCursor(), !0;
          }
          setTitle(o) {
            return this._windowTitle = o, this._onTitleChange.fire(o), !0;
          }
          setIconName(o) {
            return this._iconName = o, !0;
          }
          setOrReportIndexedColor(o) {
            const m = [], g = o.split(";");
            for (; g.length > 1; ) {
              const b = g.shift(), A = g.shift();
              if (/^\d+$/.exec(b)) {
                const P = parseInt(b);
                if (X(P)) if (A === "?") m.push({ type: 0, index: P });
                else {
                  const W = (0, T.parseColor)(A);
                  W && m.push({ type: 1, index: P, color: W });
                }
              }
            }
            return m.length && this._onColor.fire(m), !0;
          }
          setHyperlink(o) {
            const m = o.split(";");
            return !(m.length < 2) && (m[1] ? this._createHyperlink(m[0], m[1]) : !m[0] && this._finishHyperlink());
          }
          _createHyperlink(o, m) {
            this._getCurrentLinkId() && this._finishHyperlink();
            const g = o.split(":");
            let b;
            const A = g.findIndex((P) => P.startsWith("id="));
            return A !== -1 && (b = g[A].slice(3) || void 0), this._curAttrData.extended = this._curAttrData.extended.clone(), this._curAttrData.extended.urlId = this._oscLinkService.registerLink({ id: b, uri: m }), this._curAttrData.updateExtended(), !0;
          }
          _finishHyperlink() {
            return this._curAttrData.extended = this._curAttrData.extended.clone(), this._curAttrData.extended.urlId = 0, this._curAttrData.updateExtended(), !0;
          }
          _setOrReportSpecialColor(o, m) {
            const g = o.split(";");
            for (let b = 0; b < g.length && !(m >= this._specialColors.length); ++b, ++m) if (g[b] === "?") this._onColor.fire([{ type: 0, index: this._specialColors[m] }]);
            else {
              const A = (0, T.parseColor)(g[b]);
              A && this._onColor.fire([{ type: 1, index: this._specialColors[m], color: A }]);
            }
            return !0;
          }
          setOrReportFgColor(o) {
            return this._setOrReportSpecialColor(o, 0);
          }
          setOrReportBgColor(o) {
            return this._setOrReportSpecialColor(o, 1);
          }
          setOrReportCursorColor(o) {
            return this._setOrReportSpecialColor(o, 2);
          }
          restoreIndexedColor(o) {
            if (!o) return this._onColor.fire([{ type: 2 }]), !0;
            const m = [], g = o.split(";");
            for (let b = 0; b < g.length; ++b) if (/^\d+$/.exec(g[b])) {
              const A = parseInt(g[b]);
              X(A) && m.push({ type: 2, index: A });
            }
            return m.length && this._onColor.fire(m), !0;
          }
          restoreFgColor(o) {
            return this._onColor.fire([{ type: 2, index: 256 }]), !0;
          }
          restoreBgColor(o) {
            return this._onColor.fire([{ type: 2, index: 257 }]), !0;
          }
          restoreCursorColor(o) {
            return this._onColor.fire([{ type: 2, index: 258 }]), !0;
          }
          nextLine() {
            return this._activeBuffer.x = 0, this.index(), !0;
          }
          keypadApplicationMode() {
            return this._logService.debug("Serial port requested application keypad."), this._coreService.decPrivateModes.applicationKeypad = !0, this._onRequestSyncScrollBar.fire(), !0;
          }
          keypadNumericMode() {
            return this._logService.debug("Switching back to normal keypad."), this._coreService.decPrivateModes.applicationKeypad = !1, this._onRequestSyncScrollBar.fire(), !0;
          }
          selectDefaultCharset() {
            return this._charsetService.setgLevel(0), this._charsetService.setgCharset(0, c.DEFAULT_CHARSET), !0;
          }
          selectCharset(o) {
            return o.length !== 2 ? (this.selectDefaultCharset(), !0) : (o[0] === "/" || this._charsetService.setgCharset($[o[0]], c.CHARSETS[o[1]] || c.DEFAULT_CHARSET), !0);
          }
          index() {
            return this._restrictCursor(), this._activeBuffer.y++, this._activeBuffer.y === this._activeBuffer.scrollBottom + 1 ? (this._activeBuffer.y--, this._bufferService.scroll(this._eraseAttrData())) : this._activeBuffer.y >= this._bufferService.rows && (this._activeBuffer.y = this._bufferService.rows - 1), this._restrictCursor(), !0;
          }
          tabSet() {
            return this._activeBuffer.tabs[this._activeBuffer.x] = !0, !0;
          }
          reverseIndex() {
            if (this._restrictCursor(), this._activeBuffer.y === this._activeBuffer.scrollTop) {
              const o = this._activeBuffer.scrollBottom - this._activeBuffer.scrollTop;
              this._activeBuffer.lines.shiftElements(this._activeBuffer.ybase + this._activeBuffer.y, o, 1), this._activeBuffer.lines.set(this._activeBuffer.ybase + this._activeBuffer.y, this._activeBuffer.getBlankLine(this._eraseAttrData())), this._dirtyRowTracker.markRangeDirty(this._activeBuffer.scrollTop, this._activeBuffer.scrollBottom);
            } else this._activeBuffer.y--, this._restrictCursor();
            return !0;
          }
          fullReset() {
            return this._parser.reset(), this._onRequestReset.fire(), !0;
          }
          reset() {
            this._curAttrData = e.DEFAULT_ATTR_DATA.clone(), this._eraseAttrDataInternal = e.DEFAULT_ATTR_DATA.clone();
          }
          _eraseAttrData() {
            return this._eraseAttrDataInternal.bg &= -67108864, this._eraseAttrDataInternal.bg |= 67108863 & this._curAttrData.bg, this._eraseAttrDataInternal;
          }
          setgLevel(o) {
            return this._charsetService.setgLevel(o), !0;
          }
          screenAlignmentPattern() {
            const o = new s.CellData();
            o.content = 4194373, o.fg = this._curAttrData.fg, o.bg = this._curAttrData.bg, this._setCursor(0, 0);
            for (let m = 0; m < this._bufferService.rows; ++m) {
              const g = this._activeBuffer.ybase + this._activeBuffer.y + m, b = this._activeBuffer.lines.get(g);
              b && (b.fill(o), b.isWrapped = !1);
            }
            return this._dirtyRowTracker.markAllDirty(), this._setCursor(0, 0), !0;
          }
          requestStatusString(o, m) {
            const g = this._bufferService.buffer, b = this._optionsService.rawOptions;
            return ((A) => (this._coreService.triggerDataEvent(`${n.C0.ESC}${A}${n.C0.ESC}\\`), !0))(o === '"q' ? `P1$r${this._curAttrData.isProtected() ? 1 : 0}"q` : o === '"p' ? 'P1$r61;1"p' : o === "r" ? `P1$r${g.scrollTop + 1};${g.scrollBottom + 1}r` : o === "m" ? "P1$r0m" : o === " q" ? `P1$r${{ block: 2, underline: 4, bar: 6 }[b.cursorStyle] - (b.cursorBlink ? 1 : 0)} q` : "P0$r");
          }
          markRangeDirty(o, m) {
            this._dirtyRowTracker.markRangeDirty(o, m);
          }
        }
        t.InputHandler = V;
        let U = class {
          constructor(H) {
            this._bufferService = H, this.clearRange();
          }
          clearRange() {
            this.start = this._bufferService.buffer.y, this.end = this._bufferService.buffer.y;
          }
          markDirty(H) {
            H < this.start ? this.start = H : H > this.end && (this.end = H);
          }
          markRangeDirty(H, o) {
            H > o && (G = H, H = o, o = G), H < this.start && (this.start = H), o > this.end && (this.end = o);
          }
          markAllDirty() {
            this.markRangeDirty(0, this._bufferService.rows - 1);
          }
        };
        function X(H) {
          return 0 <= H && H < 256;
        }
        U = d([_(0, p.IBufferService)], U);
      }, 844: (w, t) => {
        function l(d) {
          for (const _ of d) _.dispose();
          d.length = 0;
        }
        Object.defineProperty(t, "__esModule", { value: !0 }), t.getDisposeArrayDisposable = t.disposeArray = t.toDisposable = t.MutableDisposable = t.Disposable = void 0, t.Disposable = class {
          constructor() {
            this._disposables = [], this._isDisposed = !1;
          }
          dispose() {
            this._isDisposed = !0;
            for (const d of this._disposables) d.dispose();
            this._disposables.length = 0;
          }
          register(d) {
            return this._disposables.push(d), d;
          }
          unregister(d) {
            const _ = this._disposables.indexOf(d);
            _ !== -1 && this._disposables.splice(_, 1);
          }
        }, t.MutableDisposable = class {
          constructor() {
            this._isDisposed = !1;
          }
          get value() {
            return this._isDisposed ? void 0 : this._value;
          }
          set value(d) {
            this._isDisposed || d === this._value || (this._value?.dispose(), this._value = d);
          }
          clear() {
            this.value = void 0;
          }
          dispose() {
            this._isDisposed = !0, this._value?.dispose(), this._value = void 0;
          }
        }, t.toDisposable = function(d) {
          return { dispose: d };
        }, t.disposeArray = l, t.getDisposeArrayDisposable = function(d) {
          return { dispose: () => l(d) };
        };
      }, 114: (w, t) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.isChromeOS = t.isLinux = t.isWindows = t.isIphone = t.isIpad = t.isMac = t.getSafariVersion = t.isSafari = t.isLegacyEdge = t.isFirefox = t.isNode = void 0, t.isNode = typeof process < "u" && "title" in process;
        const l = t.isNode ? "node" : navigator.userAgent, d = t.isNode ? "node" : navigator.platform;
        t.isFirefox = l.includes("Firefox"), t.isLegacyEdge = l.includes("Edge"), t.isSafari = /^((?!chrome|android).)*safari/i.test(l), t.getSafariVersion = function() {
          if (!t.isSafari) return 0;
          const _ = l.match(/Version\/(\d+)/);
          return _ === null || _.length < 2 ? 0 : parseInt(_[1]);
        }, t.isMac = ["Macintosh", "MacIntel", "MacPPC", "Mac68K"].includes(d), t.isIpad = d === "iPad", t.isIphone = d === "iPhone", t.isWindows = ["Windows", "Win16", "Win32", "WinCE"].includes(d), t.isLinux = d.indexOf("Linux") >= 0, t.isChromeOS = /\bCrOS\b/.test(l);
      }, 226: (w, t, l) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.DebouncedIdleTask = t.IdleTaskQueue = t.PriorityTaskQueue = void 0;
        const d = l(114);
        class _ {
          constructor() {
            this._tasks = [], this._i = 0;
          }
          enqueue(a) {
            this._tasks.push(a), this._start();
          }
          flush() {
            for (; this._i < this._tasks.length; ) this._tasks[this._i]() || this._i++;
            this.clear();
          }
          clear() {
            this._idleCallback && (this._cancelCallback(this._idleCallback), this._idleCallback = void 0), this._i = 0, this._tasks.length = 0;
          }
          _start() {
            this._idleCallback || (this._idleCallback = this._requestCallback(this._process.bind(this)));
          }
          _process(a) {
            this._idleCallback = void 0;
            let h = 0, u = 0, e = a.timeRemaining(), r = 0;
            for (; this._i < this._tasks.length; ) {
              if (h = Date.now(), this._tasks[this._i]() || this._i++, h = Math.max(1, Date.now() - h), u = Math.max(h, u), r = a.timeRemaining(), 1.5 * u > r) return e - h < -20 && console.warn(`task queue exceeded allotted deadline by ${Math.abs(Math.round(e - h))}ms`), void this._start();
              e = r;
            }
            this.clear();
          }
        }
        class n extends _ {
          _requestCallback(a) {
            return setTimeout(() => a(this._createDeadline(16)));
          }
          _cancelCallback(a) {
            clearTimeout(a);
          }
          _createDeadline(a) {
            const h = Date.now() + a;
            return { timeRemaining: () => Math.max(0, h - Date.now()) };
          }
        }
        t.PriorityTaskQueue = n, t.IdleTaskQueue = !d.isNode && "requestIdleCallback" in window ? class extends _ {
          _requestCallback(c) {
            return requestIdleCallback(c);
          }
          _cancelCallback(c) {
            cancelIdleCallback(c);
          }
        } : n, t.DebouncedIdleTask = class {
          constructor() {
            this._queue = new t.IdleTaskQueue();
          }
          set(c) {
            this._queue.clear(), this._queue.enqueue(c);
          }
          flush() {
            this._queue.flush();
          }
        };
      }, 282: (w, t, l) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.updateWindowsModeWrappedState = void 0;
        const d = l(643);
        t.updateWindowsModeWrappedState = function(_) {
          const n = _.buffer.lines.get(_.buffer.ybase + _.buffer.y - 1), c = n?.get(_.cols - 1), a = _.buffer.lines.get(_.buffer.ybase + _.buffer.y);
          a && c && (a.isWrapped = c[d.CHAR_DATA_CODE_INDEX] !== d.NULL_CELL_CODE && c[d.CHAR_DATA_CODE_INDEX] !== d.WHITESPACE_CELL_CODE);
        };
      }, 734: (w, t) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.ExtendedAttrs = t.AttributeData = void 0;
        class l {
          constructor() {
            this.fg = 0, this.bg = 0, this.extended = new d();
          }
          static toColorRGB(n) {
            return [n >>> 16 & 255, n >>> 8 & 255, 255 & n];
          }
          static fromColorRGB(n) {
            return (255 & n[0]) << 16 | (255 & n[1]) << 8 | 255 & n[2];
          }
          clone() {
            const n = new l();
            return n.fg = this.fg, n.bg = this.bg, n.extended = this.extended.clone(), n;
          }
          isInverse() {
            return 67108864 & this.fg;
          }
          isBold() {
            return 134217728 & this.fg;
          }
          isUnderline() {
            return this.hasExtendedAttrs() && this.extended.underlineStyle !== 0 ? 1 : 268435456 & this.fg;
          }
          isBlink() {
            return 536870912 & this.fg;
          }
          isInvisible() {
            return 1073741824 & this.fg;
          }
          isItalic() {
            return 67108864 & this.bg;
          }
          isDim() {
            return 134217728 & this.bg;
          }
          isStrikethrough() {
            return 2147483648 & this.fg;
          }
          isProtected() {
            return 536870912 & this.bg;
          }
          isOverline() {
            return 1073741824 & this.bg;
          }
          getFgColorMode() {
            return 50331648 & this.fg;
          }
          getBgColorMode() {
            return 50331648 & this.bg;
          }
          isFgRGB() {
            return (50331648 & this.fg) == 50331648;
          }
          isBgRGB() {
            return (50331648 & this.bg) == 50331648;
          }
          isFgPalette() {
            return (50331648 & this.fg) == 16777216 || (50331648 & this.fg) == 33554432;
          }
          isBgPalette() {
            return (50331648 & this.bg) == 16777216 || (50331648 & this.bg) == 33554432;
          }
          isFgDefault() {
            return (50331648 & this.fg) == 0;
          }
          isBgDefault() {
            return (50331648 & this.bg) == 0;
          }
          isAttributeDefault() {
            return this.fg === 0 && this.bg === 0;
          }
          getFgColor() {
            switch (50331648 & this.fg) {
              case 16777216:
              case 33554432:
                return 255 & this.fg;
              case 50331648:
                return 16777215 & this.fg;
              default:
                return -1;
            }
          }
          getBgColor() {
            switch (50331648 & this.bg) {
              case 16777216:
              case 33554432:
                return 255 & this.bg;
              case 50331648:
                return 16777215 & this.bg;
              default:
                return -1;
            }
          }
          hasExtendedAttrs() {
            return 268435456 & this.bg;
          }
          updateExtended() {
            this.extended.isEmpty() ? this.bg &= -268435457 : this.bg |= 268435456;
          }
          getUnderlineColor() {
            if (268435456 & this.bg && ~this.extended.underlineColor) switch (50331648 & this.extended.underlineColor) {
              case 16777216:
              case 33554432:
                return 255 & this.extended.underlineColor;
              case 50331648:
                return 16777215 & this.extended.underlineColor;
              default:
                return this.getFgColor();
            }
            return this.getFgColor();
          }
          getUnderlineColorMode() {
            return 268435456 & this.bg && ~this.extended.underlineColor ? 50331648 & this.extended.underlineColor : this.getFgColorMode();
          }
          isUnderlineColorRGB() {
            return 268435456 & this.bg && ~this.extended.underlineColor ? (50331648 & this.extended.underlineColor) == 50331648 : this.isFgRGB();
          }
          isUnderlineColorPalette() {
            return 268435456 & this.bg && ~this.extended.underlineColor ? (50331648 & this.extended.underlineColor) == 16777216 || (50331648 & this.extended.underlineColor) == 33554432 : this.isFgPalette();
          }
          isUnderlineColorDefault() {
            return 268435456 & this.bg && ~this.extended.underlineColor ? (50331648 & this.extended.underlineColor) == 0 : this.isFgDefault();
          }
          getUnderlineStyle() {
            return 268435456 & this.fg ? 268435456 & this.bg ? this.extended.underlineStyle : 1 : 0;
          }
          getUnderlineVariantOffset() {
            return this.extended.underlineVariantOffset;
          }
        }
        t.AttributeData = l;
        class d {
          get ext() {
            return this._urlId ? -469762049 & this._ext | this.underlineStyle << 26 : this._ext;
          }
          set ext(n) {
            this._ext = n;
          }
          get underlineStyle() {
            return this._urlId ? 5 : (469762048 & this._ext) >> 26;
          }
          set underlineStyle(n) {
            this._ext &= -469762049, this._ext |= n << 26 & 469762048;
          }
          get underlineColor() {
            return 67108863 & this._ext;
          }
          set underlineColor(n) {
            this._ext &= -67108864, this._ext |= 67108863 & n;
          }
          get urlId() {
            return this._urlId;
          }
          set urlId(n) {
            this._urlId = n;
          }
          get underlineVariantOffset() {
            const n = (3758096384 & this._ext) >> 29;
            return n < 0 ? 4294967288 ^ n : n;
          }
          set underlineVariantOffset(n) {
            this._ext &= 536870911, this._ext |= n << 29 & 3758096384;
          }
          constructor(n = 0, c = 0) {
            this._ext = 0, this._urlId = 0, this._ext = n, this._urlId = c;
          }
          clone() {
            return new d(this._ext, this._urlId);
          }
          isEmpty() {
            return this.underlineStyle === 0 && this._urlId === 0;
          }
        }
        t.ExtendedAttrs = d;
      }, 92: (w, t, l) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.Buffer = t.MAX_BUFFER_SIZE = void 0;
        const d = l(349), _ = l(226), n = l(734), c = l(437), a = l(634), h = l(511), u = l(643), e = l(863), r = l(116);
        t.MAX_BUFFER_SIZE = 4294967295, t.Buffer = class {
          constructor(i, s, f) {
            this._hasScrollback = i, this._optionsService = s, this._bufferService = f, this.ydisp = 0, this.ybase = 0, this.y = 0, this.x = 0, this.tabs = {}, this.savedY = 0, this.savedX = 0, this.savedCurAttrData = c.DEFAULT_ATTR_DATA.clone(), this.savedCharset = r.DEFAULT_CHARSET, this.markers = [], this._nullCell = h.CellData.fromCharData([0, u.NULL_CELL_CHAR, u.NULL_CELL_WIDTH, u.NULL_CELL_CODE]), this._whitespaceCell = h.CellData.fromCharData([0, u.WHITESPACE_CELL_CHAR, u.WHITESPACE_CELL_WIDTH, u.WHITESPACE_CELL_CODE]), this._isClearing = !1, this._memoryCleanupQueue = new _.IdleTaskQueue(), this._memoryCleanupPosition = 0, this._cols = this._bufferService.cols, this._rows = this._bufferService.rows, this.lines = new d.CircularList(this._getCorrectBufferLength(this._rows)), this.scrollTop = 0, this.scrollBottom = this._rows - 1, this.setupTabStops();
          }
          getNullCell(i) {
            return i ? (this._nullCell.fg = i.fg, this._nullCell.bg = i.bg, this._nullCell.extended = i.extended) : (this._nullCell.fg = 0, this._nullCell.bg = 0, this._nullCell.extended = new n.ExtendedAttrs()), this._nullCell;
          }
          getWhitespaceCell(i) {
            return i ? (this._whitespaceCell.fg = i.fg, this._whitespaceCell.bg = i.bg, this._whitespaceCell.extended = i.extended) : (this._whitespaceCell.fg = 0, this._whitespaceCell.bg = 0, this._whitespaceCell.extended = new n.ExtendedAttrs()), this._whitespaceCell;
          }
          getBlankLine(i, s) {
            return new c.BufferLine(this._bufferService.cols, this.getNullCell(i), s);
          }
          get hasScrollback() {
            return this._hasScrollback && this.lines.maxLength > this._rows;
          }
          get isCursorInViewport() {
            const i = this.ybase + this.y - this.ydisp;
            return i >= 0 && i < this._rows;
          }
          _getCorrectBufferLength(i) {
            if (!this._hasScrollback) return i;
            const s = i + this._optionsService.rawOptions.scrollback;
            return s > t.MAX_BUFFER_SIZE ? t.MAX_BUFFER_SIZE : s;
          }
          fillViewportRows(i) {
            if (this.lines.length === 0) {
              i === void 0 && (i = c.DEFAULT_ATTR_DATA);
              let s = this._rows;
              for (; s--; ) this.lines.push(this.getBlankLine(i));
            }
          }
          clear() {
            this.ydisp = 0, this.ybase = 0, this.y = 0, this.x = 0, this.lines = new d.CircularList(this._getCorrectBufferLength(this._rows)), this.scrollTop = 0, this.scrollBottom = this._rows - 1, this.setupTabStops();
          }
          resize(i, s) {
            const f = this.getNullCell(c.DEFAULT_ATTR_DATA);
            let p = 0;
            const S = this._getCorrectBufferLength(s);
            if (S > this.lines.maxLength && (this.lines.maxLength = S), this.lines.length > 0) {
              if (this._cols < i) for (let C = 0; C < this.lines.length; C++) p += +this.lines.get(C).resize(i, f);
              let y = 0;
              if (this._rows < s) for (let C = this._rows; C < s; C++) this.lines.length < s + this.ybase && (this._optionsService.rawOptions.windowsMode || this._optionsService.rawOptions.windowsPty.backend !== void 0 || this._optionsService.rawOptions.windowsPty.buildNumber !== void 0 ? this.lines.push(new c.BufferLine(i, f)) : this.ybase > 0 && this.lines.length <= this.ybase + this.y + y + 1 ? (this.ybase--, y++, this.ydisp > 0 && this.ydisp--) : this.lines.push(new c.BufferLine(i, f)));
              else for (let C = this._rows; C > s; C--) this.lines.length > s + this.ybase && (this.lines.length > this.ybase + this.y + 1 ? this.lines.pop() : (this.ybase++, this.ydisp++));
              if (S < this.lines.maxLength) {
                const C = this.lines.length - S;
                C > 0 && (this.lines.trimStart(C), this.ybase = Math.max(this.ybase - C, 0), this.ydisp = Math.max(this.ydisp - C, 0), this.savedY = Math.max(this.savedY - C, 0)), this.lines.maxLength = S;
              }
              this.x = Math.min(this.x, i - 1), this.y = Math.min(this.y, s - 1), y && (this.y += y), this.savedX = Math.min(this.savedX, i - 1), this.scrollTop = 0;
            }
            if (this.scrollBottom = s - 1, this._isReflowEnabled && (this._reflow(i, s), this._cols > i)) for (let y = 0; y < this.lines.length; y++) p += +this.lines.get(y).resize(i, f);
            this._cols = i, this._rows = s, this._memoryCleanupQueue.clear(), p > 0.1 * this.lines.length && (this._memoryCleanupPosition = 0, this._memoryCleanupQueue.enqueue(() => this._batchedMemoryCleanup()));
          }
          _batchedMemoryCleanup() {
            let i = !0;
            this._memoryCleanupPosition >= this.lines.length && (this._memoryCleanupPosition = 0, i = !1);
            let s = 0;
            for (; this._memoryCleanupPosition < this.lines.length; ) if (s += this.lines.get(this._memoryCleanupPosition++).cleanupMemory(), s > 100) return !0;
            return i;
          }
          get _isReflowEnabled() {
            const i = this._optionsService.rawOptions.windowsPty;
            return i && i.buildNumber ? this._hasScrollback && i.backend === "conpty" && i.buildNumber >= 21376 : this._hasScrollback && !this._optionsService.rawOptions.windowsMode;
          }
          _reflow(i, s) {
            this._cols !== i && (i > this._cols ? this._reflowLarger(i, s) : this._reflowSmaller(i, s));
          }
          _reflowLarger(i, s) {
            const f = (0, a.reflowLargerGetLinesToRemove)(this.lines, this._cols, i, this.ybase + this.y, this.getNullCell(c.DEFAULT_ATTR_DATA));
            if (f.length > 0) {
              const p = (0, a.reflowLargerCreateNewLayout)(this.lines, f);
              (0, a.reflowLargerApplyNewLayout)(this.lines, p.layout), this._reflowLargerAdjustViewport(i, s, p.countRemoved);
            }
          }
          _reflowLargerAdjustViewport(i, s, f) {
            const p = this.getNullCell(c.DEFAULT_ATTR_DATA);
            let S = f;
            for (; S-- > 0; ) this.ybase === 0 ? (this.y > 0 && this.y--, this.lines.length < s && this.lines.push(new c.BufferLine(i, p))) : (this.ydisp === this.ybase && this.ydisp--, this.ybase--);
            this.savedY = Math.max(this.savedY - f, 0);
          }
          _reflowSmaller(i, s) {
            const f = this.getNullCell(c.DEFAULT_ATTR_DATA), p = [];
            let S = 0;
            for (let y = this.lines.length - 1; y >= 0; y--) {
              let C = this.lines.get(y);
              if (!C || !C.isWrapped && C.getTrimmedLength() <= i) continue;
              const T = [C];
              for (; C.isWrapped && y > 0; ) C = this.lines.get(--y), T.unshift(C);
              const $ = this.ybase + this.y;
              if ($ >= y && $ < y + T.length) continue;
              const E = T[T.length - 1].getTrimmedLength(), B = (0, a.reflowSmallerGetNewLineLengths)(T, this._cols, i), D = B.length - T.length;
              let G;
              G = this.ybase === 0 && this.y !== this.lines.length - 1 ? Math.max(0, this.y - this.lines.maxLength + D) : Math.max(0, this.lines.length - this.lines.maxLength + D);
              const V = [];
              for (let g = 0; g < D; g++) {
                const b = this.getBlankLine(c.DEFAULT_ATTR_DATA, !0);
                V.push(b);
              }
              V.length > 0 && (p.push({ start: y + T.length + S, newLines: V }), S += V.length), T.push(...V);
              let U = B.length - 1, X = B[U];
              X === 0 && (U--, X = B[U]);
              let H = T.length - D - 1, o = E;
              for (; H >= 0; ) {
                const g = Math.min(o, X);
                if (T[U] === void 0) break;
                if (T[U].copyCellsFrom(T[H], o - g, X - g, g, !0), X -= g, X === 0 && (U--, X = B[U]), o -= g, o === 0) {
                  H--;
                  const b = Math.max(H, 0);
                  o = (0, a.getWrappedLineTrimmedLength)(T, b, this._cols);
                }
              }
              for (let g = 0; g < T.length; g++) B[g] < i && T[g].setCell(B[g], f);
              let m = D - G;
              for (; m-- > 0; ) this.ybase === 0 ? this.y < s - 1 ? (this.y++, this.lines.pop()) : (this.ybase++, this.ydisp++) : this.ybase < Math.min(this.lines.maxLength, this.lines.length + S) - s && (this.ybase === this.ydisp && this.ydisp++, this.ybase++);
              this.savedY = Math.min(this.savedY + D, this.ybase + s - 1);
            }
            if (p.length > 0) {
              const y = [], C = [];
              for (let U = 0; U < this.lines.length; U++) C.push(this.lines.get(U));
              const T = this.lines.length;
              let $ = T - 1, E = 0, B = p[E];
              this.lines.length = Math.min(this.lines.maxLength, this.lines.length + S);
              let D = 0;
              for (let U = Math.min(this.lines.maxLength - 1, T + S - 1); U >= 0; U--) if (B && B.start > $ + D) {
                for (let X = B.newLines.length - 1; X >= 0; X--) this.lines.set(U--, B.newLines[X]);
                U++, y.push({ index: $ + 1, amount: B.newLines.length }), D += B.newLines.length, B = p[++E];
              } else this.lines.set(U, C[$--]);
              let G = 0;
              for (let U = y.length - 1; U >= 0; U--) y[U].index += G, this.lines.onInsertEmitter.fire(y[U]), G += y[U].amount;
              const V = Math.max(0, T + S - this.lines.maxLength);
              V > 0 && this.lines.onTrimEmitter.fire(V);
            }
          }
          translateBufferLineToString(i, s, f = 0, p) {
            const S = this.lines.get(i);
            return S ? S.translateToString(s, f, p) : "";
          }
          getWrappedRangeForLine(i) {
            let s = i, f = i;
            for (; s > 0 && this.lines.get(s).isWrapped; ) s--;
            for (; f + 1 < this.lines.length && this.lines.get(f + 1).isWrapped; ) f++;
            return { first: s, last: f };
          }
          setupTabStops(i) {
            for (i != null ? this.tabs[i] || (i = this.prevStop(i)) : (this.tabs = {}, i = 0); i < this._cols; i += this._optionsService.rawOptions.tabStopWidth) this.tabs[i] = !0;
          }
          prevStop(i) {
            for (i == null && (i = this.x); !this.tabs[--i] && i > 0; ) ;
            return i >= this._cols ? this._cols - 1 : i < 0 ? 0 : i;
          }
          nextStop(i) {
            for (i == null && (i = this.x); !this.tabs[++i] && i < this._cols; ) ;
            return i >= this._cols ? this._cols - 1 : i < 0 ? 0 : i;
          }
          clearMarkers(i) {
            this._isClearing = !0;
            for (let s = 0; s < this.markers.length; s++) this.markers[s].line === i && (this.markers[s].dispose(), this.markers.splice(s--, 1));
            this._isClearing = !1;
          }
          clearAllMarkers() {
            this._isClearing = !0;
            for (let i = 0; i < this.markers.length; i++) this.markers[i].dispose(), this.markers.splice(i--, 1);
            this._isClearing = !1;
          }
          addMarker(i) {
            const s = new e.Marker(i);
            return this.markers.push(s), s.register(this.lines.onTrim((f) => {
              s.line -= f, s.line < 0 && s.dispose();
            })), s.register(this.lines.onInsert((f) => {
              s.line >= f.index && (s.line += f.amount);
            })), s.register(this.lines.onDelete((f) => {
              s.line >= f.index && s.line < f.index + f.amount && s.dispose(), s.line > f.index && (s.line -= f.amount);
            })), s.register(s.onDispose(() => this._removeMarker(s))), s;
          }
          _removeMarker(i) {
            this._isClearing || this.markers.splice(this.markers.indexOf(i), 1);
          }
        };
      }, 437: (w, t, l) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.BufferLine = t.DEFAULT_ATTR_DATA = void 0;
        const d = l(734), _ = l(511), n = l(643), c = l(482);
        t.DEFAULT_ATTR_DATA = Object.freeze(new d.AttributeData());
        let a = 0;
        class h {
          constructor(e, r, i = !1) {
            this.isWrapped = i, this._combined = {}, this._extendedAttrs = {}, this._data = new Uint32Array(3 * e);
            const s = r || _.CellData.fromCharData([0, n.NULL_CELL_CHAR, n.NULL_CELL_WIDTH, n.NULL_CELL_CODE]);
            for (let f = 0; f < e; ++f) this.setCell(f, s);
            this.length = e;
          }
          get(e) {
            const r = this._data[3 * e + 0], i = 2097151 & r;
            return [this._data[3 * e + 1], 2097152 & r ? this._combined[e] : i ? (0, c.stringFromCodePoint)(i) : "", r >> 22, 2097152 & r ? this._combined[e].charCodeAt(this._combined[e].length - 1) : i];
          }
          set(e, r) {
            this._data[3 * e + 1] = r[n.CHAR_DATA_ATTR_INDEX], r[n.CHAR_DATA_CHAR_INDEX].length > 1 ? (this._combined[e] = r[1], this._data[3 * e + 0] = 2097152 | e | r[n.CHAR_DATA_WIDTH_INDEX] << 22) : this._data[3 * e + 0] = r[n.CHAR_DATA_CHAR_INDEX].charCodeAt(0) | r[n.CHAR_DATA_WIDTH_INDEX] << 22;
          }
          getWidth(e) {
            return this._data[3 * e + 0] >> 22;
          }
          hasWidth(e) {
            return 12582912 & this._data[3 * e + 0];
          }
          getFg(e) {
            return this._data[3 * e + 1];
          }
          getBg(e) {
            return this._data[3 * e + 2];
          }
          hasContent(e) {
            return 4194303 & this._data[3 * e + 0];
          }
          getCodePoint(e) {
            const r = this._data[3 * e + 0];
            return 2097152 & r ? this._combined[e].charCodeAt(this._combined[e].length - 1) : 2097151 & r;
          }
          isCombined(e) {
            return 2097152 & this._data[3 * e + 0];
          }
          getString(e) {
            const r = this._data[3 * e + 0];
            return 2097152 & r ? this._combined[e] : 2097151 & r ? (0, c.stringFromCodePoint)(2097151 & r) : "";
          }
          isProtected(e) {
            return 536870912 & this._data[3 * e + 2];
          }
          loadCell(e, r) {
            return a = 3 * e, r.content = this._data[a + 0], r.fg = this._data[a + 1], r.bg = this._data[a + 2], 2097152 & r.content && (r.combinedData = this._combined[e]), 268435456 & r.bg && (r.extended = this._extendedAttrs[e]), r;
          }
          setCell(e, r) {
            2097152 & r.content && (this._combined[e] = r.combinedData), 268435456 & r.bg && (this._extendedAttrs[e] = r.extended), this._data[3 * e + 0] = r.content, this._data[3 * e + 1] = r.fg, this._data[3 * e + 2] = r.bg;
          }
          setCellFromCodepoint(e, r, i, s) {
            268435456 & s.bg && (this._extendedAttrs[e] = s.extended), this._data[3 * e + 0] = r | i << 22, this._data[3 * e + 1] = s.fg, this._data[3 * e + 2] = s.bg;
          }
          addCodepointToCell(e, r, i) {
            let s = this._data[3 * e + 0];
            2097152 & s ? this._combined[e] += (0, c.stringFromCodePoint)(r) : 2097151 & s ? (this._combined[e] = (0, c.stringFromCodePoint)(2097151 & s) + (0, c.stringFromCodePoint)(r), s &= -2097152, s |= 2097152) : s = r | 4194304, i && (s &= -12582913, s |= i << 22), this._data[3 * e + 0] = s;
          }
          insertCells(e, r, i) {
            if ((e %= this.length) && this.getWidth(e - 1) === 2 && this.setCellFromCodepoint(e - 1, 0, 1, i), r < this.length - e) {
              const s = new _.CellData();
              for (let f = this.length - e - r - 1; f >= 0; --f) this.setCell(e + r + f, this.loadCell(e + f, s));
              for (let f = 0; f < r; ++f) this.setCell(e + f, i);
            } else for (let s = e; s < this.length; ++s) this.setCell(s, i);
            this.getWidth(this.length - 1) === 2 && this.setCellFromCodepoint(this.length - 1, 0, 1, i);
          }
          deleteCells(e, r, i) {
            if (e %= this.length, r < this.length - e) {
              const s = new _.CellData();
              for (let f = 0; f < this.length - e - r; ++f) this.setCell(e + f, this.loadCell(e + r + f, s));
              for (let f = this.length - r; f < this.length; ++f) this.setCell(f, i);
            } else for (let s = e; s < this.length; ++s) this.setCell(s, i);
            e && this.getWidth(e - 1) === 2 && this.setCellFromCodepoint(e - 1, 0, 1, i), this.getWidth(e) !== 0 || this.hasContent(e) || this.setCellFromCodepoint(e, 0, 1, i);
          }
          replaceCells(e, r, i, s = !1) {
            if (s) for (e && this.getWidth(e - 1) === 2 && !this.isProtected(e - 1) && this.setCellFromCodepoint(e - 1, 0, 1, i), r < this.length && this.getWidth(r - 1) === 2 && !this.isProtected(r) && this.setCellFromCodepoint(r, 0, 1, i); e < r && e < this.length; ) this.isProtected(e) || this.setCell(e, i), e++;
            else for (e && this.getWidth(e - 1) === 2 && this.setCellFromCodepoint(e - 1, 0, 1, i), r < this.length && this.getWidth(r - 1) === 2 && this.setCellFromCodepoint(r, 0, 1, i); e < r && e < this.length; ) this.setCell(e++, i);
          }
          resize(e, r) {
            if (e === this.length) return 4 * this._data.length * 2 < this._data.buffer.byteLength;
            const i = 3 * e;
            if (e > this.length) {
              if (this._data.buffer.byteLength >= 4 * i) this._data = new Uint32Array(this._data.buffer, 0, i);
              else {
                const s = new Uint32Array(i);
                s.set(this._data), this._data = s;
              }
              for (let s = this.length; s < e; ++s) this.setCell(s, r);
            } else {
              this._data = this._data.subarray(0, i);
              const s = Object.keys(this._combined);
              for (let p = 0; p < s.length; p++) {
                const S = parseInt(s[p], 10);
                S >= e && delete this._combined[S];
              }
              const f = Object.keys(this._extendedAttrs);
              for (let p = 0; p < f.length; p++) {
                const S = parseInt(f[p], 10);
                S >= e && delete this._extendedAttrs[S];
              }
            }
            return this.length = e, 4 * i * 2 < this._data.buffer.byteLength;
          }
          cleanupMemory() {
            if (4 * this._data.length * 2 < this._data.buffer.byteLength) {
              const e = new Uint32Array(this._data.length);
              return e.set(this._data), this._data = e, 1;
            }
            return 0;
          }
          fill(e, r = !1) {
            if (r) for (let i = 0; i < this.length; ++i) this.isProtected(i) || this.setCell(i, e);
            else {
              this._combined = {}, this._extendedAttrs = {};
              for (let i = 0; i < this.length; ++i) this.setCell(i, e);
            }
          }
          copyFrom(e) {
            this.length !== e.length ? this._data = new Uint32Array(e._data) : this._data.set(e._data), this.length = e.length, this._combined = {};
            for (const r in e._combined) this._combined[r] = e._combined[r];
            this._extendedAttrs = {};
            for (const r in e._extendedAttrs) this._extendedAttrs[r] = e._extendedAttrs[r];
            this.isWrapped = e.isWrapped;
          }
          clone() {
            const e = new h(0);
            e._data = new Uint32Array(this._data), e.length = this.length;
            for (const r in this._combined) e._combined[r] = this._combined[r];
            for (const r in this._extendedAttrs) e._extendedAttrs[r] = this._extendedAttrs[r];
            return e.isWrapped = this.isWrapped, e;
          }
          getTrimmedLength() {
            for (let e = this.length - 1; e >= 0; --e) if (4194303 & this._data[3 * e + 0]) return e + (this._data[3 * e + 0] >> 22);
            return 0;
          }
          getNoBgTrimmedLength() {
            for (let e = this.length - 1; e >= 0; --e) if (4194303 & this._data[3 * e + 0] || 50331648 & this._data[3 * e + 2]) return e + (this._data[3 * e + 0] >> 22);
            return 0;
          }
          copyCellsFrom(e, r, i, s, f) {
            const p = e._data;
            if (f) for (let y = s - 1; y >= 0; y--) {
              for (let C = 0; C < 3; C++) this._data[3 * (i + y) + C] = p[3 * (r + y) + C];
              268435456 & p[3 * (r + y) + 2] && (this._extendedAttrs[i + y] = e._extendedAttrs[r + y]);
            }
            else for (let y = 0; y < s; y++) {
              for (let C = 0; C < 3; C++) this._data[3 * (i + y) + C] = p[3 * (r + y) + C];
              268435456 & p[3 * (r + y) + 2] && (this._extendedAttrs[i + y] = e._extendedAttrs[r + y]);
            }
            const S = Object.keys(e._combined);
            for (let y = 0; y < S.length; y++) {
              const C = parseInt(S[y], 10);
              C >= r && (this._combined[C - r + i] = e._combined[C]);
            }
          }
          translateToString(e, r, i, s) {
            r = r ?? 0, i = i ?? this.length, e && (i = Math.min(i, this.getTrimmedLength())), s && (s.length = 0);
            let f = "";
            for (; r < i; ) {
              const p = this._data[3 * r + 0], S = 2097151 & p, y = 2097152 & p ? this._combined[r] : S ? (0, c.stringFromCodePoint)(S) : n.WHITESPACE_CELL_CHAR;
              if (f += y, s) for (let C = 0; C < y.length; ++C) s.push(r);
              r += p >> 22 || 1;
            }
            return s && s.push(r), f;
          }
        }
        t.BufferLine = h;
      }, 634: (w, t) => {
        function l(d, _, n) {
          if (_ === d.length - 1) return d[_].getTrimmedLength();
          const c = !d[_].hasContent(n - 1) && d[_].getWidth(n - 1) === 1, a = d[_ + 1].getWidth(0) === 2;
          return c && a ? n - 1 : n;
        }
        Object.defineProperty(t, "__esModule", { value: !0 }), t.getWrappedLineTrimmedLength = t.reflowSmallerGetNewLineLengths = t.reflowLargerApplyNewLayout = t.reflowLargerCreateNewLayout = t.reflowLargerGetLinesToRemove = void 0, t.reflowLargerGetLinesToRemove = function(d, _, n, c, a) {
          const h = [];
          for (let u = 0; u < d.length - 1; u++) {
            let e = u, r = d.get(++e);
            if (!r.isWrapped) continue;
            const i = [d.get(u)];
            for (; e < d.length && r.isWrapped; ) i.push(r), r = d.get(++e);
            if (c >= u && c < e) {
              u += i.length - 1;
              continue;
            }
            let s = 0, f = l(i, s, _), p = 1, S = 0;
            for (; p < i.length; ) {
              const C = l(i, p, _), T = C - S, $ = n - f, E = Math.min(T, $);
              i[s].copyCellsFrom(i[p], S, f, E, !1), f += E, f === n && (s++, f = 0), S += E, S === C && (p++, S = 0), f === 0 && s !== 0 && i[s - 1].getWidth(n - 1) === 2 && (i[s].copyCellsFrom(i[s - 1], n - 1, f++, 1, !1), i[s - 1].setCell(n - 1, a));
            }
            i[s].replaceCells(f, n, a);
            let y = 0;
            for (let C = i.length - 1; C > 0 && (C > s || i[C].getTrimmedLength() === 0); C--) y++;
            y > 0 && (h.push(u + i.length - y), h.push(y)), u += i.length - 1;
          }
          return h;
        }, t.reflowLargerCreateNewLayout = function(d, _) {
          const n = [];
          let c = 0, a = _[c], h = 0;
          for (let u = 0; u < d.length; u++) if (a === u) {
            const e = _[++c];
            d.onDeleteEmitter.fire({ index: u - h, amount: e }), u += e - 1, h += e, a = _[++c];
          } else n.push(u);
          return { layout: n, countRemoved: h };
        }, t.reflowLargerApplyNewLayout = function(d, _) {
          const n = [];
          for (let c = 0; c < _.length; c++) n.push(d.get(_[c]));
          for (let c = 0; c < n.length; c++) d.set(c, n[c]);
          d.length = _.length;
        }, t.reflowSmallerGetNewLineLengths = function(d, _, n) {
          const c = [], a = d.map((r, i) => l(d, i, _)).reduce((r, i) => r + i);
          let h = 0, u = 0, e = 0;
          for (; e < a; ) {
            if (a - e < n) {
              c.push(a - e);
              break;
            }
            h += n;
            const r = l(d, u, _);
            h > r && (h -= r, u++);
            const i = d[u].getWidth(h - 1) === 2;
            i && h--;
            const s = i ? n - 1 : n;
            c.push(s), e += s;
          }
          return c;
        }, t.getWrappedLineTrimmedLength = l;
      }, 295: (w, t, l) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.BufferSet = void 0;
        const d = l(460), _ = l(844), n = l(92);
        class c extends _.Disposable {
          constructor(h, u) {
            super(), this._optionsService = h, this._bufferService = u, this._onBufferActivate = this.register(new d.EventEmitter()), this.onBufferActivate = this._onBufferActivate.event, this.reset(), this.register(this._optionsService.onSpecificOptionChange("scrollback", () => this.resize(this._bufferService.cols, this._bufferService.rows))), this.register(this._optionsService.onSpecificOptionChange("tabStopWidth", () => this.setupTabStops()));
          }
          reset() {
            this._normal = new n.Buffer(!0, this._optionsService, this._bufferService), this._normal.fillViewportRows(), this._alt = new n.Buffer(!1, this._optionsService, this._bufferService), this._activeBuffer = this._normal, this._onBufferActivate.fire({ activeBuffer: this._normal, inactiveBuffer: this._alt }), this.setupTabStops();
          }
          get alt() {
            return this._alt;
          }
          get active() {
            return this._activeBuffer;
          }
          get normal() {
            return this._normal;
          }
          activateNormalBuffer() {
            this._activeBuffer !== this._normal && (this._normal.x = this._alt.x, this._normal.y = this._alt.y, this._alt.clearAllMarkers(), this._alt.clear(), this._activeBuffer = this._normal, this._onBufferActivate.fire({ activeBuffer: this._normal, inactiveBuffer: this._alt }));
          }
          activateAltBuffer(h) {
            this._activeBuffer !== this._alt && (this._alt.fillViewportRows(h), this._alt.x = this._normal.x, this._alt.y = this._normal.y, this._activeBuffer = this._alt, this._onBufferActivate.fire({ activeBuffer: this._alt, inactiveBuffer: this._normal }));
          }
          resize(h, u) {
            this._normal.resize(h, u), this._alt.resize(h, u), this.setupTabStops(h);
          }
          setupTabStops(h) {
            this._normal.setupTabStops(h), this._alt.setupTabStops(h);
          }
        }
        t.BufferSet = c;
      }, 511: (w, t, l) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.CellData = void 0;
        const d = l(482), _ = l(643), n = l(734);
        class c extends n.AttributeData {
          constructor() {
            super(...arguments), this.content = 0, this.fg = 0, this.bg = 0, this.extended = new n.ExtendedAttrs(), this.combinedData = "";
          }
          static fromCharData(h) {
            const u = new c();
            return u.setFromCharData(h), u;
          }
          isCombined() {
            return 2097152 & this.content;
          }
          getWidth() {
            return this.content >> 22;
          }
          getChars() {
            return 2097152 & this.content ? this.combinedData : 2097151 & this.content ? (0, d.stringFromCodePoint)(2097151 & this.content) : "";
          }
          getCode() {
            return this.isCombined() ? this.combinedData.charCodeAt(this.combinedData.length - 1) : 2097151 & this.content;
          }
          setFromCharData(h) {
            this.fg = h[_.CHAR_DATA_ATTR_INDEX], this.bg = 0;
            let u = !1;
            if (h[_.CHAR_DATA_CHAR_INDEX].length > 2) u = !0;
            else if (h[_.CHAR_DATA_CHAR_INDEX].length === 2) {
              const e = h[_.CHAR_DATA_CHAR_INDEX].charCodeAt(0);
              if (55296 <= e && e <= 56319) {
                const r = h[_.CHAR_DATA_CHAR_INDEX].charCodeAt(1);
                56320 <= r && r <= 57343 ? this.content = 1024 * (e - 55296) + r - 56320 + 65536 | h[_.CHAR_DATA_WIDTH_INDEX] << 22 : u = !0;
              } else u = !0;
            } else this.content = h[_.CHAR_DATA_CHAR_INDEX].charCodeAt(0) | h[_.CHAR_DATA_WIDTH_INDEX] << 22;
            u && (this.combinedData = h[_.CHAR_DATA_CHAR_INDEX], this.content = 2097152 | h[_.CHAR_DATA_WIDTH_INDEX] << 22);
          }
          getAsCharData() {
            return [this.fg, this.getChars(), this.getWidth(), this.getCode()];
          }
        }
        t.CellData = c;
      }, 643: (w, t) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.WHITESPACE_CELL_CODE = t.WHITESPACE_CELL_WIDTH = t.WHITESPACE_CELL_CHAR = t.NULL_CELL_CODE = t.NULL_CELL_WIDTH = t.NULL_CELL_CHAR = t.CHAR_DATA_CODE_INDEX = t.CHAR_DATA_WIDTH_INDEX = t.CHAR_DATA_CHAR_INDEX = t.CHAR_DATA_ATTR_INDEX = t.DEFAULT_EXT = t.DEFAULT_ATTR = t.DEFAULT_COLOR = void 0, t.DEFAULT_COLOR = 0, t.DEFAULT_ATTR = 256 | t.DEFAULT_COLOR << 9, t.DEFAULT_EXT = 0, t.CHAR_DATA_ATTR_INDEX = 0, t.CHAR_DATA_CHAR_INDEX = 1, t.CHAR_DATA_WIDTH_INDEX = 2, t.CHAR_DATA_CODE_INDEX = 3, t.NULL_CELL_CHAR = "", t.NULL_CELL_WIDTH = 1, t.NULL_CELL_CODE = 0, t.WHITESPACE_CELL_CHAR = " ", t.WHITESPACE_CELL_WIDTH = 1, t.WHITESPACE_CELL_CODE = 32;
      }, 863: (w, t, l) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.Marker = void 0;
        const d = l(460), _ = l(844);
        class n {
          get id() {
            return this._id;
          }
          constructor(a) {
            this.line = a, this.isDisposed = !1, this._disposables = [], this._id = n._nextId++, this._onDispose = this.register(new d.EventEmitter()), this.onDispose = this._onDispose.event;
          }
          dispose() {
            this.isDisposed || (this.isDisposed = !0, this.line = -1, this._onDispose.fire(), (0, _.disposeArray)(this._disposables), this._disposables.length = 0);
          }
          register(a) {
            return this._disposables.push(a), a;
          }
        }
        t.Marker = n, n._nextId = 1;
      }, 116: (w, t) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.DEFAULT_CHARSET = t.CHARSETS = void 0, t.CHARSETS = {}, t.DEFAULT_CHARSET = t.CHARSETS.B, t.CHARSETS[0] = { "`": "◆", a: "▒", b: "␉", c: "␌", d: "␍", e: "␊", f: "°", g: "±", h: "␤", i: "␋", j: "┘", k: "┐", l: "┌", m: "└", n: "┼", o: "⎺", p: "⎻", q: "─", r: "⎼", s: "⎽", t: "├", u: "┤", v: "┴", w: "┬", x: "│", y: "≤", z: "≥", "{": "π", "|": "≠", "}": "£", "~": "·" }, t.CHARSETS.A = { "#": "£" }, t.CHARSETS.B = void 0, t.CHARSETS[4] = { "#": "£", "@": "¾", "[": "ij", "\\": "½", "]": "|", "{": "¨", "|": "f", "}": "¼", "~": "´" }, t.CHARSETS.C = t.CHARSETS[5] = { "[": "Ä", "\\": "Ö", "]": "Å", "^": "Ü", "`": "é", "{": "ä", "|": "ö", "}": "å", "~": "ü" }, t.CHARSETS.R = { "#": "£", "@": "à", "[": "°", "\\": "ç", "]": "§", "{": "é", "|": "ù", "}": "è", "~": "¨" }, t.CHARSETS.Q = { "@": "à", "[": "â", "\\": "ç", "]": "ê", "^": "î", "`": "ô", "{": "é", "|": "ù", "}": "è", "~": "û" }, t.CHARSETS.K = { "@": "§", "[": "Ä", "\\": "Ö", "]": "Ü", "{": "ä", "|": "ö", "}": "ü", "~": "ß" }, t.CHARSETS.Y = { "#": "£", "@": "§", "[": "°", "\\": "ç", "]": "é", "`": "ù", "{": "à", "|": "ò", "}": "è", "~": "ì" }, t.CHARSETS.E = t.CHARSETS[6] = { "@": "Ä", "[": "Æ", "\\": "Ø", "]": "Å", "^": "Ü", "`": "ä", "{": "æ", "|": "ø", "}": "å", "~": "ü" }, t.CHARSETS.Z = { "#": "£", "@": "§", "[": "¡", "\\": "Ñ", "]": "¿", "{": "°", "|": "ñ", "}": "ç" }, t.CHARSETS.H = t.CHARSETS[7] = { "@": "É", "[": "Ä", "\\": "Ö", "]": "Å", "^": "Ü", "`": "é", "{": "ä", "|": "ö", "}": "å", "~": "ü" }, t.CHARSETS["="] = { "#": "ù", "@": "à", "[": "é", "\\": "ç", "]": "ê", "^": "î", _: "è", "`": "ô", "{": "ä", "|": "ö", "}": "ü", "~": "û" };
      }, 584: (w, t) => {
        var l, d, _;
        Object.defineProperty(t, "__esModule", { value: !0 }), t.C1_ESCAPED = t.C1 = t.C0 = void 0, function(n) {
          n.NUL = "\0", n.SOH = "", n.STX = "", n.ETX = "", n.EOT = "", n.ENQ = "", n.ACK = "", n.BEL = "\x07", n.BS = "\b", n.HT = "	", n.LF = `
`, n.VT = "\v", n.FF = "\f", n.CR = "\r", n.SO = "", n.SI = "", n.DLE = "", n.DC1 = "", n.DC2 = "", n.DC3 = "", n.DC4 = "", n.NAK = "", n.SYN = "", n.ETB = "", n.CAN = "", n.EM = "", n.SUB = "", n.ESC = "\x1B", n.FS = "", n.GS = "", n.RS = "", n.US = "", n.SP = " ", n.DEL = "";
        }(l || (t.C0 = l = {})), function(n) {
          n.PAD = "", n.HOP = "", n.BPH = "", n.NBH = "", n.IND = "", n.NEL = "", n.SSA = "", n.ESA = "", n.HTS = "", n.HTJ = "", n.VTS = "", n.PLD = "", n.PLU = "", n.RI = "", n.SS2 = "", n.SS3 = "", n.DCS = "", n.PU1 = "", n.PU2 = "", n.STS = "", n.CCH = "", n.MW = "", n.SPA = "", n.EPA = "", n.SOS = "", n.SGCI = "", n.SCI = "", n.CSI = "", n.ST = "", n.OSC = "", n.PM = "", n.APC = "";
        }(d || (t.C1 = d = {})), function(n) {
          n.ST = `${l.ESC}\\`;
        }(_ || (t.C1_ESCAPED = _ = {}));
      }, 482: (w, t) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.Utf8ToUtf32 = t.StringToUtf32 = t.utf32ToString = t.stringFromCodePoint = void 0, t.stringFromCodePoint = function(l) {
          return l > 65535 ? (l -= 65536, String.fromCharCode(55296 + (l >> 10)) + String.fromCharCode(l % 1024 + 56320)) : String.fromCharCode(l);
        }, t.utf32ToString = function(l, d = 0, _ = l.length) {
          let n = "";
          for (let c = d; c < _; ++c) {
            let a = l[c];
            a > 65535 ? (a -= 65536, n += String.fromCharCode(55296 + (a >> 10)) + String.fromCharCode(a % 1024 + 56320)) : n += String.fromCharCode(a);
          }
          return n;
        }, t.StringToUtf32 = class {
          constructor() {
            this._interim = 0;
          }
          clear() {
            this._interim = 0;
          }
          decode(l, d) {
            const _ = l.length;
            if (!_) return 0;
            let n = 0, c = 0;
            if (this._interim) {
              const a = l.charCodeAt(c++);
              56320 <= a && a <= 57343 ? d[n++] = 1024 * (this._interim - 55296) + a - 56320 + 65536 : (d[n++] = this._interim, d[n++] = a), this._interim = 0;
            }
            for (let a = c; a < _; ++a) {
              const h = l.charCodeAt(a);
              if (55296 <= h && h <= 56319) {
                if (++a >= _) return this._interim = h, n;
                const u = l.charCodeAt(a);
                56320 <= u && u <= 57343 ? d[n++] = 1024 * (h - 55296) + u - 56320 + 65536 : (d[n++] = h, d[n++] = u);
              } else h !== 65279 && (d[n++] = h);
            }
            return n;
          }
        }, t.Utf8ToUtf32 = class {
          constructor() {
            this.interim = new Uint8Array(3);
          }
          clear() {
            this.interim.fill(0);
          }
          decode(l, d) {
            const _ = l.length;
            if (!_) return 0;
            let n, c, a, h, u = 0, e = 0, r = 0;
            if (this.interim[0]) {
              let f = !1, p = this.interim[0];
              p &= (224 & p) == 192 ? 31 : (240 & p) == 224 ? 15 : 7;
              let S, y = 0;
              for (; (S = 63 & this.interim[++y]) && y < 4; ) p <<= 6, p |= S;
              const C = (224 & this.interim[0]) == 192 ? 2 : (240 & this.interim[0]) == 224 ? 3 : 4, T = C - y;
              for (; r < T; ) {
                if (r >= _) return 0;
                if (S = l[r++], (192 & S) != 128) {
                  r--, f = !0;
                  break;
                }
                this.interim[y++] = S, p <<= 6, p |= 63 & S;
              }
              f || (C === 2 ? p < 128 ? r-- : d[u++] = p : C === 3 ? p < 2048 || p >= 55296 && p <= 57343 || p === 65279 || (d[u++] = p) : p < 65536 || p > 1114111 || (d[u++] = p)), this.interim.fill(0);
            }
            const i = _ - 4;
            let s = r;
            for (; s < _; ) {
              for (; !(!(s < i) || 128 & (n = l[s]) || 128 & (c = l[s + 1]) || 128 & (a = l[s + 2]) || 128 & (h = l[s + 3])); ) d[u++] = n, d[u++] = c, d[u++] = a, d[u++] = h, s += 4;
              if (n = l[s++], n < 128) d[u++] = n;
              else if ((224 & n) == 192) {
                if (s >= _) return this.interim[0] = n, u;
                if (c = l[s++], (192 & c) != 128) {
                  s--;
                  continue;
                }
                if (e = (31 & n) << 6 | 63 & c, e < 128) {
                  s--;
                  continue;
                }
                d[u++] = e;
              } else if ((240 & n) == 224) {
                if (s >= _) return this.interim[0] = n, u;
                if (c = l[s++], (192 & c) != 128) {
                  s--;
                  continue;
                }
                if (s >= _) return this.interim[0] = n, this.interim[1] = c, u;
                if (a = l[s++], (192 & a) != 128) {
                  s--;
                  continue;
                }
                if (e = (15 & n) << 12 | (63 & c) << 6 | 63 & a, e < 2048 || e >= 55296 && e <= 57343 || e === 65279) continue;
                d[u++] = e;
              } else if ((248 & n) == 240) {
                if (s >= _) return this.interim[0] = n, u;
                if (c = l[s++], (192 & c) != 128) {
                  s--;
                  continue;
                }
                if (s >= _) return this.interim[0] = n, this.interim[1] = c, u;
                if (a = l[s++], (192 & a) != 128) {
                  s--;
                  continue;
                }
                if (s >= _) return this.interim[0] = n, this.interim[1] = c, this.interim[2] = a, u;
                if (h = l[s++], (192 & h) != 128) {
                  s--;
                  continue;
                }
                if (e = (7 & n) << 18 | (63 & c) << 12 | (63 & a) << 6 | 63 & h, e < 65536 || e > 1114111) continue;
                d[u++] = e;
              }
            }
            return u;
          }
        };
      }, 225: (w, t, l) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.UnicodeV6 = void 0;
        const d = l(480), _ = [[768, 879], [1155, 1158], [1160, 1161], [1425, 1469], [1471, 1471], [1473, 1474], [1476, 1477], [1479, 1479], [1536, 1539], [1552, 1557], [1611, 1630], [1648, 1648], [1750, 1764], [1767, 1768], [1770, 1773], [1807, 1807], [1809, 1809], [1840, 1866], [1958, 1968], [2027, 2035], [2305, 2306], [2364, 2364], [2369, 2376], [2381, 2381], [2385, 2388], [2402, 2403], [2433, 2433], [2492, 2492], [2497, 2500], [2509, 2509], [2530, 2531], [2561, 2562], [2620, 2620], [2625, 2626], [2631, 2632], [2635, 2637], [2672, 2673], [2689, 2690], [2748, 2748], [2753, 2757], [2759, 2760], [2765, 2765], [2786, 2787], [2817, 2817], [2876, 2876], [2879, 2879], [2881, 2883], [2893, 2893], [2902, 2902], [2946, 2946], [3008, 3008], [3021, 3021], [3134, 3136], [3142, 3144], [3146, 3149], [3157, 3158], [3260, 3260], [3263, 3263], [3270, 3270], [3276, 3277], [3298, 3299], [3393, 3395], [3405, 3405], [3530, 3530], [3538, 3540], [3542, 3542], [3633, 3633], [3636, 3642], [3655, 3662], [3761, 3761], [3764, 3769], [3771, 3772], [3784, 3789], [3864, 3865], [3893, 3893], [3895, 3895], [3897, 3897], [3953, 3966], [3968, 3972], [3974, 3975], [3984, 3991], [3993, 4028], [4038, 4038], [4141, 4144], [4146, 4146], [4150, 4151], [4153, 4153], [4184, 4185], [4448, 4607], [4959, 4959], [5906, 5908], [5938, 5940], [5970, 5971], [6002, 6003], [6068, 6069], [6071, 6077], [6086, 6086], [6089, 6099], [6109, 6109], [6155, 6157], [6313, 6313], [6432, 6434], [6439, 6440], [6450, 6450], [6457, 6459], [6679, 6680], [6912, 6915], [6964, 6964], [6966, 6970], [6972, 6972], [6978, 6978], [7019, 7027], [7616, 7626], [7678, 7679], [8203, 8207], [8234, 8238], [8288, 8291], [8298, 8303], [8400, 8431], [12330, 12335], [12441, 12442], [43014, 43014], [43019, 43019], [43045, 43046], [64286, 64286], [65024, 65039], [65056, 65059], [65279, 65279], [65529, 65531]], n = [[68097, 68099], [68101, 68102], [68108, 68111], [68152, 68154], [68159, 68159], [119143, 119145], [119155, 119170], [119173, 119179], [119210, 119213], [119362, 119364], [917505, 917505], [917536, 917631], [917760, 917999]];
        let c;
        t.UnicodeV6 = class {
          constructor() {
            if (this.version = "6", !c) {
              c = new Uint8Array(65536), c.fill(1), c[0] = 0, c.fill(0, 1, 32), c.fill(0, 127, 160), c.fill(2, 4352, 4448), c[9001] = 2, c[9002] = 2, c.fill(2, 11904, 42192), c[12351] = 1, c.fill(2, 44032, 55204), c.fill(2, 63744, 64256), c.fill(2, 65040, 65050), c.fill(2, 65072, 65136), c.fill(2, 65280, 65377), c.fill(2, 65504, 65511);
              for (let a = 0; a < _.length; ++a) c.fill(0, _[a][0], _[a][1] + 1);
            }
          }
          wcwidth(a) {
            return a < 32 ? 0 : a < 127 ? 1 : a < 65536 ? c[a] : function(h, u) {
              let e, r = 0, i = u.length - 1;
              if (h < u[0][0] || h > u[i][1]) return !1;
              for (; i >= r; ) if (e = r + i >> 1, h > u[e][1]) r = e + 1;
              else {
                if (!(h < u[e][0])) return !0;
                i = e - 1;
              }
              return !1;
            }(a, n) ? 0 : a >= 131072 && a <= 196605 || a >= 196608 && a <= 262141 ? 2 : 1;
          }
          charProperties(a, h) {
            let u = this.wcwidth(a), e = u === 0 && h !== 0;
            if (e) {
              const r = d.UnicodeService.extractWidth(h);
              r === 0 ? e = !1 : r > u && (u = r);
            }
            return d.UnicodeService.createPropertyValue(0, u, e);
          }
        };
      }, 981: (w, t, l) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.WriteBuffer = void 0;
        const d = l(460), _ = l(844);
        class n extends _.Disposable {
          constructor(a) {
            super(), this._action = a, this._writeBuffer = [], this._callbacks = [], this._pendingData = 0, this._bufferOffset = 0, this._isSyncWriting = !1, this._syncCalls = 0, this._didUserInput = !1, this._onWriteParsed = this.register(new d.EventEmitter()), this.onWriteParsed = this._onWriteParsed.event;
          }
          handleUserInput() {
            this._didUserInput = !0;
          }
          writeSync(a, h) {
            if (h !== void 0 && this._syncCalls > h) return void (this._syncCalls = 0);
            if (this._pendingData += a.length, this._writeBuffer.push(a), this._callbacks.push(void 0), this._syncCalls++, this._isSyncWriting) return;
            let u;
            for (this._isSyncWriting = !0; u = this._writeBuffer.shift(); ) {
              this._action(u);
              const e = this._callbacks.shift();
              e && e();
            }
            this._pendingData = 0, this._bufferOffset = 2147483647, this._isSyncWriting = !1, this._syncCalls = 0;
          }
          write(a, h) {
            if (this._pendingData > 5e7) throw new Error("write data discarded, use flow control to avoid losing data");
            if (!this._writeBuffer.length) {
              if (this._bufferOffset = 0, this._didUserInput) return this._didUserInput = !1, this._pendingData += a.length, this._writeBuffer.push(a), this._callbacks.push(h), void this._innerWrite();
              setTimeout(() => this._innerWrite());
            }
            this._pendingData += a.length, this._writeBuffer.push(a), this._callbacks.push(h);
          }
          _innerWrite(a = 0, h = !0) {
            const u = a || Date.now();
            for (; this._writeBuffer.length > this._bufferOffset; ) {
              const e = this._writeBuffer[this._bufferOffset], r = this._action(e, h);
              if (r) {
                const s = (f) => Date.now() - u >= 12 ? setTimeout(() => this._innerWrite(0, f)) : this._innerWrite(u, f);
                return void r.catch((f) => (queueMicrotask(() => {
                  throw f;
                }), Promise.resolve(!1))).then(s);
              }
              const i = this._callbacks[this._bufferOffset];
              if (i && i(), this._bufferOffset++, this._pendingData -= e.length, Date.now() - u >= 12) break;
            }
            this._writeBuffer.length > this._bufferOffset ? (this._bufferOffset > 50 && (this._writeBuffer = this._writeBuffer.slice(this._bufferOffset), this._callbacks = this._callbacks.slice(this._bufferOffset), this._bufferOffset = 0), setTimeout(() => this._innerWrite())) : (this._writeBuffer.length = 0, this._callbacks.length = 0, this._pendingData = 0, this._bufferOffset = 0), this._onWriteParsed.fire();
          }
        }
        t.WriteBuffer = n;
      }, 941: (w, t) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.toRgbString = t.parseColor = void 0;
        const l = /^([\da-f])\/([\da-f])\/([\da-f])$|^([\da-f]{2})\/([\da-f]{2})\/([\da-f]{2})$|^([\da-f]{3})\/([\da-f]{3})\/([\da-f]{3})$|^([\da-f]{4})\/([\da-f]{4})\/([\da-f]{4})$/, d = /^[\da-f]+$/;
        function _(n, c) {
          const a = n.toString(16), h = a.length < 2 ? "0" + a : a;
          switch (c) {
            case 4:
              return a[0];
            case 8:
              return h;
            case 12:
              return (h + h).slice(0, 3);
            default:
              return h + h;
          }
        }
        t.parseColor = function(n) {
          if (!n) return;
          let c = n.toLowerCase();
          if (c.indexOf("rgb:") === 0) {
            c = c.slice(4);
            const a = l.exec(c);
            if (a) {
              const h = a[1] ? 15 : a[4] ? 255 : a[7] ? 4095 : 65535;
              return [Math.round(parseInt(a[1] || a[4] || a[7] || a[10], 16) / h * 255), Math.round(parseInt(a[2] || a[5] || a[8] || a[11], 16) / h * 255), Math.round(parseInt(a[3] || a[6] || a[9] || a[12], 16) / h * 255)];
            }
          } else if (c.indexOf("#") === 0 && (c = c.slice(1), d.exec(c) && [3, 6, 9, 12].includes(c.length))) {
            const a = c.length / 3, h = [0, 0, 0];
            for (let u = 0; u < 3; ++u) {
              const e = parseInt(c.slice(a * u, a * u + a), 16);
              h[u] = a === 1 ? e << 4 : a === 2 ? e : a === 3 ? e >> 4 : e >> 8;
            }
            return h;
          }
        }, t.toRgbString = function(n, c = 16) {
          const [a, h, u] = n;
          return `rgb:${_(a, c)}/${_(h, c)}/${_(u, c)}`;
        };
      }, 770: (w, t) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.PAYLOAD_LIMIT = void 0, t.PAYLOAD_LIMIT = 1e7;
      }, 351: (w, t, l) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.DcsHandler = t.DcsParser = void 0;
        const d = l(482), _ = l(742), n = l(770), c = [];
        t.DcsParser = class {
          constructor() {
            this._handlers = /* @__PURE__ */ Object.create(null), this._active = c, this._ident = 0, this._handlerFb = () => {
            }, this._stack = { paused: !1, loopPosition: 0, fallThrough: !1 };
          }
          dispose() {
            this._handlers = /* @__PURE__ */ Object.create(null), this._handlerFb = () => {
            }, this._active = c;
          }
          registerHandler(h, u) {
            this._handlers[h] === void 0 && (this._handlers[h] = []);
            const e = this._handlers[h];
            return e.push(u), { dispose: () => {
              const r = e.indexOf(u);
              r !== -1 && e.splice(r, 1);
            } };
          }
          clearHandler(h) {
            this._handlers[h] && delete this._handlers[h];
          }
          setHandlerFallback(h) {
            this._handlerFb = h;
          }
          reset() {
            if (this._active.length) for (let h = this._stack.paused ? this._stack.loopPosition - 1 : this._active.length - 1; h >= 0; --h) this._active[h].unhook(!1);
            this._stack.paused = !1, this._active = c, this._ident = 0;
          }
          hook(h, u) {
            if (this.reset(), this._ident = h, this._active = this._handlers[h] || c, this._active.length) for (let e = this._active.length - 1; e >= 0; e--) this._active[e].hook(u);
            else this._handlerFb(this._ident, "HOOK", u);
          }
          put(h, u, e) {
            if (this._active.length) for (let r = this._active.length - 1; r >= 0; r--) this._active[r].put(h, u, e);
            else this._handlerFb(this._ident, "PUT", (0, d.utf32ToString)(h, u, e));
          }
          unhook(h, u = !0) {
            if (this._active.length) {
              let e = !1, r = this._active.length - 1, i = !1;
              if (this._stack.paused && (r = this._stack.loopPosition - 1, e = u, i = this._stack.fallThrough, this._stack.paused = !1), !i && e === !1) {
                for (; r >= 0 && (e = this._active[r].unhook(h), e !== !0); r--) if (e instanceof Promise) return this._stack.paused = !0, this._stack.loopPosition = r, this._stack.fallThrough = !1, e;
                r--;
              }
              for (; r >= 0; r--) if (e = this._active[r].unhook(!1), e instanceof Promise) return this._stack.paused = !0, this._stack.loopPosition = r, this._stack.fallThrough = !0, e;
            } else this._handlerFb(this._ident, "UNHOOK", h);
            this._active = c, this._ident = 0;
          }
        };
        const a = new _.Params();
        a.addParam(0), t.DcsHandler = class {
          constructor(h) {
            this._handler = h, this._data = "", this._params = a, this._hitLimit = !1;
          }
          hook(h) {
            this._params = h.length > 1 || h.params[0] ? h.clone() : a, this._data = "", this._hitLimit = !1;
          }
          put(h, u, e) {
            this._hitLimit || (this._data += (0, d.utf32ToString)(h, u, e), this._data.length > n.PAYLOAD_LIMIT && (this._data = "", this._hitLimit = !0));
          }
          unhook(h) {
            let u = !1;
            if (this._hitLimit) u = !1;
            else if (h && (u = this._handler(this._data, this._params), u instanceof Promise)) return u.then((e) => (this._params = a, this._data = "", this._hitLimit = !1, e));
            return this._params = a, this._data = "", this._hitLimit = !1, u;
          }
        };
      }, 15: (w, t, l) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.EscapeSequenceParser = t.VT500_TRANSITION_TABLE = t.TransitionTable = void 0;
        const d = l(844), _ = l(742), n = l(242), c = l(351);
        class a {
          constructor(r) {
            this.table = new Uint8Array(r);
          }
          setDefault(r, i) {
            this.table.fill(r << 4 | i);
          }
          add(r, i, s, f) {
            this.table[i << 8 | r] = s << 4 | f;
          }
          addMany(r, i, s, f) {
            for (let p = 0; p < r.length; p++) this.table[i << 8 | r[p]] = s << 4 | f;
          }
        }
        t.TransitionTable = a;
        const h = 160;
        t.VT500_TRANSITION_TABLE = function() {
          const e = new a(4095), r = Array.apply(null, Array(256)).map((y, C) => C), i = (y, C) => r.slice(y, C), s = i(32, 127), f = i(0, 24);
          f.push(25), f.push.apply(f, i(28, 32));
          const p = i(0, 14);
          let S;
          for (S in e.setDefault(1, 0), e.addMany(s, 0, 2, 0), p) e.addMany([24, 26, 153, 154], S, 3, 0), e.addMany(i(128, 144), S, 3, 0), e.addMany(i(144, 152), S, 3, 0), e.add(156, S, 0, 0), e.add(27, S, 11, 1), e.add(157, S, 4, 8), e.addMany([152, 158, 159], S, 0, 7), e.add(155, S, 11, 3), e.add(144, S, 11, 9);
          return e.addMany(f, 0, 3, 0), e.addMany(f, 1, 3, 1), e.add(127, 1, 0, 1), e.addMany(f, 8, 0, 8), e.addMany(f, 3, 3, 3), e.add(127, 3, 0, 3), e.addMany(f, 4, 3, 4), e.add(127, 4, 0, 4), e.addMany(f, 6, 3, 6), e.addMany(f, 5, 3, 5), e.add(127, 5, 0, 5), e.addMany(f, 2, 3, 2), e.add(127, 2, 0, 2), e.add(93, 1, 4, 8), e.addMany(s, 8, 5, 8), e.add(127, 8, 5, 8), e.addMany([156, 27, 24, 26, 7], 8, 6, 0), e.addMany(i(28, 32), 8, 0, 8), e.addMany([88, 94, 95], 1, 0, 7), e.addMany(s, 7, 0, 7), e.addMany(f, 7, 0, 7), e.add(156, 7, 0, 0), e.add(127, 7, 0, 7), e.add(91, 1, 11, 3), e.addMany(i(64, 127), 3, 7, 0), e.addMany(i(48, 60), 3, 8, 4), e.addMany([60, 61, 62, 63], 3, 9, 4), e.addMany(i(48, 60), 4, 8, 4), e.addMany(i(64, 127), 4, 7, 0), e.addMany([60, 61, 62, 63], 4, 0, 6), e.addMany(i(32, 64), 6, 0, 6), e.add(127, 6, 0, 6), e.addMany(i(64, 127), 6, 0, 0), e.addMany(i(32, 48), 3, 9, 5), e.addMany(i(32, 48), 5, 9, 5), e.addMany(i(48, 64), 5, 0, 6), e.addMany(i(64, 127), 5, 7, 0), e.addMany(i(32, 48), 4, 9, 5), e.addMany(i(32, 48), 1, 9, 2), e.addMany(i(32, 48), 2, 9, 2), e.addMany(i(48, 127), 2, 10, 0), e.addMany(i(48, 80), 1, 10, 0), e.addMany(i(81, 88), 1, 10, 0), e.addMany([89, 90, 92], 1, 10, 0), e.addMany(i(96, 127), 1, 10, 0), e.add(80, 1, 11, 9), e.addMany(f, 9, 0, 9), e.add(127, 9, 0, 9), e.addMany(i(28, 32), 9, 0, 9), e.addMany(i(32, 48), 9, 9, 12), e.addMany(i(48, 60), 9, 8, 10), e.addMany([60, 61, 62, 63], 9, 9, 10), e.addMany(f, 11, 0, 11), e.addMany(i(32, 128), 11, 0, 11), e.addMany(i(28, 32), 11, 0, 11), e.addMany(f, 10, 0, 10), e.add(127, 10, 0, 10), e.addMany(i(28, 32), 10, 0, 10), e.addMany(i(48, 60), 10, 8, 10), e.addMany([60, 61, 62, 63], 10, 0, 11), e.addMany(i(32, 48), 10, 9, 12), e.addMany(f, 12, 0, 12), e.add(127, 12, 0, 12), e.addMany(i(28, 32), 12, 0, 12), e.addMany(i(32, 48), 12, 9, 12), e.addMany(i(48, 64), 12, 0, 11), e.addMany(i(64, 127), 12, 12, 13), e.addMany(i(64, 127), 10, 12, 13), e.addMany(i(64, 127), 9, 12, 13), e.addMany(f, 13, 13, 13), e.addMany(s, 13, 13, 13), e.add(127, 13, 0, 13), e.addMany([27, 156, 24, 26], 13, 14, 0), e.add(h, 0, 2, 0), e.add(h, 8, 5, 8), e.add(h, 6, 0, 6), e.add(h, 11, 0, 11), e.add(h, 13, 13, 13), e;
        }();
        class u extends d.Disposable {
          constructor(r = t.VT500_TRANSITION_TABLE) {
            super(), this._transitions = r, this._parseStack = { state: 0, handlers: [], handlerPos: 0, transition: 0, chunkPos: 0 }, this.initialState = 0, this.currentState = this.initialState, this._params = new _.Params(), this._params.addParam(0), this._collect = 0, this.precedingJoinState = 0, this._printHandlerFb = (i, s, f) => {
            }, this._executeHandlerFb = (i) => {
            }, this._csiHandlerFb = (i, s) => {
            }, this._escHandlerFb = (i) => {
            }, this._errorHandlerFb = (i) => i, this._printHandler = this._printHandlerFb, this._executeHandlers = /* @__PURE__ */ Object.create(null), this._csiHandlers = /* @__PURE__ */ Object.create(null), this._escHandlers = /* @__PURE__ */ Object.create(null), this.register((0, d.toDisposable)(() => {
              this._csiHandlers = /* @__PURE__ */ Object.create(null), this._executeHandlers = /* @__PURE__ */ Object.create(null), this._escHandlers = /* @__PURE__ */ Object.create(null);
            })), this._oscParser = this.register(new n.OscParser()), this._dcsParser = this.register(new c.DcsParser()), this._errorHandler = this._errorHandlerFb, this.registerEscHandler({ final: "\\" }, () => !0);
          }
          _identifier(r, i = [64, 126]) {
            let s = 0;
            if (r.prefix) {
              if (r.prefix.length > 1) throw new Error("only one byte as prefix supported");
              if (s = r.prefix.charCodeAt(0), s && 60 > s || s > 63) throw new Error("prefix must be in range 0x3c .. 0x3f");
            }
            if (r.intermediates) {
              if (r.intermediates.length > 2) throw new Error("only two bytes as intermediates are supported");
              for (let p = 0; p < r.intermediates.length; ++p) {
                const S = r.intermediates.charCodeAt(p);
                if (32 > S || S > 47) throw new Error("intermediate must be in range 0x20 .. 0x2f");
                s <<= 8, s |= S;
              }
            }
            if (r.final.length !== 1) throw new Error("final must be a single byte");
            const f = r.final.charCodeAt(0);
            if (i[0] > f || f > i[1]) throw new Error(`final must be in range ${i[0]} .. ${i[1]}`);
            return s <<= 8, s |= f, s;
          }
          identToString(r) {
            const i = [];
            for (; r; ) i.push(String.fromCharCode(255 & r)), r >>= 8;
            return i.reverse().join("");
          }
          setPrintHandler(r) {
            this._printHandler = r;
          }
          clearPrintHandler() {
            this._printHandler = this._printHandlerFb;
          }
          registerEscHandler(r, i) {
            const s = this._identifier(r, [48, 126]);
            this._escHandlers[s] === void 0 && (this._escHandlers[s] = []);
            const f = this._escHandlers[s];
            return f.push(i), { dispose: () => {
              const p = f.indexOf(i);
              p !== -1 && f.splice(p, 1);
            } };
          }
          clearEscHandler(r) {
            this._escHandlers[this._identifier(r, [48, 126])] && delete this._escHandlers[this._identifier(r, [48, 126])];
          }
          setEscHandlerFallback(r) {
            this._escHandlerFb = r;
          }
          setExecuteHandler(r, i) {
            this._executeHandlers[r.charCodeAt(0)] = i;
          }
          clearExecuteHandler(r) {
            this._executeHandlers[r.charCodeAt(0)] && delete this._executeHandlers[r.charCodeAt(0)];
          }
          setExecuteHandlerFallback(r) {
            this._executeHandlerFb = r;
          }
          registerCsiHandler(r, i) {
            const s = this._identifier(r);
            this._csiHandlers[s] === void 0 && (this._csiHandlers[s] = []);
            const f = this._csiHandlers[s];
            return f.push(i), { dispose: () => {
              const p = f.indexOf(i);
              p !== -1 && f.splice(p, 1);
            } };
          }
          clearCsiHandler(r) {
            this._csiHandlers[this._identifier(r)] && delete this._csiHandlers[this._identifier(r)];
          }
          setCsiHandlerFallback(r) {
            this._csiHandlerFb = r;
          }
          registerDcsHandler(r, i) {
            return this._dcsParser.registerHandler(this._identifier(r), i);
          }
          clearDcsHandler(r) {
            this._dcsParser.clearHandler(this._identifier(r));
          }
          setDcsHandlerFallback(r) {
            this._dcsParser.setHandlerFallback(r);
          }
          registerOscHandler(r, i) {
            return this._oscParser.registerHandler(r, i);
          }
          clearOscHandler(r) {
            this._oscParser.clearHandler(r);
          }
          setOscHandlerFallback(r) {
            this._oscParser.setHandlerFallback(r);
          }
          setErrorHandler(r) {
            this._errorHandler = r;
          }
          clearErrorHandler() {
            this._errorHandler = this._errorHandlerFb;
          }
          reset() {
            this.currentState = this.initialState, this._oscParser.reset(), this._dcsParser.reset(), this._params.reset(), this._params.addParam(0), this._collect = 0, this.precedingJoinState = 0, this._parseStack.state !== 0 && (this._parseStack.state = 2, this._parseStack.handlers = []);
          }
          _preserveStack(r, i, s, f, p) {
            this._parseStack.state = r, this._parseStack.handlers = i, this._parseStack.handlerPos = s, this._parseStack.transition = f, this._parseStack.chunkPos = p;
          }
          parse(r, i, s) {
            let f, p = 0, S = 0, y = 0;
            if (this._parseStack.state) if (this._parseStack.state === 2) this._parseStack.state = 0, y = this._parseStack.chunkPos + 1;
            else {
              if (s === void 0 || this._parseStack.state === 1) throw this._parseStack.state = 1, new Error("improper continuation due to previous async handler, giving up parsing");
              const C = this._parseStack.handlers;
              let T = this._parseStack.handlerPos - 1;
              switch (this._parseStack.state) {
                case 3:
                  if (s === !1 && T > -1) {
                    for (; T >= 0 && (f = C[T](this._params), f !== !0); T--) if (f instanceof Promise) return this._parseStack.handlerPos = T, f;
                  }
                  this._parseStack.handlers = [];
                  break;
                case 4:
                  if (s === !1 && T > -1) {
                    for (; T >= 0 && (f = C[T](), f !== !0); T--) if (f instanceof Promise) return this._parseStack.handlerPos = T, f;
                  }
                  this._parseStack.handlers = [];
                  break;
                case 6:
                  if (p = r[this._parseStack.chunkPos], f = this._dcsParser.unhook(p !== 24 && p !== 26, s), f) return f;
                  p === 27 && (this._parseStack.transition |= 1), this._params.reset(), this._params.addParam(0), this._collect = 0;
                  break;
                case 5:
                  if (p = r[this._parseStack.chunkPos], f = this._oscParser.end(p !== 24 && p !== 26, s), f) return f;
                  p === 27 && (this._parseStack.transition |= 1), this._params.reset(), this._params.addParam(0), this._collect = 0;
              }
              this._parseStack.state = 0, y = this._parseStack.chunkPos + 1, this.precedingJoinState = 0, this.currentState = 15 & this._parseStack.transition;
            }
            for (let C = y; C < i; ++C) {
              switch (p = r[C], S = this._transitions.table[this.currentState << 8 | (p < 160 ? p : h)], S >> 4) {
                case 2:
                  for (let D = C + 1; ; ++D) {
                    if (D >= i || (p = r[D]) < 32 || p > 126 && p < h) {
                      this._printHandler(r, C, D), C = D - 1;
                      break;
                    }
                    if (++D >= i || (p = r[D]) < 32 || p > 126 && p < h) {
                      this._printHandler(r, C, D), C = D - 1;
                      break;
                    }
                    if (++D >= i || (p = r[D]) < 32 || p > 126 && p < h) {
                      this._printHandler(r, C, D), C = D - 1;
                      break;
                    }
                    if (++D >= i || (p = r[D]) < 32 || p > 126 && p < h) {
                      this._printHandler(r, C, D), C = D - 1;
                      break;
                    }
                  }
                  break;
                case 3:
                  this._executeHandlers[p] ? this._executeHandlers[p]() : this._executeHandlerFb(p), this.precedingJoinState = 0;
                  break;
                case 0:
                  break;
                case 1:
                  if (this._errorHandler({ position: C, code: p, currentState: this.currentState, collect: this._collect, params: this._params, abort: !1 }).abort) return;
                  break;
                case 7:
                  const T = this._csiHandlers[this._collect << 8 | p];
                  let $ = T ? T.length - 1 : -1;
                  for (; $ >= 0 && (f = T[$](this._params), f !== !0); $--) if (f instanceof Promise) return this._preserveStack(3, T, $, S, C), f;
                  $ < 0 && this._csiHandlerFb(this._collect << 8 | p, this._params), this.precedingJoinState = 0;
                  break;
                case 8:
                  do
                    switch (p) {
                      case 59:
                        this._params.addParam(0);
                        break;
                      case 58:
                        this._params.addSubParam(-1);
                        break;
                      default:
                        this._params.addDigit(p - 48);
                    }
                  while (++C < i && (p = r[C]) > 47 && p < 60);
                  C--;
                  break;
                case 9:
                  this._collect <<= 8, this._collect |= p;
                  break;
                case 10:
                  const E = this._escHandlers[this._collect << 8 | p];
                  let B = E ? E.length - 1 : -1;
                  for (; B >= 0 && (f = E[B](), f !== !0); B--) if (f instanceof Promise) return this._preserveStack(4, E, B, S, C), f;
                  B < 0 && this._escHandlerFb(this._collect << 8 | p), this.precedingJoinState = 0;
                  break;
                case 11:
                  this._params.reset(), this._params.addParam(0), this._collect = 0;
                  break;
                case 12:
                  this._dcsParser.hook(this._collect << 8 | p, this._params);
                  break;
                case 13:
                  for (let D = C + 1; ; ++D) if (D >= i || (p = r[D]) === 24 || p === 26 || p === 27 || p > 127 && p < h) {
                    this._dcsParser.put(r, C, D), C = D - 1;
                    break;
                  }
                  break;
                case 14:
                  if (f = this._dcsParser.unhook(p !== 24 && p !== 26), f) return this._preserveStack(6, [], 0, S, C), f;
                  p === 27 && (S |= 1), this._params.reset(), this._params.addParam(0), this._collect = 0, this.precedingJoinState = 0;
                  break;
                case 4:
                  this._oscParser.start();
                  break;
                case 5:
                  for (let D = C + 1; ; D++) if (D >= i || (p = r[D]) < 32 || p > 127 && p < h) {
                    this._oscParser.put(r, C, D), C = D - 1;
                    break;
                  }
                  break;
                case 6:
                  if (f = this._oscParser.end(p !== 24 && p !== 26), f) return this._preserveStack(5, [], 0, S, C), f;
                  p === 27 && (S |= 1), this._params.reset(), this._params.addParam(0), this._collect = 0, this.precedingJoinState = 0;
              }
              this.currentState = 15 & S;
            }
          }
        }
        t.EscapeSequenceParser = u;
      }, 242: (w, t, l) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.OscHandler = t.OscParser = void 0;
        const d = l(770), _ = l(482), n = [];
        t.OscParser = class {
          constructor() {
            this._state = 0, this._active = n, this._id = -1, this._handlers = /* @__PURE__ */ Object.create(null), this._handlerFb = () => {
            }, this._stack = { paused: !1, loopPosition: 0, fallThrough: !1 };
          }
          registerHandler(c, a) {
            this._handlers[c] === void 0 && (this._handlers[c] = []);
            const h = this._handlers[c];
            return h.push(a), { dispose: () => {
              const u = h.indexOf(a);
              u !== -1 && h.splice(u, 1);
            } };
          }
          clearHandler(c) {
            this._handlers[c] && delete this._handlers[c];
          }
          setHandlerFallback(c) {
            this._handlerFb = c;
          }
          dispose() {
            this._handlers = /* @__PURE__ */ Object.create(null), this._handlerFb = () => {
            }, this._active = n;
          }
          reset() {
            if (this._state === 2) for (let c = this._stack.paused ? this._stack.loopPosition - 1 : this._active.length - 1; c >= 0; --c) this._active[c].end(!1);
            this._stack.paused = !1, this._active = n, this._id = -1, this._state = 0;
          }
          _start() {
            if (this._active = this._handlers[this._id] || n, this._active.length) for (let c = this._active.length - 1; c >= 0; c--) this._active[c].start();
            else this._handlerFb(this._id, "START");
          }
          _put(c, a, h) {
            if (this._active.length) for (let u = this._active.length - 1; u >= 0; u--) this._active[u].put(c, a, h);
            else this._handlerFb(this._id, "PUT", (0, _.utf32ToString)(c, a, h));
          }
          start() {
            this.reset(), this._state = 1;
          }
          put(c, a, h) {
            if (this._state !== 3) {
              if (this._state === 1) for (; a < h; ) {
                const u = c[a++];
                if (u === 59) {
                  this._state = 2, this._start();
                  break;
                }
                if (u < 48 || 57 < u) return void (this._state = 3);
                this._id === -1 && (this._id = 0), this._id = 10 * this._id + u - 48;
              }
              this._state === 2 && h - a > 0 && this._put(c, a, h);
            }
          }
          end(c, a = !0) {
            if (this._state !== 0) {
              if (this._state !== 3) if (this._state === 1 && this._start(), this._active.length) {
                let h = !1, u = this._active.length - 1, e = !1;
                if (this._stack.paused && (u = this._stack.loopPosition - 1, h = a, e = this._stack.fallThrough, this._stack.paused = !1), !e && h === !1) {
                  for (; u >= 0 && (h = this._active[u].end(c), h !== !0); u--) if (h instanceof Promise) return this._stack.paused = !0, this._stack.loopPosition = u, this._stack.fallThrough = !1, h;
                  u--;
                }
                for (; u >= 0; u--) if (h = this._active[u].end(!1), h instanceof Promise) return this._stack.paused = !0, this._stack.loopPosition = u, this._stack.fallThrough = !0, h;
              } else this._handlerFb(this._id, "END", c);
              this._active = n, this._id = -1, this._state = 0;
            }
          }
        }, t.OscHandler = class {
          constructor(c) {
            this._handler = c, this._data = "", this._hitLimit = !1;
          }
          start() {
            this._data = "", this._hitLimit = !1;
          }
          put(c, a, h) {
            this._hitLimit || (this._data += (0, _.utf32ToString)(c, a, h), this._data.length > d.PAYLOAD_LIMIT && (this._data = "", this._hitLimit = !0));
          }
          end(c) {
            let a = !1;
            if (this._hitLimit) a = !1;
            else if (c && (a = this._handler(this._data), a instanceof Promise)) return a.then((h) => (this._data = "", this._hitLimit = !1, h));
            return this._data = "", this._hitLimit = !1, a;
          }
        };
      }, 742: (w, t) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.Params = void 0;
        const l = 2147483647;
        class d {
          static fromArray(n) {
            const c = new d();
            if (!n.length) return c;
            for (let a = Array.isArray(n[0]) ? 1 : 0; a < n.length; ++a) {
              const h = n[a];
              if (Array.isArray(h)) for (let u = 0; u < h.length; ++u) c.addSubParam(h[u]);
              else c.addParam(h);
            }
            return c;
          }
          constructor(n = 32, c = 32) {
            if (this.maxLength = n, this.maxSubParamsLength = c, c > 256) throw new Error("maxSubParamsLength must not be greater than 256");
            this.params = new Int32Array(n), this.length = 0, this._subParams = new Int32Array(c), this._subParamsLength = 0, this._subParamsIdx = new Uint16Array(n), this._rejectDigits = !1, this._rejectSubDigits = !1, this._digitIsSub = !1;
          }
          clone() {
            const n = new d(this.maxLength, this.maxSubParamsLength);
            return n.params.set(this.params), n.length = this.length, n._subParams.set(this._subParams), n._subParamsLength = this._subParamsLength, n._subParamsIdx.set(this._subParamsIdx), n._rejectDigits = this._rejectDigits, n._rejectSubDigits = this._rejectSubDigits, n._digitIsSub = this._digitIsSub, n;
          }
          toArray() {
            const n = [];
            for (let c = 0; c < this.length; ++c) {
              n.push(this.params[c]);
              const a = this._subParamsIdx[c] >> 8, h = 255 & this._subParamsIdx[c];
              h - a > 0 && n.push(Array.prototype.slice.call(this._subParams, a, h));
            }
            return n;
          }
          reset() {
            this.length = 0, this._subParamsLength = 0, this._rejectDigits = !1, this._rejectSubDigits = !1, this._digitIsSub = !1;
          }
          addParam(n) {
            if (this._digitIsSub = !1, this.length >= this.maxLength) this._rejectDigits = !0;
            else {
              if (n < -1) throw new Error("values lesser than -1 are not allowed");
              this._subParamsIdx[this.length] = this._subParamsLength << 8 | this._subParamsLength, this.params[this.length++] = n > l ? l : n;
            }
          }
          addSubParam(n) {
            if (this._digitIsSub = !0, this.length) if (this._rejectDigits || this._subParamsLength >= this.maxSubParamsLength) this._rejectSubDigits = !0;
            else {
              if (n < -1) throw new Error("values lesser than -1 are not allowed");
              this._subParams[this._subParamsLength++] = n > l ? l : n, this._subParamsIdx[this.length - 1]++;
            }
          }
          hasSubParams(n) {
            return (255 & this._subParamsIdx[n]) - (this._subParamsIdx[n] >> 8) > 0;
          }
          getSubParams(n) {
            const c = this._subParamsIdx[n] >> 8, a = 255 & this._subParamsIdx[n];
            return a - c > 0 ? this._subParams.subarray(c, a) : null;
          }
          getSubParamsAll() {
            const n = {};
            for (let c = 0; c < this.length; ++c) {
              const a = this._subParamsIdx[c] >> 8, h = 255 & this._subParamsIdx[c];
              h - a > 0 && (n[c] = this._subParams.slice(a, h));
            }
            return n;
          }
          addDigit(n) {
            let c;
            if (this._rejectDigits || !(c = this._digitIsSub ? this._subParamsLength : this.length) || this._digitIsSub && this._rejectSubDigits) return;
            const a = this._digitIsSub ? this._subParams : this.params, h = a[c - 1];
            a[c - 1] = ~h ? Math.min(10 * h + n, l) : n;
          }
        }
        t.Params = d;
      }, 741: (w, t) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.AddonManager = void 0, t.AddonManager = class {
          constructor() {
            this._addons = [];
          }
          dispose() {
            for (let l = this._addons.length - 1; l >= 0; l--) this._addons[l].instance.dispose();
          }
          loadAddon(l, d) {
            const _ = { instance: d, dispose: d.dispose, isDisposed: !1 };
            this._addons.push(_), d.dispose = () => this._wrappedAddonDispose(_), d.activate(l);
          }
          _wrappedAddonDispose(l) {
            if (l.isDisposed) return;
            let d = -1;
            for (let _ = 0; _ < this._addons.length; _++) if (this._addons[_] === l) {
              d = _;
              break;
            }
            if (d === -1) throw new Error("Could not dispose an addon that has not been loaded");
            l.isDisposed = !0, l.dispose.apply(l.instance), this._addons.splice(d, 1);
          }
        };
      }, 771: (w, t, l) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.BufferApiView = void 0;
        const d = l(785), _ = l(511);
        t.BufferApiView = class {
          constructor(n, c) {
            this._buffer = n, this.type = c;
          }
          init(n) {
            return this._buffer = n, this;
          }
          get cursorY() {
            return this._buffer.y;
          }
          get cursorX() {
            return this._buffer.x;
          }
          get viewportY() {
            return this._buffer.ydisp;
          }
          get baseY() {
            return this._buffer.ybase;
          }
          get length() {
            return this._buffer.lines.length;
          }
          getLine(n) {
            const c = this._buffer.lines.get(n);
            if (c) return new d.BufferLineApiView(c);
          }
          getNullCell() {
            return new _.CellData();
          }
        };
      }, 785: (w, t, l) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.BufferLineApiView = void 0;
        const d = l(511);
        t.BufferLineApiView = class {
          constructor(_) {
            this._line = _;
          }
          get isWrapped() {
            return this._line.isWrapped;
          }
          get length() {
            return this._line.length;
          }
          getCell(_, n) {
            if (!(_ < 0 || _ >= this._line.length)) return n ? (this._line.loadCell(_, n), n) : this._line.loadCell(_, new d.CellData());
          }
          translateToString(_, n, c) {
            return this._line.translateToString(_, n, c);
          }
        };
      }, 285: (w, t, l) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.BufferNamespaceApi = void 0;
        const d = l(771), _ = l(460), n = l(844);
        class c extends n.Disposable {
          constructor(h) {
            super(), this._core = h, this._onBufferChange = this.register(new _.EventEmitter()), this.onBufferChange = this._onBufferChange.event, this._normal = new d.BufferApiView(this._core.buffers.normal, "normal"), this._alternate = new d.BufferApiView(this._core.buffers.alt, "alternate"), this._core.buffers.onBufferActivate(() => this._onBufferChange.fire(this.active));
          }
          get active() {
            if (this._core.buffers.active === this._core.buffers.normal) return this.normal;
            if (this._core.buffers.active === this._core.buffers.alt) return this.alternate;
            throw new Error("Active buffer is neither normal nor alternate");
          }
          get normal() {
            return this._normal.init(this._core.buffers.normal);
          }
          get alternate() {
            return this._alternate.init(this._core.buffers.alt);
          }
        }
        t.BufferNamespaceApi = c;
      }, 975: (w, t) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.ParserApi = void 0, t.ParserApi = class {
          constructor(l) {
            this._core = l;
          }
          registerCsiHandler(l, d) {
            return this._core.registerCsiHandler(l, (_) => d(_.toArray()));
          }
          addCsiHandler(l, d) {
            return this.registerCsiHandler(l, d);
          }
          registerDcsHandler(l, d) {
            return this._core.registerDcsHandler(l, (_, n) => d(_, n.toArray()));
          }
          addDcsHandler(l, d) {
            return this.registerDcsHandler(l, d);
          }
          registerEscHandler(l, d) {
            return this._core.registerEscHandler(l, d);
          }
          addEscHandler(l, d) {
            return this.registerEscHandler(l, d);
          }
          registerOscHandler(l, d) {
            return this._core.registerOscHandler(l, d);
          }
          addOscHandler(l, d) {
            return this.registerOscHandler(l, d);
          }
        };
      }, 90: (w, t) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.UnicodeApi = void 0, t.UnicodeApi = class {
          constructor(l) {
            this._core = l;
          }
          register(l) {
            this._core.unicodeService.register(l);
          }
          get versions() {
            return this._core.unicodeService.versions;
          }
          get activeVersion() {
            return this._core.unicodeService.activeVersion;
          }
          set activeVersion(l) {
            this._core.unicodeService.activeVersion = l;
          }
        };
      }, 744: function(w, t, l) {
        var d = this && this.__decorate || function(e, r, i, s) {
          var f, p = arguments.length, S = p < 3 ? r : s === null ? s = Object.getOwnPropertyDescriptor(r, i) : s;
          if (typeof Reflect == "object" && typeof Reflect.decorate == "function") S = Reflect.decorate(e, r, i, s);
          else for (var y = e.length - 1; y >= 0; y--) (f = e[y]) && (S = (p < 3 ? f(S) : p > 3 ? f(r, i, S) : f(r, i)) || S);
          return p > 3 && S && Object.defineProperty(r, i, S), S;
        }, _ = this && this.__param || function(e, r) {
          return function(i, s) {
            r(i, s, e);
          };
        };
        Object.defineProperty(t, "__esModule", { value: !0 }), t.BufferService = t.MINIMUM_ROWS = t.MINIMUM_COLS = void 0;
        const n = l(460), c = l(844), a = l(295), h = l(585);
        t.MINIMUM_COLS = 2, t.MINIMUM_ROWS = 1;
        let u = t.BufferService = class extends c.Disposable {
          get buffer() {
            return this.buffers.active;
          }
          constructor(e) {
            super(), this.isUserScrolling = !1, this._onResize = this.register(new n.EventEmitter()), this.onResize = this._onResize.event, this._onScroll = this.register(new n.EventEmitter()), this.onScroll = this._onScroll.event, this.cols = Math.max(e.rawOptions.cols || 0, t.MINIMUM_COLS), this.rows = Math.max(e.rawOptions.rows || 0, t.MINIMUM_ROWS), this.buffers = this.register(new a.BufferSet(e, this));
          }
          resize(e, r) {
            this.cols = e, this.rows = r, this.buffers.resize(e, r), this._onResize.fire({ cols: e, rows: r });
          }
          reset() {
            this.buffers.reset(), this.isUserScrolling = !1;
          }
          scroll(e, r = !1) {
            const i = this.buffer;
            let s;
            s = this._cachedBlankLine, s && s.length === this.cols && s.getFg(0) === e.fg && s.getBg(0) === e.bg || (s = i.getBlankLine(e, r), this._cachedBlankLine = s), s.isWrapped = r;
            const f = i.ybase + i.scrollTop, p = i.ybase + i.scrollBottom;
            if (i.scrollTop === 0) {
              const S = i.lines.isFull;
              p === i.lines.length - 1 ? S ? i.lines.recycle().copyFrom(s) : i.lines.push(s.clone()) : i.lines.splice(p + 1, 0, s.clone()), S ? this.isUserScrolling && (i.ydisp = Math.max(i.ydisp - 1, 0)) : (i.ybase++, this.isUserScrolling || i.ydisp++);
            } else {
              const S = p - f + 1;
              i.lines.shiftElements(f + 1, S - 1, -1), i.lines.set(p, s.clone());
            }
            this.isUserScrolling || (i.ydisp = i.ybase), this._onScroll.fire(i.ydisp);
          }
          scrollLines(e, r, i) {
            const s = this.buffer;
            if (e < 0) {
              if (s.ydisp === 0) return;
              this.isUserScrolling = !0;
            } else e + s.ydisp >= s.ybase && (this.isUserScrolling = !1);
            const f = s.ydisp;
            s.ydisp = Math.max(Math.min(s.ydisp + e, s.ybase), 0), f !== s.ydisp && (r || this._onScroll.fire(s.ydisp));
          }
        };
        t.BufferService = u = d([_(0, h.IOptionsService)], u);
      }, 994: (w, t) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.CharsetService = void 0, t.CharsetService = class {
          constructor() {
            this.glevel = 0, this._charsets = [];
          }
          reset() {
            this.charset = void 0, this._charsets = [], this.glevel = 0;
          }
          setgLevel(l) {
            this.glevel = l, this.charset = this._charsets[l];
          }
          setgCharset(l, d) {
            this._charsets[l] = d, this.glevel === l && (this.charset = d);
          }
        };
      }, 753: function(w, t, l) {
        var d = this && this.__decorate || function(s, f, p, S) {
          var y, C = arguments.length, T = C < 3 ? f : S === null ? S = Object.getOwnPropertyDescriptor(f, p) : S;
          if (typeof Reflect == "object" && typeof Reflect.decorate == "function") T = Reflect.decorate(s, f, p, S);
          else for (var $ = s.length - 1; $ >= 0; $--) (y = s[$]) && (T = (C < 3 ? y(T) : C > 3 ? y(f, p, T) : y(f, p)) || T);
          return C > 3 && T && Object.defineProperty(f, p, T), T;
        }, _ = this && this.__param || function(s, f) {
          return function(p, S) {
            f(p, S, s);
          };
        };
        Object.defineProperty(t, "__esModule", { value: !0 }), t.CoreMouseService = void 0;
        const n = l(585), c = l(460), a = l(844), h = { NONE: { events: 0, restrict: () => !1 }, X10: { events: 1, restrict: (s) => s.button !== 4 && s.action === 1 && (s.ctrl = !1, s.alt = !1, s.shift = !1, !0) }, VT200: { events: 19, restrict: (s) => s.action !== 32 }, DRAG: { events: 23, restrict: (s) => s.action !== 32 || s.button !== 3 }, ANY: { events: 31, restrict: (s) => !0 } };
        function u(s, f) {
          let p = (s.ctrl ? 16 : 0) | (s.shift ? 4 : 0) | (s.alt ? 8 : 0);
          return s.button === 4 ? (p |= 64, p |= s.action) : (p |= 3 & s.button, 4 & s.button && (p |= 64), 8 & s.button && (p |= 128), s.action === 32 ? p |= 32 : s.action !== 0 || f || (p |= 3)), p;
        }
        const e = String.fromCharCode, r = { DEFAULT: (s) => {
          const f = [u(s, !1) + 32, s.col + 32, s.row + 32];
          return f[0] > 255 || f[1] > 255 || f[2] > 255 ? "" : `\x1B[M${e(f[0])}${e(f[1])}${e(f[2])}`;
        }, SGR: (s) => {
          const f = s.action === 0 && s.button !== 4 ? "m" : "M";
          return `\x1B[<${u(s, !0)};${s.col};${s.row}${f}`;
        }, SGR_PIXELS: (s) => {
          const f = s.action === 0 && s.button !== 4 ? "m" : "M";
          return `\x1B[<${u(s, !0)};${s.x};${s.y}${f}`;
        } };
        let i = t.CoreMouseService = class extends a.Disposable {
          constructor(s, f) {
            super(), this._bufferService = s, this._coreService = f, this._protocols = {}, this._encodings = {}, this._activeProtocol = "", this._activeEncoding = "", this._lastEvent = null, this._onProtocolChange = this.register(new c.EventEmitter()), this.onProtocolChange = this._onProtocolChange.event;
            for (const p of Object.keys(h)) this.addProtocol(p, h[p]);
            for (const p of Object.keys(r)) this.addEncoding(p, r[p]);
            this.reset();
          }
          addProtocol(s, f) {
            this._protocols[s] = f;
          }
          addEncoding(s, f) {
            this._encodings[s] = f;
          }
          get activeProtocol() {
            return this._activeProtocol;
          }
          get areMouseEventsActive() {
            return this._protocols[this._activeProtocol].events !== 0;
          }
          set activeProtocol(s) {
            if (!this._protocols[s]) throw new Error(`unknown protocol "${s}"`);
            this._activeProtocol = s, this._onProtocolChange.fire(this._protocols[s].events);
          }
          get activeEncoding() {
            return this._activeEncoding;
          }
          set activeEncoding(s) {
            if (!this._encodings[s]) throw new Error(`unknown encoding "${s}"`);
            this._activeEncoding = s;
          }
          reset() {
            this.activeProtocol = "NONE", this.activeEncoding = "DEFAULT", this._lastEvent = null;
          }
          triggerMouseEvent(s) {
            if (s.col < 0 || s.col >= this._bufferService.cols || s.row < 0 || s.row >= this._bufferService.rows || s.button === 4 && s.action === 32 || s.button === 3 && s.action !== 32 || s.button !== 4 && (s.action === 2 || s.action === 3) || (s.col++, s.row++, s.action === 32 && this._lastEvent && this._equalEvents(this._lastEvent, s, this._activeEncoding === "SGR_PIXELS")) || !this._protocols[this._activeProtocol].restrict(s)) return !1;
            const f = this._encodings[this._activeEncoding](s);
            return f && (this._activeEncoding === "DEFAULT" ? this._coreService.triggerBinaryEvent(f) : this._coreService.triggerDataEvent(f, !0)), this._lastEvent = s, !0;
          }
          explainEvents(s) {
            return { down: !!(1 & s), up: !!(2 & s), drag: !!(4 & s), move: !!(8 & s), wheel: !!(16 & s) };
          }
          _equalEvents(s, f, p) {
            if (p) {
              if (s.x !== f.x || s.y !== f.y) return !1;
            } else if (s.col !== f.col || s.row !== f.row) return !1;
            return s.button === f.button && s.action === f.action && s.ctrl === f.ctrl && s.alt === f.alt && s.shift === f.shift;
          }
        };
        t.CoreMouseService = i = d([_(0, n.IBufferService), _(1, n.ICoreService)], i);
      }, 83: function(w, t, l) {
        var d = this && this.__decorate || function(i, s, f, p) {
          var S, y = arguments.length, C = y < 3 ? s : p === null ? p = Object.getOwnPropertyDescriptor(s, f) : p;
          if (typeof Reflect == "object" && typeof Reflect.decorate == "function") C = Reflect.decorate(i, s, f, p);
          else for (var T = i.length - 1; T >= 0; T--) (S = i[T]) && (C = (y < 3 ? S(C) : y > 3 ? S(s, f, C) : S(s, f)) || C);
          return y > 3 && C && Object.defineProperty(s, f, C), C;
        }, _ = this && this.__param || function(i, s) {
          return function(f, p) {
            s(f, p, i);
          };
        };
        Object.defineProperty(t, "__esModule", { value: !0 }), t.CoreService = void 0;
        const n = l(439), c = l(460), a = l(844), h = l(585), u = Object.freeze({ insertMode: !1 }), e = Object.freeze({ applicationCursorKeys: !1, applicationKeypad: !1, bracketedPasteMode: !1, origin: !1, reverseWraparound: !1, sendFocus: !1, wraparound: !0 });
        let r = t.CoreService = class extends a.Disposable {
          constructor(i, s, f) {
            super(), this._bufferService = i, this._logService = s, this._optionsService = f, this.isCursorInitialized = !1, this.isCursorHidden = !1, this._onData = this.register(new c.EventEmitter()), this.onData = this._onData.event, this._onUserInput = this.register(new c.EventEmitter()), this.onUserInput = this._onUserInput.event, this._onBinary = this.register(new c.EventEmitter()), this.onBinary = this._onBinary.event, this._onRequestScrollToBottom = this.register(new c.EventEmitter()), this.onRequestScrollToBottom = this._onRequestScrollToBottom.event, this.modes = (0, n.clone)(u), this.decPrivateModes = (0, n.clone)(e);
          }
          reset() {
            this.modes = (0, n.clone)(u), this.decPrivateModes = (0, n.clone)(e);
          }
          triggerDataEvent(i, s = !1) {
            if (this._optionsService.rawOptions.disableStdin) return;
            const f = this._bufferService.buffer;
            s && this._optionsService.rawOptions.scrollOnUserInput && f.ybase !== f.ydisp && this._onRequestScrollToBottom.fire(), s && this._onUserInput.fire(), this._logService.debug(`sending data "${i}"`, () => i.split("").map((p) => p.charCodeAt(0))), this._onData.fire(i);
          }
          triggerBinaryEvent(i) {
            this._optionsService.rawOptions.disableStdin || (this._logService.debug(`sending binary "${i}"`, () => i.split("").map((s) => s.charCodeAt(0))), this._onBinary.fire(i));
          }
        };
        t.CoreService = r = d([_(0, h.IBufferService), _(1, h.ILogService), _(2, h.IOptionsService)], r);
      }, 348: (w, t, l) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.InstantiationService = t.ServiceCollection = void 0;
        const d = l(585), _ = l(343);
        class n {
          constructor(...a) {
            this._entries = /* @__PURE__ */ new Map();
            for (const [h, u] of a) this.set(h, u);
          }
          set(a, h) {
            const u = this._entries.get(a);
            return this._entries.set(a, h), u;
          }
          forEach(a) {
            for (const [h, u] of this._entries.entries()) a(h, u);
          }
          has(a) {
            return this._entries.has(a);
          }
          get(a) {
            return this._entries.get(a);
          }
        }
        t.ServiceCollection = n, t.InstantiationService = class {
          constructor() {
            this._services = new n(), this._services.set(d.IInstantiationService, this);
          }
          setService(c, a) {
            this._services.set(c, a);
          }
          getService(c) {
            return this._services.get(c);
          }
          createInstance(c, ...a) {
            const h = (0, _.getServiceDependencies)(c).sort((r, i) => r.index - i.index), u = [];
            for (const r of h) {
              const i = this._services.get(r.id);
              if (!i) throw new Error(`[createInstance] ${c.name} depends on UNKNOWN service ${r.id}.`);
              u.push(i);
            }
            const e = h.length > 0 ? h[0].index : a.length;
            if (a.length !== e) throw new Error(`[createInstance] First service dependency of ${c.name} at position ${e + 1} conflicts with ${a.length} static arguments`);
            return new c(...a, ...u);
          }
        };
      }, 866: function(w, t, l) {
        var d = this && this.__decorate || function(e, r, i, s) {
          var f, p = arguments.length, S = p < 3 ? r : s === null ? s = Object.getOwnPropertyDescriptor(r, i) : s;
          if (typeof Reflect == "object" && typeof Reflect.decorate == "function") S = Reflect.decorate(e, r, i, s);
          else for (var y = e.length - 1; y >= 0; y--) (f = e[y]) && (S = (p < 3 ? f(S) : p > 3 ? f(r, i, S) : f(r, i)) || S);
          return p > 3 && S && Object.defineProperty(r, i, S), S;
        }, _ = this && this.__param || function(e, r) {
          return function(i, s) {
            r(i, s, e);
          };
        };
        Object.defineProperty(t, "__esModule", { value: !0 }), t.traceCall = t.setTraceLogger = t.LogService = void 0;
        const n = l(844), c = l(585), a = { trace: c.LogLevelEnum.TRACE, debug: c.LogLevelEnum.DEBUG, info: c.LogLevelEnum.INFO, warn: c.LogLevelEnum.WARN, error: c.LogLevelEnum.ERROR, off: c.LogLevelEnum.OFF };
        let h, u = t.LogService = class extends n.Disposable {
          get logLevel() {
            return this._logLevel;
          }
          constructor(e) {
            super(), this._optionsService = e, this._logLevel = c.LogLevelEnum.OFF, this._updateLogLevel(), this.register(this._optionsService.onSpecificOptionChange("logLevel", () => this._updateLogLevel())), h = this;
          }
          _updateLogLevel() {
            this._logLevel = a[this._optionsService.rawOptions.logLevel];
          }
          _evalLazyOptionalParams(e) {
            for (let r = 0; r < e.length; r++) typeof e[r] == "function" && (e[r] = e[r]());
          }
          _log(e, r, i) {
            this._evalLazyOptionalParams(i), e.call(console, (this._optionsService.options.logger ? "" : "xterm.js: ") + r, ...i);
          }
          trace(e, ...r) {
            this._logLevel <= c.LogLevelEnum.TRACE && this._log(this._optionsService.options.logger?.trace.bind(this._optionsService.options.logger) ?? console.log, e, r);
          }
          debug(e, ...r) {
            this._logLevel <= c.LogLevelEnum.DEBUG && this._log(this._optionsService.options.logger?.debug.bind(this._optionsService.options.logger) ?? console.log, e, r);
          }
          info(e, ...r) {
            this._logLevel <= c.LogLevelEnum.INFO && this._log(this._optionsService.options.logger?.info.bind(this._optionsService.options.logger) ?? console.info, e, r);
          }
          warn(e, ...r) {
            this._logLevel <= c.LogLevelEnum.WARN && this._log(this._optionsService.options.logger?.warn.bind(this._optionsService.options.logger) ?? console.warn, e, r);
          }
          error(e, ...r) {
            this._logLevel <= c.LogLevelEnum.ERROR && this._log(this._optionsService.options.logger?.error.bind(this._optionsService.options.logger) ?? console.error, e, r);
          }
        };
        t.LogService = u = d([_(0, c.IOptionsService)], u), t.setTraceLogger = function(e) {
          h = e;
        }, t.traceCall = function(e, r, i) {
          if (typeof i.value != "function") throw new Error("not supported");
          const s = i.value;
          i.value = function(...f) {
            if (h.logLevel !== c.LogLevelEnum.TRACE) return s.apply(this, f);
            h.trace(`GlyphRenderer#${s.name}(${f.map((S) => JSON.stringify(S)).join(", ")})`);
            const p = s.apply(this, f);
            return h.trace(`GlyphRenderer#${s.name} return`, p), p;
          };
        };
      }, 302: (w, t, l) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.OptionsService = t.DEFAULT_OPTIONS = void 0;
        const d = l(460), _ = l(844), n = l(114);
        t.DEFAULT_OPTIONS = { cols: 80, rows: 24, cursorBlink: !1, cursorStyle: "block", cursorWidth: 1, cursorInactiveStyle: "outline", customGlyphs: !0, drawBoldTextInBrightColors: !0, documentOverride: null, fastScrollModifier: "alt", fastScrollSensitivity: 5, fontFamily: "courier-new, courier, monospace", fontSize: 15, fontWeight: "normal", fontWeightBold: "bold", ignoreBracketedPasteMode: !1, lineHeight: 1, letterSpacing: 0, linkHandler: null, logLevel: "info", logger: null, scrollback: 1e3, scrollOnUserInput: !0, scrollSensitivity: 1, screenReaderMode: !1, smoothScrollDuration: 0, macOptionIsMeta: !1, macOptionClickForcesSelection: !1, minimumContrastRatio: 1, disableStdin: !1, allowProposedApi: !1, allowTransparency: !1, tabStopWidth: 8, theme: {}, rescaleOverlappingGlyphs: !1, rightClickSelectsWord: n.isMac, windowOptions: {}, windowsMode: !1, windowsPty: {}, wordSeparator: " ()[]{}',\"`", altClickMovesCursor: !0, convertEol: !1, termName: "xterm", cancelEvents: !1, overviewRulerWidth: 0 };
        const c = ["normal", "bold", "100", "200", "300", "400", "500", "600", "700", "800", "900"];
        class a extends _.Disposable {
          constructor(u) {
            super(), this._onOptionChange = this.register(new d.EventEmitter()), this.onOptionChange = this._onOptionChange.event;
            const e = { ...t.DEFAULT_OPTIONS };
            for (const r in u) if (r in e) try {
              const i = u[r];
              e[r] = this._sanitizeAndValidateOption(r, i);
            } catch (i) {
              console.error(i);
            }
            this.rawOptions = e, this.options = { ...e }, this._setupOptions(), this.register((0, _.toDisposable)(() => {
              this.rawOptions.linkHandler = null, this.rawOptions.documentOverride = null;
            }));
          }
          onSpecificOptionChange(u, e) {
            return this.onOptionChange((r) => {
              r === u && e(this.rawOptions[u]);
            });
          }
          onMultipleOptionChange(u, e) {
            return this.onOptionChange((r) => {
              u.indexOf(r) !== -1 && e();
            });
          }
          _setupOptions() {
            const u = (r) => {
              if (!(r in t.DEFAULT_OPTIONS)) throw new Error(`No option with key "${r}"`);
              return this.rawOptions[r];
            }, e = (r, i) => {
              if (!(r in t.DEFAULT_OPTIONS)) throw new Error(`No option with key "${r}"`);
              i = this._sanitizeAndValidateOption(r, i), this.rawOptions[r] !== i && (this.rawOptions[r] = i, this._onOptionChange.fire(r));
            };
            for (const r in this.rawOptions) {
              const i = { get: u.bind(this, r), set: e.bind(this, r) };
              Object.defineProperty(this.options, r, i);
            }
          }
          _sanitizeAndValidateOption(u, e) {
            switch (u) {
              case "cursorStyle":
                if (e || (e = t.DEFAULT_OPTIONS[u]), !/* @__PURE__ */ function(r) {
                  return r === "block" || r === "underline" || r === "bar";
                }(e)) throw new Error(`"${e}" is not a valid value for ${u}`);
                break;
              case "wordSeparator":
                e || (e = t.DEFAULT_OPTIONS[u]);
                break;
              case "fontWeight":
              case "fontWeightBold":
                if (typeof e == "number" && 1 <= e && e <= 1e3) break;
                e = c.includes(e) ? e : t.DEFAULT_OPTIONS[u];
                break;
              case "cursorWidth":
                e = Math.floor(e);
              case "lineHeight":
              case "tabStopWidth":
                if (e < 1) throw new Error(`${u} cannot be less than 1, value: ${e}`);
                break;
              case "minimumContrastRatio":
                e = Math.max(1, Math.min(21, Math.round(10 * e) / 10));
                break;
              case "scrollback":
                if ((e = Math.min(e, 4294967295)) < 0) throw new Error(`${u} cannot be less than 0, value: ${e}`);
                break;
              case "fastScrollSensitivity":
              case "scrollSensitivity":
                if (e <= 0) throw new Error(`${u} cannot be less than or equal to 0, value: ${e}`);
                break;
              case "rows":
              case "cols":
                if (!e && e !== 0) throw new Error(`${u} must be numeric, value: ${e}`);
                break;
              case "windowsPty":
                e = e ?? {};
            }
            return e;
          }
        }
        t.OptionsService = a;
      }, 660: function(w, t, l) {
        var d = this && this.__decorate || function(a, h, u, e) {
          var r, i = arguments.length, s = i < 3 ? h : e === null ? e = Object.getOwnPropertyDescriptor(h, u) : e;
          if (typeof Reflect == "object" && typeof Reflect.decorate == "function") s = Reflect.decorate(a, h, u, e);
          else for (var f = a.length - 1; f >= 0; f--) (r = a[f]) && (s = (i < 3 ? r(s) : i > 3 ? r(h, u, s) : r(h, u)) || s);
          return i > 3 && s && Object.defineProperty(h, u, s), s;
        }, _ = this && this.__param || function(a, h) {
          return function(u, e) {
            h(u, e, a);
          };
        };
        Object.defineProperty(t, "__esModule", { value: !0 }), t.OscLinkService = void 0;
        const n = l(585);
        let c = t.OscLinkService = class {
          constructor(a) {
            this._bufferService = a, this._nextId = 1, this._entriesWithId = /* @__PURE__ */ new Map(), this._dataByLinkId = /* @__PURE__ */ new Map();
          }
          registerLink(a) {
            const h = this._bufferService.buffer;
            if (a.id === void 0) {
              const f = h.addMarker(h.ybase + h.y), p = { data: a, id: this._nextId++, lines: [f] };
              return f.onDispose(() => this._removeMarkerFromLink(p, f)), this._dataByLinkId.set(p.id, p), p.id;
            }
            const u = a, e = this._getEntryIdKey(u), r = this._entriesWithId.get(e);
            if (r) return this.addLineToLink(r.id, h.ybase + h.y), r.id;
            const i = h.addMarker(h.ybase + h.y), s = { id: this._nextId++, key: this._getEntryIdKey(u), data: u, lines: [i] };
            return i.onDispose(() => this._removeMarkerFromLink(s, i)), this._entriesWithId.set(s.key, s), this._dataByLinkId.set(s.id, s), s.id;
          }
          addLineToLink(a, h) {
            const u = this._dataByLinkId.get(a);
            if (u && u.lines.every((e) => e.line !== h)) {
              const e = this._bufferService.buffer.addMarker(h);
              u.lines.push(e), e.onDispose(() => this._removeMarkerFromLink(u, e));
            }
          }
          getLinkData(a) {
            return this._dataByLinkId.get(a)?.data;
          }
          _getEntryIdKey(a) {
            return `${a.id};;${a.uri}`;
          }
          _removeMarkerFromLink(a, h) {
            const u = a.lines.indexOf(h);
            u !== -1 && (a.lines.splice(u, 1), a.lines.length === 0 && (a.data.id !== void 0 && this._entriesWithId.delete(a.key), this._dataByLinkId.delete(a.id)));
          }
        };
        t.OscLinkService = c = d([_(0, n.IBufferService)], c);
      }, 343: (w, t) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.createDecorator = t.getServiceDependencies = t.serviceRegistry = void 0;
        const l = "di$target", d = "di$dependencies";
        t.serviceRegistry = /* @__PURE__ */ new Map(), t.getServiceDependencies = function(_) {
          return _[d] || [];
        }, t.createDecorator = function(_) {
          if (t.serviceRegistry.has(_)) return t.serviceRegistry.get(_);
          const n = function(c, a, h) {
            if (arguments.length !== 3) throw new Error("@IServiceName-decorator can only be used to decorate a parameter");
            (function(u, e, r) {
              e[l] === e ? e[d].push({ id: u, index: r }) : (e[d] = [{ id: u, index: r }], e[l] = e);
            })(n, c, h);
          };
          return n.toString = () => _, t.serviceRegistry.set(_, n), n;
        };
      }, 585: (w, t, l) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.IDecorationService = t.IUnicodeService = t.IOscLinkService = t.IOptionsService = t.ILogService = t.LogLevelEnum = t.IInstantiationService = t.ICharsetService = t.ICoreService = t.ICoreMouseService = t.IBufferService = void 0;
        const d = l(343);
        var _;
        t.IBufferService = (0, d.createDecorator)("BufferService"), t.ICoreMouseService = (0, d.createDecorator)("CoreMouseService"), t.ICoreService = (0, d.createDecorator)("CoreService"), t.ICharsetService = (0, d.createDecorator)("CharsetService"), t.IInstantiationService = (0, d.createDecorator)("InstantiationService"), function(n) {
          n[n.TRACE = 0] = "TRACE", n[n.DEBUG = 1] = "DEBUG", n[n.INFO = 2] = "INFO", n[n.WARN = 3] = "WARN", n[n.ERROR = 4] = "ERROR", n[n.OFF = 5] = "OFF";
        }(_ || (t.LogLevelEnum = _ = {})), t.ILogService = (0, d.createDecorator)("LogService"), t.IOptionsService = (0, d.createDecorator)("OptionsService"), t.IOscLinkService = (0, d.createDecorator)("OscLinkService"), t.IUnicodeService = (0, d.createDecorator)("UnicodeService"), t.IDecorationService = (0, d.createDecorator)("DecorationService");
      }, 480: (w, t, l) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.UnicodeService = void 0;
        const d = l(460), _ = l(225);
        class n {
          static extractShouldJoin(a) {
            return (1 & a) != 0;
          }
          static extractWidth(a) {
            return a >> 1 & 3;
          }
          static extractCharKind(a) {
            return a >> 3;
          }
          static createPropertyValue(a, h, u = !1) {
            return (16777215 & a) << 3 | (3 & h) << 1 | (u ? 1 : 0);
          }
          constructor() {
            this._providers = /* @__PURE__ */ Object.create(null), this._active = "", this._onChange = new d.EventEmitter(), this.onChange = this._onChange.event;
            const a = new _.UnicodeV6();
            this.register(a), this._active = a.version, this._activeProvider = a;
          }
          dispose() {
            this._onChange.dispose();
          }
          get versions() {
            return Object.keys(this._providers);
          }
          get activeVersion() {
            return this._active;
          }
          set activeVersion(a) {
            if (!this._providers[a]) throw new Error(`unknown Unicode version "${a}"`);
            this._active = a, this._activeProvider = this._providers[a], this._onChange.fire(a);
          }
          register(a) {
            this._providers[a.version] = a;
          }
          wcwidth(a) {
            return this._activeProvider.wcwidth(a);
          }
          getStringCellWidth(a) {
            let h = 0, u = 0;
            const e = a.length;
            for (let r = 0; r < e; ++r) {
              let i = a.charCodeAt(r);
              if (55296 <= i && i <= 56319) {
                if (++r >= e) return h + this.wcwidth(i);
                const p = a.charCodeAt(r);
                56320 <= p && p <= 57343 ? i = 1024 * (i - 55296) + p - 56320 + 65536 : h += this.wcwidth(p);
              }
              const s = this.charProperties(i, u);
              let f = n.extractWidth(s);
              n.extractShouldJoin(s) && (f -= n.extractWidth(u)), h += f, u = s;
            }
            return h;
          }
          charProperties(a, h) {
            return this._activeProvider.charProperties(a, h);
          }
        }
        t.UnicodeService = n;
      }, 781: (w, t, l) => {
        Object.defineProperty(t, "__esModule", { value: !0 }), t.Terminal = void 0;
        const d = l(437), _ = l(969), n = l(460);
        class c extends _.CoreTerminal {
          constructor(h = {}) {
            super(h), this._onBell = this.register(new n.EventEmitter()), this.onBell = this._onBell.event, this._onCursorMove = this.register(new n.EventEmitter()), this.onCursorMove = this._onCursorMove.event, this._onTitleChange = this.register(new n.EventEmitter()), this.onTitleChange = this._onTitleChange.event, this._onA11yCharEmitter = this.register(new n.EventEmitter()), this.onA11yChar = this._onA11yCharEmitter.event, this._onA11yTabEmitter = this.register(new n.EventEmitter()), this.onA11yTab = this._onA11yTabEmitter.event, this._setup(), this.register(this._inputHandler.onRequestBell(() => this.bell())), this.register(this._inputHandler.onRequestReset(() => this.reset())), this.register((0, n.forwardEvent)(this._inputHandler.onCursorMove, this._onCursorMove)), this.register((0, n.forwardEvent)(this._inputHandler.onTitleChange, this._onTitleChange)), this.register((0, n.forwardEvent)(this._inputHandler.onA11yChar, this._onA11yCharEmitter)), this.register((0, n.forwardEvent)(this._inputHandler.onA11yTab, this._onA11yTabEmitter));
          }
          get buffer() {
            return this.buffers.active;
          }
          get markers() {
            return this.buffer.markers;
          }
          addMarker(h) {
            if (this.buffer === this.buffers.normal) return this.buffer.addMarker(this.buffer.ybase + this.buffer.y + h);
          }
          bell() {
            this._onBell.fire();
          }
          input(h, u = !0) {
            this.coreService.triggerDataEvent(h, u);
          }
          resize(h, u) {
            h === this.cols && u === this.rows || super.resize(h, u);
          }
          clear() {
            if (this.buffer.ybase !== 0 || this.buffer.y !== 0) {
              this.buffer.lines.set(0, this.buffer.lines.get(this.buffer.ybase + this.buffer.y)), this.buffer.lines.length = 1, this.buffer.ydisp = 0, this.buffer.ybase = 0, this.buffer.y = 0;
              for (let h = 1; h < this.rows; h++) this.buffer.lines.push(this.buffer.getBlankLine(d.DEFAULT_ATTR_DATA));
              this._onScroll.fire({ position: this.buffer.ydisp, source: 0 });
            }
          }
          reset() {
            this.options.rows = this.rows, this.options.cols = this.cols, this._setup(), super.reset();
          }
        }
        t.Terminal = c;
      } }, I = {};
      function k(w) {
        var t = I[w];
        if (t !== void 0) return t.exports;
        var l = I[w] = { exports: {} };
        return O[w].call(l.exports, l, l.exports, k), l.exports;
      }
      var R = {};
      (() => {
        var w = R;
        Object.defineProperty(w, "__esModule", { value: !0 }), w.Terminal = void 0;
        const t = k(285), l = k(975), d = k(90), _ = k(781), n = k(741), c = k(844), a = ["cols", "rows"];
        class h extends c.Disposable {
          constructor(e) {
            super(), this._core = this.register(new _.Terminal(e)), this._addonManager = this.register(new n.AddonManager()), this._publicOptions = { ...this._core.options };
            const r = (s) => this._core.options[s], i = (s, f) => {
              this._checkReadonlyOptions(s), this._core.options[s] = f;
            };
            for (const s in this._core.options) {
              Object.defineProperty(this._publicOptions, s, { get: () => this._core.options[s], set: (p) => {
                this._checkReadonlyOptions(s), this._core.options[s] = p;
              } });
              const f = { get: r.bind(this, s), set: i.bind(this, s) };
              Object.defineProperty(this._publicOptions, s, f);
            }
          }
          _checkReadonlyOptions(e) {
            if (a.includes(e)) throw new Error(`Option "${e}" can only be set in the constructor`);
          }
          _checkProposedApi() {
            if (!this._core.optionsService.options.allowProposedApi) throw new Error("You must set the allowProposedApi option to true to use proposed API");
          }
          get onBell() {
            return this._core.onBell;
          }
          get onBinary() {
            return this._core.onBinary;
          }
          get onCursorMove() {
            return this._core.onCursorMove;
          }
          get onData() {
            return this._core.onData;
          }
          get onLineFeed() {
            return this._core.onLineFeed;
          }
          get onResize() {
            return this._core.onResize;
          }
          get onScroll() {
            return this._core.onScroll;
          }
          get onTitleChange() {
            return this._core.onTitleChange;
          }
          get parser() {
            return this._checkProposedApi(), this._parser || (this._parser = new l.ParserApi(this._core)), this._parser;
          }
          get unicode() {
            return this._checkProposedApi(), new d.UnicodeApi(this._core);
          }
          get rows() {
            return this._core.rows;
          }
          get cols() {
            return this._core.cols;
          }
          get buffer() {
            return this._checkProposedApi(), this._buffer || (this._buffer = this.register(new t.BufferNamespaceApi(this._core))), this._buffer;
          }
          get markers() {
            return this._checkProposedApi(), this._core.markers;
          }
          get modes() {
            const e = this._core.coreService.decPrivateModes;
            let r = "none";
            switch (this._core.coreMouseService.activeProtocol) {
              case "X10":
                r = "x10";
                break;
              case "VT200":
                r = "vt200";
                break;
              case "DRAG":
                r = "drag";
                break;
              case "ANY":
                r = "any";
            }
            return { applicationCursorKeysMode: e.applicationCursorKeys, applicationKeypadMode: e.applicationKeypad, bracketedPasteMode: e.bracketedPasteMode, insertMode: this._core.coreService.modes.insertMode, mouseTrackingMode: r, originMode: e.origin, reverseWraparoundMode: e.reverseWraparound, sendFocusMode: e.sendFocus, wraparoundMode: e.wraparound };
          }
          get options() {
            return this._publicOptions;
          }
          set options(e) {
            for (const r in e) this._publicOptions[r] = e[r];
          }
          input(e, r = !0) {
            this._core.input(e, r);
          }
          resize(e, r) {
            this._verifyIntegers(e, r), this._core.resize(e, r);
          }
          registerMarker(e = 0) {
            return this._checkProposedApi(), this._verifyIntegers(e), this._core.addMarker(e);
          }
          addMarker(e) {
            return this.registerMarker(e);
          }
          dispose() {
            super.dispose();
          }
          scrollLines(e) {
            this._verifyIntegers(e), this._core.scrollLines(e);
          }
          scrollPages(e) {
            this._verifyIntegers(e), this._core.scrollPages(e);
          }
          scrollToTop() {
            this._core.scrollToTop();
          }
          scrollToBottom() {
            this._core.scrollToBottom();
          }
          scrollToLine(e) {
            this._verifyIntegers(e), this._core.scrollToLine(e);
          }
          clear() {
            this._core.clear();
          }
          write(e, r) {
            this._core.write(e, r);
          }
          writeln(e, r) {
            this._core.write(e), this._core.write(`\r
`, r);
          }
          reset() {
            this._core.reset();
          }
          loadAddon(e) {
            this._addonManager.loadAddon(this, e);
          }
          _verifyIntegers(...e) {
            for (const r of e) if (r === 1 / 0 || isNaN(r) || r % 1 != 0) throw new Error("This API only accepts integers");
          }
        }
        w.Terminal = h;
      })();
      var q = L;
      for (var N in R) q[N] = R[N];
      R.__esModule && Object.defineProperty(q, "__esModule", { value: !0 });
    })();
  }(ie)), ie;
}
var De = Le();
function xe() {
  return new De.Terminal({
    cols: 1e3,
    rows: 500,
    allowProposedApi: !0,
    convertEol: !0,
    scrollback: 1e4,
    cursorBlink: !1
  });
}
const Pe = {
  maxCharacterLength: ge
};
function He(L, O = Pe) {
  const I = L.buffer.active, k = [], R = Math.max(0, O.maxCharacterLength);
  for (let N = 0; N < I.length; N++) {
    const w = I.getLine(N);
    if (!w)
      continue;
    const t = w.translateToString(!1).trimEnd();
    t.trim().length === 0 && k.at(-1)?.trim().length === 0 || k.push(t);
  }
  let q = 0;
  for (let N = k.length - 1; N > 0; N--) {
    const w = k[N];
    if (q += w.length, q > R) {
      const l = N, d = `Truncated ${l} ${ve(l, "line")} above this point`, _ = k.length - N - 1, n = _ === 1 ? "Below is the last line of the output" : `Below are the last ${_} lines of the output`;
      return {
        output: [
          `--- ${d}. ${n}. ---`,
          ...k.slice(N)
        ].join(`
`),
        hidden_truncatedOutput: k.slice(0, N).join(`
`)
      };
    }
  }
  return {
    output: k.join(`
`)
  };
}
function Re(L, O, I) {
  const k = xe();
  return Ce(
    L,
    O.settings["terminal.commands.nodeSpawn.loadProfile"]
  ).pipe(
    oe(
      (R) => me(I.cmd, [], {
        shell: le.userInfo().shell || !0,
        cwd: L,
        stdio: ["ignore", "pipe", "pipe"],
        env: R
      })
    ),
    oe(({ lastData: R, exitCode: q, exited: N }) => be(
      new Promise((w) => {
        const t = R?.subarray() ?? new Uint8Array();
        k.write(t, () => {
          const { output: l } = He(k);
          w(N ? {
            status: "done",
            result: { output: l, exitCode: q ?? -1 }
          } : {
            status: "in-progress",
            progress: { output: l }
          });
        });
      })
    ))
  );
}
const Ne = ({ args: L, userInput: O }, { dir: I, dirs: k, config: R, toolMessages: q }) => {
  Se(I);
  const N = fe(I.fsPath, L.cwd ? Me(L.cwd) : "."), w = (c) => ({
    status: "done",
    result: {
      output: `Error: ${c}`,
      exitCode: 1
    }
  }), t = Oe(L.cmd);
  if (t)
    return se.of(w(t));
  const l = Ie(N);
  if (l)
    return se.of(w(l));
  const d = [N, ...k.map((c) => c.fsPath)], _ = ke(L.cmd, d, {
    config: R.settings
  }), n = _.parts.flatMap(
    (c) => (
      // Only offer adding program to allowlist when parsable, non-dynamic, non-concatenated, and within cwd.
      c.requiresConfirmation ? typeof c.command == "string" || c.command.program === ne || Array.isArray(c.command.program) || c.command.arguments.some(
        (a) => a === ne || typeof a == "string" && ue(a, d)
      ) ? [] : [c.command.program] : []
    )
  );
  if (_.requiresConfirmation) {
    if (O && !O.accepted)
      return se.of({ status: "rejected-by-user", toAllow: n });
    if (!O?.accepted)
      return se.of({ status: "blocked-on-user", toAllow: n });
  }
  return R.settings["terminal.commands.environment"], Re(N, R, L);
}, ce = le.homedir();
function Me(L) {
  return ce ? L.replace(/^~(?=$|\/|\\)/, ce) : L;
}
function Oe(L) {
  if (typeof L != "string")
    return "`cmd` must be a string representing a command.";
}
function Ie(L) {
  if (L !== void 0 && typeof L != "string")
    return "`cwd` must be a string representing a relative path within the workspace.";
}
export {
  Ne as nodeBashTool
};
